#include <mem.h>
#include <mosaic.h>

semaphore readylist;

void seminit(semaphore *s)
{
  memset(s,0,sizeof(semaphore));
}

void _signal(semaphore *s)
{
  disable();
  if (!s->head) s->count++;
  else
  {
    if (!readylist.head) readylist.head = s->head;
    else *readylist.tail = s->head;
    readylist.tail = s->head;
    s->head = *s->head;
    *readylist.tail = NULL;
  }
  enable();
}

void _start(void (*task)(void), char *stack, int size)
{
  void **p;

  // init stack frame as though call to wait(), with link on TOS
  // i.e. LINK DI SI BP RET
  p = (void **)(stack+size-(2+2+2+2+2));        // point p to link
  *p = NULL;                                    // we link to 0
  if (!readylist.head) readylist.head = p;      // if first time, we are head
  else *readylist.tail = p;                     // else link old tail to us
  readylist.tail = p;                           // anyway, we are now tail
  * (void **)(stack+size-2) = task;             // and set return address
}

#pragma argsused
void _wait(semaphore *s)
{
  // hopefully, compiler inserts:
  // push bp
  // mov bp, sp
  // push si
  // push di
  asm mov di,[bp+4]                     // di == s
  asm mov bx,0                          // now push a zero
  asm push bx
  asm mov bx,sp                         // point bx to it
  asm cmp word ptr [di],0               // if no head link
  asm jne append
  asm mov word ptr [di],bx              // we are it
  asm jmp next
append:
  asm mov si,word ptr [di+2]            // else link old tail to us
  asm mov word ptr [si],bx
next:
  asm mov word ptr [di+2],bx            // anyway, make us new tail

await:
  asm cmp word ptr [readylist.head],0   // now wait for ready task
  asm je await

  asm mov bx,word ptr [readylist.head]  // set stack to ready task
  asm mov sp,bx
  asm pop si                            // pop linked address into si
  asm mov word ptr [readylist.head],si  // and update readylist
  // hopefully, compiler inserts:
  // pop di
  // pop si
  // pop bp
  // ret
}
