#include <mem.h>
#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <time.h>

typedef struct {
  void **head, **tail;
  int count;
} semaphore;

static semaphore readylist;
semaphore keyhit;
semaphore second;
semaphore tock;

void seminit(semaphore *s)
{
  memset(s,0,sizeof(semaphore));
}

void _signal(semaphore *s)
{
  disable();
  if (!s->head) s->count++;
  else
  {
    if (!readylist.head) readylist.head = s->head;
    else *readylist.tail = s->head;
    readylist.tail = s->head;
    s->head = *s->head;
    *readylist.tail = NULL;
  }
  enable();
}

void _start(void (*task)(void), char *stack, int size)
{
  void **p;

  // init stack frame as though call to wait(), with link on TOS
  // i.e. LINK DI SI BP RET
  p = (void **)(stack+size-(2+2+2+2+2));        // point p to link
  *p = NULL;                                    // we link to 0
  if (!readylist.head) readylist.head = p;      // if first time, we are head
  else *readylist.tail = p;                     // else link old tail to us
  readylist.tail = p;                           // anyway, we are now tail
  * (void **)(stack+size-2) = task;             // and set return address
}

#define task(t) _start(t,t##_stack,sizeof(t##_stack))

#pragma argsused
void _wait(semaphore *s)
{
  // hopefully, compiler inserts:
  // push bp
  // mov bp, sp
  // push si
  // push di
  asm mov di,[bp+4]                     // di == s
  asm mov bx,0                          // now push a zero
  asm push bx
  asm mov bx,sp                         // point bx to it
  asm cmp word ptr [di],0               // if no head link
  asm jne append
  asm mov word ptr [di],bx              // we are it
  asm jmp next
append:
  asm mov si,word ptr [di+2]            // else link old tail to us
  asm mov word ptr [si],bx
next:
  asm mov word ptr [di+2],bx            // anyway, make us new tail

await:
  asm cmp word ptr [readylist.head],0   // now wait for ready task
  asm je await

  asm mov bx,word ptr [readylist.head]  // set stack to ready task
  asm mov sp,bx
  asm pop si                            // pop linked address into si
  asm mov word ptr [readylist.head],si  // and update readylist
  // hopefully, compiler inserts:
  // pop di
  // pop si
  // pop bp
  // ret
}

#define pause() {if (readylist.head) _wait(&readylist);}
#define wait(s) {if (!s.count) _wait(&s); else { s.count--; _wait(&readylist); }}
#define signal(s) _signal(&s)


// application code
void tick(void)
{
  clock_t ticks;

  for(;;)
  {
    ticks = clock();
    while (ticks == clock()) pause();
    while (tock.head) signal(tock);
  }
}

void snooze(int n)
{
  while (n--) wait(tock);
}

void out(char c)
{
  putch(c);
  snooze(11);
}

void task1(void)
{
  for(;;)
  {
    printf("Hello %s\n","there!");
    snooze(5);
  }
}

void task2(void)
{
  for(;;)
  {
    out('Y');
    out('U');
  }
}

char key;
void keywait(void)
{
  for(;;)
  {
    while (!kbhit()) pause();
    key = getch();
    signal(keyhit);
  }
}

void main(void)
{
  char task1_stack[1000];
  char task2_stack[100];
  char keywait_stack[100];
  char tick_stack[100];

  int x;

  memset(task1_stack,0x5a,sizeof(task1_stack));

  seminit(&readylist);
  seminit(&keyhit);
  seminit(&second);
  seminit(&tock);

  task(task1);
  task(task2);
  task(keywait);
  task(tick);

  // main task runs here!
  for(;;)
  {
    for(x = 0; x < 10; x++)
    {
      printf("%d",x);
      wait(tock);
    }
    printf("\n%d keys pressed!\n",keyhit.count);
    wait(keyhit);
    if (key == 27) return;
    key = 0;
  }
}
