// LSP cryptosystem
#include "cipher.h"

// init random number generator with given string
// assumes string contains only legal base-104 characters
void rnginit(char *k, RNG *r)
{
  int x, y;
  char *s=k;

  for (x=0; x<RNGSIZE; x++) r->t[x]=x;

  r->a=r->b=0;

  do
  {
    x=r->t[r->a=(r->a+1)%RNGSIZE];
    y=r->t[r->b=(r->b+x+*s)%RNGSIZE];
    r->t[r->a]=y;
    r->t[r->b]=x;
    if (*k && !*++s) s=k;
  } while (r->a);
}

// return next random value from specified RNG
char rng(RNG *r)
{
  int x, y;

  x=r->t[r->a=(r->a+1)%RNGSIZE];            // a++
  y=r->t[r->b=(r->b+x)%RNGSIZE];            // b+=t[a]
  r->t[r->b]=x;                             // swap t[a] and t[b]
  r->t[r->a]=y;
  return(r->t[(x+y)%RNGSIZE]);              // random is t[t[a]+t[b]]}
}

// convert LSP legal char to value 0 thru 101, -1 == invalid
// legal: SOH=1, EOT=4, BELL=7, BS=8, LF=10, ESC=27, ASCII=32-127
static const unsigned char unalias[128]=
{255,  0,255,255,  1,255,255,  2,  3,  4,  5,255,255,  6,255,255,
 255,255,255,255,255,255,255,255,255,255,255,  7,255,255,255,255,
   8,  9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
  24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
  40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
  56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
  72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87,
  88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103 };

// convert 0 thru 103 to LSP legal
static const char alias[RNGSIZE] =
{  1,  4,  7,  8,  9, 10, 13, 27, 32, 33, 34, 35, 36, 37, 38, 39,
  40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
  56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
  72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87,
  88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,
 104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,
 120,121,122,123,124,125,126,127 };



char encipher(int c, RNG *r)
{
  if ((c>127)||((c=unalias[c])>127)) return(0);
  return(alias[(c+rng(r))%RNGSIZE]);
}

char decipher(int c, RNG *r)
{
  if ((c>127)||((c=unalias[c])>127)) return(0);
  return(alias[(c+(RNGSIZE-rng(r)))%RNGSIZE]);
}

