#!/use/bin/perl

use warnings ;
use strict ;
use XML::Parser;
use Time::Local qw(timegm);
use CGI;
use DBI;

sub cwarn($) { -t STDOUT and warn @_; }
#sub cwarn($) { warn @_; }

# connect to database
my $DB;
for (reverse 0..10)
{
  $DB = DBI->connect('DBI:mysql:news:db','news',undef,{RaiseError => 1, AutoCommit=>1}) and last;
  $_ or die "can't connect to database\n";
  sleep 2; 
}

# clean out really old news
$DB->do('delete from news where date < unix_timestamp(now()-interval 20 day)');

my $sendid;
my @group;
my $headline;
my $text;
my $date; 

my $state='';

sub start(@)
{
  my ($p, $e, %attr)=@_;

  my %state = (
    'cancel' => sub
    {
      $sendid=$attr{'itemname'};
      $state and warn("$sendid: ignoring cancel in $state state\n"), return;
      $DB->do("delete from news where sendid=?",undef,$sendid);
      warn "$sendid: deleted\n";
      return;
    }, 

    'article' => sub 
    {  
      $sendid=$attr{'itemname'} or warn("invalid, no itemname\n"), return;
      $sendid =~ s/^(?:.*\/)?(.*).html$/$1/; # strip the path and .html 
      cwarn "$sendid: processing article\n";
      $state and warn "$sendid: restarting from $state state\n";
      #$DB->selectrow_array("select 1 from news where sendid=? limit 1",undef,$sendid) and warn("$sendid: skipping\n"), return; 
      $state = 'article';
      $text=''; 
      @group=(); 
      $headline='';
      return;
    },

    'newsgroup'=> sub { push @group, $attr{name} },
    'date.issue'=> sub { $date = $attr{value} },
    'headline' => sub { $state eq 'article' and $state = 'headline' },
    'text' => sub { $state eq 'article' and $state = 'text' },
  );
  
  exists $state{$e} and $state{$e}->();
}

sub end(@)
{
  my ($p, $e)=@_;

  my %state = (
    'article' => sub 
    {
      my $st = $state or return; $state = '';
      $st eq 'article' or warn("$sendid: ignoring end in $st state\n"), return;

      $text ne '' or warn("$sendid: no text\n"), return;

      $headline ne '' or warn("$sendid: no headline\n"), return;

      (my $group = (grep /\.top/, @group)[0]) =~ s/\..*top//;
      $group or warn("$sendid: no 'top' group\n"), return;

      # make it human readable (or just map it to another group)
      my %xlat=('biz'=>'Business',
                'tw'=>'Technology', 
                'tech'=>'Technology',
                'usa'=>'USA',
                'news'=>'Living');
      if (exists $xlat{$group}) { $group = $xlat{$group} } else { $group =~ s/(^|\W)./\U$&/g } # uppercase words  

      # <date.issue value="20031001T071755-0700" />	
      $date or warn("$sendid: no date.issue\n"), return;
      my ($Y,$M,$D,$h,$m,$s,$oo,$oh,$om)=($date =~ /(\d{4})(\d{2})(\d{2})T(\d{2})(\d{2})(\d{2})([-+])(\d{2})(\d{2})/) or warn("$sendid: invalid date.issue\n"), return; 
      $date = timegm($s,$m,$h,$D,$M-1,$Y) or warn("$sendid: malformed date.issue\n");   
      my $o = ($oh*3600)+($om*60); $oo eq '+' and $o *= -1; $date += $o; # + means 'earlier than GMT

      if ($DB->selectrow_array('select 1 from news where sendid=?',undef,$sendid))
      {
        cwarn("$sendid: updated\n");
        $DB->do('update news set date=?, grp=?, headline=?, text=? where sendid=?',undef,$date,$group,$headline,$text,$sendid);
      } else
      {
        cwarn("$sendid: inserted\n");
        $DB->do('insert into news set sendid=?, date=?, grp=?, headline=?, text=?',undef,$sendid,$date,$group,$headline,$text);
      }
      return;
    },
  
    'p' => sub
    {
      $state eq 'text' and $text .= "<p>";
    },

    'headline' => sub
    {
      $state eq 'headline' and $state = 'article'; 
    },

    'text' => sub
    {
      $state eq 'text' and $state = 'article';
    },
  );

  exists $state{$e} and $state{$e}->();
}

sub char(@)
{
  my ($p, $s)=@_;

  $state eq 'headline' and $headline .= $s;          # don't escape the headline so we know how long it is
  $state eq 'text' and $text .= CGI::escapeHTML($s); # do escape the story so we can embed markup
}  

new XML::Parser(Handlers =>{Start=>\&start, End=>\&end, Char=>\&char})->parse(*STDIN);

__END__

# now create html pages

my $dir="/var/www/www.mailbug.net"; # where do the files go?
my $news="$dir/news.".time;         # make the directory
mkdir $news or die "mkdir failed: $!";

# make main index
open F, ">$news/index.html" or die "open failed: $!";
print F "<html><head><title>Headline News - Select a Category</title><body>";
my $g = $DB->selectall_arrayref('select distinct grp from news order by grp asc');
$g or die "database is empty";

for (@$g)
{
  ($_->[1] = $_->[0]) =~ tr/ /_/;
  print F "<form action='$_->[0].index.html'>&lt;--<input type=submit value='$_->[1]' bind=left></form>";
}    
print F "</body></html>";
close F;

# make index for each group
for (@$g)
{
  open F,">$news/$_->[1].index.html" or die "open failed: $!"; 
  print F "<html><head><title>$_->[0] - Select a Headline</title><body>";
  for my $item (@{$DB->selectall_arrayref('select sendid, headline, text from news where grp=? order by date desc limit 10',undef,$_->[0])})
  {
    # add the link 
    print F "<form action='$item->[0].html'>&lt;--<input type=submit value='",CGI::escapeHTML(substr($item->[1],0,75)),"' bind=left></form>";

    # make the actual article
    open T,">$news/$item->[0].html" or die "open failed: $!";
    print T "<html><head><title>$_->[0]</title><body>",CGI::escapeHTML($item->[1]),"<p>$item->[2]</body></html>";
    close T;
  }
  print F "</body></html>";
  close F;
}

# make news stories
#for (@{$DB->selectall_arrayref('select sendid, headline, text, grp from news')})
#{
#  open F,">$news/$_->[0].html" or die "open failed: $!";
#  print F "<html><head><title>$_->[3]</title><body>",CGI::escapeHTML($_->[1]),"<p>$_->[2]</body></html>";
#  close F;
#}

# ok, now we want to symlink 'news' to our new directory
# first get the current one
my $current=readlink "$dir/news";
$current or cwarn "No current news\n";
# unlink it
$current and (unlink "$dir/news" or die "unlink $dir/news failed: $!\n");
# now link to new
symlink $news, "$dir/news" or die "link $news failed: $!\n";
# nuke the old
$current and (opendir(D,$current), unlink(map "$current/$_", readdir D), closedir(D), rmdir $current or die "rmdir $current failed: $!\n");

