#!/usr/bin/perl
# databug session log 
use warnings;
use strict;

use lib '/crux/dbug/pm';
use dbCGI::init;
use dbCGI::gui;

use CGI qw(param escapeHTML self_url -oldstyle_urls);
use GD::Graph::mixed;
use POSIX;
use Time::Local qw{timelocal_nocheck timegm timelocal};
use URI::Escape;

# 'die' will show error to user instead of barfing apache  
# but not in the 'create graph' incarnation
param('graph') or local $SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

$PERMISSION > 1 or die "Forbidden\n";

# legal values for 'width'
#              label       seconds	    seconds     periods    ticks     x-axis
#              				    per period  per tick   per mark  format
my @widths = (['1 hour',     1*3600,          20,        15,        3,       "%H:%M"],      # 5 min, 15 mins
              ['2 hours',    2*3600,          40,        15,        6,       "%H:%M"],      # 10 min, hour
              ['4 hours',    4*3600,          75,        12,        4,       "%H:%M"],      # 15 min, hour
              ['8 hours',    8*3600,         150,        12,        2,       "%H:%M"],      # 30 min, hour
              ['16 hours',  16*3600,         300,        12,        1,       "%H"],         # hour, hour
              ['1 day',     24*3600,         450,         8,       24,       "%H"],         # hour, day 
              ['2 days',    2*86400,         900,         4,       24,       "%H"],         # hour, day 
              ['7 days',    7*86400,        3600,        24,        1,       "%a %d"],	    # day, day 
              ['14 days',  16*86400,        7200,        12,        1,       "%b %d"],	    # day, day	
              ['28 days',  28*86400,       14400,        42,        1,       "%b %d"],      # week, week
              ['56 days',  56*86400,       28800,        21,        1,       "%b %d"],      # week, week
              ['84 days',  84*86400,       43200,        14,        1,       "%b %d"]);     # week, week
my $defwidth=5;

# get params
my $width = param('width')||$widths[$defwidth][0]; 
 
my $before = param 'before';
my ($bminute, $bhour, $bday, $nmonth,$nyear) = (localtime)[1..5]; $nmonth++;  
defined param 'bminute' and $bminute = param 'bminute'; 
defined param 'bhour' and $bhour = param 'bhour'; 
defined param 'bday' and $bday = param 'bday'; 
my $bmonth = param('bmonth')||$nmonth;
$bmonth > $nmonth and $nyear--; 

#die "$nmonth $nyear $bmonth $bday $bhour $bminute\n";

my $deviceid = param('deviceid')||'';
my $cpeid = (param 'cpeid') || '';  
my $scriptid = (param 'scriptid') || ''; 
my $phone = (param 'phone') || '';

if (!param 'graph')
{
  #my $refresh;
  #$width eq $_->[0] and $refresh=$_->[2], last for @widths;
  #$refresh or $refresh=$widths[0]->[2];
  #$refresh < 60 and $refresh = 60;
  #$HEAD .= "<meta http-equiv='refresh' content='$refresh'>"; 
  
  # here, generate html page 


  my $html .= "<table cellpadding=0 cellspacing=0><tr><td>
                <form method=get>
                  <table border=1 cellpadding=3 cellspacing=0>";

  # width
  $html .= "<tr><td>Show<td><select name=width>"; $html .= opt($_, $_ eq $width) for map $_->[0], @widths; $html .= "</select>";

  # time
  $html .= sprintf '<tr><td><input type=checkbox name=before%s>Before<td>',$before?" checked":"";
  $html .= "<select name=bmonth>"; $html .= opt(sprintf("%.2d", $_), $_==$bmonth) for map((($nmonth+$_)%12)+1, 8..11); $html .= "</select>-";
  $html .= "<select name=bday>"; $html .= opt(sprintf("%.2d", $_), $_==$bday) for 1..31; $html .= "</select>";
  $html .= "&nbsp;<select name=bhour>"; $html .= opt(sprintf("%.2d", $_), $_==$bhour) for 0..23; $html .= "</select>:";
  $html .= "<select name=bminute>"; $html .= opt(sprintf("%.2d", $_), $_==$bminute) for 0..59; $html .= "</select>";

  # device id
  $html .= sprintf "<tr><td>Device&nbsp;ID<td colspan=2>%s<input type=text name=deviceid value='%s'>",($CLIENT ne 'system')?"$CLIENT.":'',(escapeHTML $deviceid);

  # cpeid 
  $html .= "<tr><td>CPE&nbsp;ID<td colspan=2><input type=text name=cpeid value='".(escapeHTML $cpeid)."'>";

  # scriptid
  $html .= "<tr><td>Script&nbsp;ID<td colspan=2><input type=text name=scriptid value='".(escapeHTML $scriptid)."'>";

  # phone
  $html .= "<tr><td>Phone<td colspan=2><input type=text name=phone value='$phone'>";

  #
  param graph => 1;

  $html .= "</table>
            <td style={vertical-align:bottom}>
            <input type=submit value='Filter'>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
    </form>
    <td valign=bottom>
    <form action='dbman.cgi' method=get>
      <input type=submit value='Done'>
      <input type=hidden name=client value='$CLIENT'>
      <input type=hidden name=pass value='$PASS'>
    </form>
  </table>
  <hr>

  <img alt='Databug activity graph' src='".self_url."'>";

  html $html;
}

# here, called as <img src, create graph  
my @filter;

if ($CLIENT ne 'system')
{
  my $s = "$CLIENT.".(($deviceid ne '')?$deviceid:'%');
  push @filter, "deviceid like ".$DB->quote($s);
} elsif ($deviceid ne '')
{
  my $s = $deviceid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter, "deviceid $not like ".$DB->quote($s);
} 

if ($cpeid ne '')
{
  my $s = $cpeid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter, "cpeid $not like ".$DB->quote($s);
}

if ($scriptid ne '')
{
  my $s = $scriptid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter,"scriptid $not like ".$DB->quote($s);
}

if ($phone ne '')
{
  my $s = $phone;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter,"phone $not like ".$DB->quote($s);
}


# parse the width
my $interval; 
my $spp; 	# seconds per period
my $ppt; 	# periods per tick
my $tpm; 	# ticks per mark (red line)
my $fmt;	# display format

$width eq $_->[0] and $interval=$_->[1], $spp=$_->[2], $ppt=$_->[3], $tpm=$_->[4], $fmt=$_->[5], last for @widths;
$interval or ($width, $interval, $spp, $ppt, $tpm, $fmt)=@{$widths[$defwidth]}; # use default

$interval = $spp*int($interval/$spp); 			# align to exact periods


# get the specified 'before' time
my $now = $before?timelocal_nocheck(0,$bminute,$bhour,$bday,$bmonth-1,$nyear):time;
my $offset = $now - timegm localtime $now;

$now = $spp*int($now/$spp);		        	# align to start of period
my $then = $now-$interval;				# determine start time	

push @filter, "unix_timestamp(time) >= $then"; # filter it
push @filter, "unix_timestamp(time) < $now";

my $skipt=($ppt-(int($then-$offset)/$spp)%$ppt)%$ppt; 		# skip to aligned tick

my $ppm = $ppt*$tpm;	                	  	# periods per mark
my $skipm=($ppm-(int($then-$offset)/$spp)%$ppm)%$ppm; 		# skip to aligned mark


my %slot;
$slot{$_->[0]}=[$_->[1],$_->[2]] for @{$DB->selectall_arrayref("select truncate(unix_timestamp(time)/$spp,0) as t, count(*), sum(records) 
      						          from databug.session where ".join(" and ",@filter)." group by t")};


my $spps =  ($spp<60)?"$spp second":($spp<3600)?($spp/60)." minute":($spp/3600)." hour"; 

# build data tables
my (@keys, @sessions, @records, @mark);
my $yticks = 5;
my $maxs=$yticks;
my $maxr=$yticks;

for (int($then/$spp)..int($now/$spp))
{
  if (@keys >= $skipm && !((@keys-$skipm)%$ppm))
  {
    push @mark, scalar @keys; # remember where marks go
  }

  if (@keys >= $skipt && !((@keys-$skipt)%$ppt))
  {
    push @keys, strftime $fmt, localtime($_*$spp); 
  } else 
  {
    push @keys, undef;
  }

  my ($s, $r) = $slot{$_}?@{$slot{$_}}:(0,0);
  $s > $maxs and $maxs = $s;
  $r > $maxr and $maxr = $r;
  push @sessions,$s; 
  push @records,$r; 
}  

$maxs = $yticks * int(($maxs+($yticks-1))/$yticks);
$maxr = $yticks * int(($maxr+($yticks-1))/$yticks);

my $graph = GD::Graph::bars->new(1000, 250);
$graph->set(
   title => strftime("%a, %b %d %H:%M %Z", localtime $then)." to ".strftime("%a, %b %d %H:%M %Z",localtime $now)." ($spps window)",
   tick_length => -4,
   fgclr => 'black',
   bgclr => 'white',
   transparent => 0,
   accentclr=>'green',
   labelclr => 'black',
   axislabelclr => 'black',
   legendclr => 'black',
   textclr => 'black',
   x_label_skip => $ppt,
   x_tick_offset => $skipt, 
   dclrs=>['green','lblue'],
   two_axes => 1,
#  x_labels_vertical => 1,
   y_tick_number => $yticks,
   y1_min_value=>0,
   y2_min_value=>0,
   y1_max_value=>$maxs,
   y2_max_value=>$maxr,
   y1_label => 'Sessions (green)',
   y2_label => 'Records (blue)',
) or warn $graph->error;

$graph->set_title_font(GD::gdSmallFont);
$graph->set_x_axis_font(GD::gdTinyFont);

my $image = $graph->plot([\@keys,\@sessions,\@records]) or die $graph->error;

# last, plot marks 
my ($x1,$y1,$x2,$y2)=@{$graph->get_feature_coordinates('axes')}[1..4];	# get axes

for (@mark)	# for each marked array index 		
{
  my $c = $image->colorAllocate(255,0,0);
  my ($x, $y)=($graph->get_hotspot(1,$_))[3,2];              		# get x,y of upper end of bar
  $x > $x1+2 && $x < $x2-2 and $image->rectangle($x,$y-1,$x,$y2+1,$c);  
} 

print "Content-type: image/png\n\n",$image->png;                              

