#!/usr/bin/perl 
# show idle devices, allow delete
use warnings;
use strict;
use lib '/crux/dbug/pm';
use dbCGI::init;
use dbCGI::gui;
use CGI qw(param escapeHTML self_url -oldstyle_urls);

# 'die' will show error to user instead of barfing apache  
local $SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

$PERMISSION > 1 or die "Forbidden\n";

# this is the state table, a named array of anonymous subroutines
my %states = ( 
  
top => sub # top level display 
{
  my @filter;

  # create filter string
  my ($ayear, $amonth, $aday, $ahour, $aminute, $asecond) = timestamp 2,qw{ayear amonth aday ahour aminute asecond};

  my $since = "$ayear-$amonth-$aday $ahour:$aminute:$asecond";

  # build it
  my $html .= "<table cellpadding=0 cellspacing=0><tr><td>
                <form method=get>
                  <table border=1 cellpadding=3 cellspacing=0>";
  
  # time
  $html .= sprintf '<tr><td>Idle Since<td>';
  $html .= "<select name=ayear>"; $html .= opt(sprintf("%.2d", $_), $_==$ayear) for 2000..2050; $html .= "</select>-";
  $html .= "<select name=amonth>"; $html .= opt(sprintf("%.2d", $_), $_==$amonth) for 1..12; $html .= "</select>-";
  $html .= "<select name=aday>"; $html .= opt(sprintf("%.2d", $_), $_==$aday) for 1..31; $html .= "</select>";
  $html .= "&nbsp;<select name=ahour>"; $html .= opt(sprintf("%.2d", $_), $_==$ahour) for 0..23; $html .= "</select>:";
  $html .= "<select name=aminute>"; $html .= opt(sprintf("%.2d", $_), $_==$aminute) for 0..59; $html .= "</select>:";
  $html .= "<select name=asecond>"; $html .= opt(sprintf("%.2d", $_), $_==$asecond) for 0..59; $html .= "</select>";

  $html .= "</table>
            <td valign=bottom>
            <input type=submit value='Filter'>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
    </form>
    <td valign=bottom>
    <form action='dbman.cgi' method=get>
      <input type=submit value='Done'>
      <input type=hidden name=client value='$CLIENT'>
      <input type=hidden name=pass value='$PASS'>
    </form>
  </table>
  <hr>";

  $html .= "<table border=1 cellpadding=2 cellspacing=0><tr>";

  my %active;
  # make list of all active devices
  for (@{$DB->selectall_arrayref('select distinct substring_index(deviceid,".",2), substring_index(cpeid,".",1) 
                                  from session where time > ?',undef,$since)})
  {
    $active{"$_->[0]:$_->[1]"}=1;
  } 

 
  # unalias all active aliases 
  for (@{$DB->selectall_arrayref('select client.client, cpe.cpeid, alias.alias, device.deviceid from client, device, cpe, alias
                                  where device.client_id=client.id and device.cpe_id=cpe.id and alias.device_id=device.id')})
  {
    my ($client, $cpeid, $alias, $deviceid)=@$_;
    $active{"$client.$alias:$cpeid"} and $active{"$client.$deviceid:$cpeid"}=1;
  }

  # die map "$_\n", keys %active;

  # hash of inactive "device name" => [isalias?, id, updated]  
  my %inactive;

  # get old devices (based on update timestamp)  
  for (@{$DB->selectall_arrayref('select client.client, device.deviceid, cpe.cpeid, device.id, date_format(device.time,"%Y-%m-%d %T") 
                                  from device, client, cpe
                                  where device.time <= ?
                                  and device.client_id=client.id and client.client'.
                                  (($CLIENT ne 'system')?'='.$DB->quote($CLIENT):"!='system'").
                                  ' and device.cpe_id=cpe.id',undef,$since)})
  {
    my $d = "$_->[0].$_->[1]:$_->[2]";
    $active{$d} or $inactive{$d}=[0,$_->[3],$_->[4]];
  }

  # aliases must be directly referenced
  for (@{$DB->selectall_arrayref('select client.client, alias.alias, cpe.cpeid, alias.id,date_format(alias.time,"%Y-%m-%d %T"), device.deviceid 
                                  from device, client, cpe, alias
                                  where alias.time <= ?
                                  and alias.device_id=device.id 
				  and device.client_id=client.id and client.client'.
                                  (($CLIENT ne 'system')?'='.$DB->quote($CLIENT):"!='system'"). 
                                  ' and device.cpe_id=cpe.id',undef,$since)})
  {
    my $d = "$_->[0].$_->[1]:$_->[2]";
    $active{$d} or $inactive{"$_->[0].$_->[1]=>$_->[5]:$_->[2]"}=[1,$_->[3],$_->[4]];
  }

  if (!keys %inactive)
  {
    $html = "No idle devices found";
  } else
  { 

    $html .= "<table border=1 cellpadding=2 cellspacing=0><tr><td><b>Device ID</b><td><b>Last Updated</b>";
    $CLIENT ne 'system' and $html .= "<td>&nbsp;";

    for (sort keys %inactive)
    {
      my ($isalias, $id, $created)=@{$inactive{$_}};
      $html .= '<tr><td class=data>'.escapeHTML($_).'<td class=data>'.escapeHTML($created);
      if ($CLIENT ne 'system')
      {
        $html .= "<td><form method=get>
                  <input type=submit value='Delete'>";
        if ($isalias)
        {
          $html .= "<input type=hidden name=state value='delalias'>
                    <input type=hidden name=alias_id value='$id'>";
        } else
        {
          $html .= "<input type=hidden name=state value='deldevice'>
                    <input type=hidden name=device_id value='$id'>";
        }
      }
      $html .= "<input type=hidden name=client value='$CLIENT'>
      	        <input type=hidden name=pass value='$PASS'>
                </form>";
      
    }

    $html .= "<tr><td><b>".(scalar keys %inactive)."</b> idle devices<td>&nbsp;</table>";
  }


  html $html;
},

deldevice => sub # query, delete deviceid?
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";
  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;

  html "<h3>You are about to delete device '$CLIENT.$deviceid:$cpeid' and associated aliases.</h3><p>
        <form method=post>
          Click <input type=submit value='YES, DELETE DEVICE!'> to continue.  
          <input type=hidden name=state value=dodeldevice>
	  <input type=hidden name=client value='$CLIENT'>
	  <input type=hidden name=pass value='$PASS'>
	  <input type=hidden name=device_id value='$device_id'>
	</form>";
},
 
dodeldevice => sub # yes, delete it
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";
  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;

  $DB->do("delete from device where client_id=? and id=?", undef, $CLIENT_ID, $device_id);
  $DB->do("delete from alias where device_id=?", undef, $device_id);
  logger "deleted $deviceid:$cpeid\n";
},

delalias => sub # query, delete deviceid?
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";
  my $alias_id = supplied 'alias_id';
  my ($alias, $deviceid, $cpeid)=getalias $alias_id;

  html "<h3>You are about to delete alias '$CLIENT.$alias:$cpeid -> $CLIENT.$deviceid:$cpeid'.</h3><p>
        <form method=post>
          Click <input type=submit value='YES, DELETE ALIAS!'> to continue.  
          <input type=hidden name=state value=dodelalias>
          <input type=hidden name=client value='$CLIENT'>
          <input type=hidden name=pass value='$PASS'>
          <input type=hidden name=alias_id value='$alias_id'>
        </form>";
},
 
dodelalias => sub # yes, delete it
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";
  my $alias_id = supplied 'alias_id';
  my ($alias, $deviceid, $cpeid)=getalias $alias_id;

  $DB->do("delete from alias where id=?", undef, $alias_id);
  logger "deleted alias $alias->$deviceid:$cpeid\n";
},

);

# here, process submitted state
my $n = param 'state';

if (defined $n)							# if exists
{
  die "Illegal state '$n'\n" unless exists $states{$n};		# and valid	
  $states{$n}->();						# maybe it returns...
}

param state=>'top';						# refresh to 'top'
$HEAD .= "<meta http-equiv='refresh' content='0;url=".self_url."'>";
html "Please wait...";

