#!/usr/bin/perl 
# databug web management tool, main
use warnings;
use strict;
use lib "/crux/dbug/pm";
use dbCGI::init;
use dbCGI::gui;
use dbCGI::compile;

use CGI qw(param upload escapeHTML Delete self_url -oldstyle_urls);
use DBI;

# wrap text like a Mailbug
sub wrap($)
{
  my $s = shift; $s =~ s/[\r\t]//g; $s =~ s/\s+$//s; 
  my @o;
  # get line if shorter than 79, or last 79, or up to last space, or whole block of 79 
  push @o, $+ while $s =~ /\G(.{0,78}?)\n|\G(.{1,79})$|\G(.{0,78})\s|\G(.{79})/gs;
  map s/\s+$//, @o;
  join "\n", @o; 
}

# abort http with error code
#sub fail(@)
#{
#  print "Status: @_\n\n@_";
#  exit 0;
#}

#if (defined param 'putscript')
#{
#  $PERMISSION or fail "401 unauthorized";
# 
#  # get parameters
#  param 'file' or fail "400 missing file";
#
#  my $cpeid = param 'cpeid' or fail "400 missing cpeid";
#  my $cpe_id = $DB->selectrow_array("select id from cpe where cpeid=?",undef,$cpeid) or fail "400 bad cpeid"; 
#
#  my $deviceid = param 'script' or fail "400 missing script"; # 'script' parameter is the device name
#  $deviceid =~ tr/A-Z/a-z/;
#  $deviceid =~ /[^-a-z0-9]/i and fail "400 invalid character in script parameter";
#
#  # find specified device, create it if necessary
#  my $device_id;
#  $DB->do("lock tables device write"); # atomic test-and-create 
#  unless ($device_id = $DB->selectrow_array("select id from device where client_id=? and deviceid=? and cpe_id=?",undef,$CLIENT_ID,$deviceid,$cpe_id))
#  {
#    # create it 
#    $DB->do("insert device (deviceid, cpe_id, client_id) values (?,?,?)",undef,$deviceid,$cpe_id,$CLIENT_ID);
#    $device_id = $DB->selectrow_array("select last_insert_id()");
#  }
#  $DB->do("unlock tables");
#  # go compile
#  local $/; my $handle=upload 'file';
#  my @result=compile($device_id, <$handle>);
#  $result[0] == 200 and logger "putscript updated '$CLIENT.$deviceid:$cpeid'";
#  fail @result; # misnomer if 200
#}

# user interface starts here

# 'die' will now show error to user instead of barfing apache  
local $SIG{__DIE__} = sub { html "Sorry, an error has occurred1:<p><pre>@_</pre>" };

# check authentication
$CLIENT && $PASS or html "<form method=get>
                          <table>
                            <tr><td>Client:   <td><input type=text name=client>
                            <tr><td>Password: <td><input type=password name=pass>
                          </table>
                          <input type=submit value=OK>

                          </form>";

$CLIENT_ID or die "Invalid login\n";

$PASS = escapeHTML $PASS; # for subsequent inclusion in forms. Note $CLIENT doesn't need this.

# this is the state table, a named array of anonymous subroutines
my %states = ( 
  
top => sub # top level display 
{
  my $html='';
  
  my $devices=$DB->selectall_arrayref('select device.id, device.deviceid, cpe.cpeid from device, cpe 
                                       where device.client_id=? and cpe.id=device.cpe_id 
                                       order by device.deviceid,cpe.cpeid',undef,$CLIENT_ID);

  $html .= "<table border=0 cellpadding=1 cellspacing=0>";
  if (@$devices)
  {
    $html .= "<form method=get>
              <input type=hidden name=client value='$CLIENT'>
              <input type=hidden name=pass value='$PASS'>
              <tr>
                <td>Device ID: <select name=device_id>
                  @{[map opt(qq($CLIENT.$_->[1]:$_->[2]),undef,$_->[0]), @$devices]}
                </select>
                <td><input type=submit name=state value='View'>
                <td><input type=submit name=state value='Update'>"; 
    
    $PERMISSION == 100 and $html .= "<td><input type=submit name=state value='Binary'>"; 

    $CLIENT ne 'system' and 
    $html .= "<td><input type=submit name=state value='Properties'>
              <td><input type=submit name=state value='Alias'>
              <td><input type=submit name=state value='Delete'>";
    
    $html .= "</form>";
  } else
  {
    $html .= "<tr><td>No devices";
  }
  
  $html .= "<td><large>&nbsp;|&nbsp;
            <td><form method=get>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
            <input type=hidden name=state value='adddeviceid'>
            <input type=submit value='New'>
            </form></table><hr>";


  my $aliases=$DB->selectall_arrayref('select alias.id, alias.alias, device.deviceid, cpe.cpeid from alias, device, cpe 
                                       where device.client_id=? and alias.device_id=device.id and cpe.id=device.cpe_id
                                       order by alias.alias,cpe.cpeid',undef,$CLIENT_ID);
  @$aliases and
  $html .= "<table border=0 cellpadding=1 cellspacing=0>
            <form method=get>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
            <tr>
              <td>Alias: <select name=alias_id>
                @{[map opt(qq($CLIENT.$_->[1]=>$_->[2]:$_->[3]),undef,$_->[0]), @$aliases]}
              </select>
              <td><input type=submit name=state value='Delete Alias'>
            </form>
            </table>
            <hr>";

  if ($CLIENT eq 'system')
  {
    my $clients=$DB->selectall_arrayref('select id, client from client where client != "system" order by client'); 
    $html .= "<table border=0 cellpadding=1 cellspacing=0>";
    if (@$clients)
    { $html .= "<tr>
                <form method=get>
                <input type=hidden name=client value='$CLIENT'>
                <input type=hidden name=pass value='$PASS'>
                <td>Client: <select name=client_id>@{[map opt($_->[1],undef,$_->[0]),@$clients]}</select>
                <td><input type=submit name=state value='Become'>
                <td><input type=submit name=state value='Client Properties'>
                <td><input type=submit name=state value='Delete Client'>
                </form>";
     } else
     {
       $html .= "<tr><td>No clients";
     }
    
     $html .= "<td><large>&nbsp;|&nbsp;
               <td><form method=get>
               <input type=hidden name=client value='$CLIENT'>
               <input type=hidden name=pass value='$PASS'>
               <input type=submit name=state value='New Client'></form>
              </table><hr>";
  }

  $html .= "<table border=0 cellpadding=1 cellspacing=0>
            <tr>
            <td><form action='dbrecords.cgi' method=get>
                <input type=submit value='Records'>
                <input type=hidden name=client value='$CLIENT'>
                <input type=hidden name=pass value='$PASS'>
                </form>

            <td><form action='dbmsgs.cgi' method=get>
                <input type=submit value='Device Messages'>
                <input type=hidden name=client value='$CLIENT'>
                <input type=hidden name=pass value='$PASS'>
                </form>";

  # some can see the session log
  $PERMISSION > 1 and 
  $html .= "<td><form action='dbsessions.cgi' method=get>
                <input type=submit value='Sessions'>
                <input type=hidden name=client value='$CLIENT'>
                <input type=hidden name=pass value='$PASS'>
                </form>".

           # <td><form action='dbactivity.cgi' method=get>
           #     <input type=submit value='Activity'>
           #     <input type=hidden name=client value='$CLIENT'>
           #     <input type=hidden name=pass value='$PASS'>
           #     </form>
 
           "<td><form action='dbidle.cgi' method=get>
                <input type=submit value='Idle Devices'>
                <input type=hidden name=client value='$CLIENT'>
                <input type=hidden name=pass value='$PASS'>
                </form>

            <td><large>&nbsp;|&nbsp;";

  $CLIENT ne 'system' and 
  $html .= "<td><form method=get>
                <input type=submit name=state value='Global Properties'>
                <input type=hidden name=client value='$CLIENT'>
                <input type=hidden name=pass value='$PASS'>
                </form>";

  if ($CLIENT ne 'system' and $PERMISSION == 100) # system has 'become' a user
  {
    # allow back to 'system' 
    $html .= "<td><form method=get>
                  <input type=hidden name=client value='system'>
                  <input type=hidden name=pass value='$PASS'>
                  <input type=submit value='System'>";
  } else
  {
    $html .= "<td><form method=get><input type=submit value='Logout'></form>";
  }


  html $html;
},

'View' => sub # view script source
{
  my $device_id = supplied 'device_id';
  my $text = $DB->selectrow_array('select source from device where id=? and client_id=?',undef,$device_id, $CLIENT_ID) || '(no data)';
  print "Content-type: text/plain\n\n$text";
  exit;
},

'Binary' => sub # get compiled code
{
  my $device_id = supplied 'device_id';
  my ($deviceid,$cpeid) = getdevice $device_id;
  my ($obj254, $obj255) = ($DB->selectrow_array('select obj254, obj255 from device where id=? and client_id=?',undef,$device_id, $CLIENT_ID));
  defined $obj254 and die "Can't download binary for $cpeid device.\n";
  print "Content-type: binary/octet-stream\nContent-Disposition: attachment; filename=$CLIENT.$deviceid:$cpeid.bin\n\n$obj255";
  exit;
},

'Update' => sub # query for new script, post
{
  my $device_id = supplied 'device_id';
  my ($deviceid,$cpeid) = getdevice $device_id;

  html "Enter filename of new script for <b>$CLIENT.$deviceid:$cpeid</b>:<p> 
	<form method=post enctype='multipart/form-data'>
	  Filename: <input type=file name=file> 
	  <input type=submit value=OK>
	  <input type=hidden name=state value=compilescript>
	  <input type=hidden name=client value='$CLIENT'>
	  <input type=hidden name=pass value='$PASS'>
	  <input type=hidden name=device_id value='$device_id'>
	</form>";
},

compilescript => sub # get posted file and compile
{
  my $device_id = supplied 'device_id';

  param 'file' or die "You didn't specify a file name\n";
  
  my ($deviceid, $cpeid)=getdevice $device_id;
  my $handle=upload 'file';
  local $/; 
  my @result = compile ($device_id, <$handle>);
  $result[0] != 200 and die "$result[1]\n";
  logger "updated script for $CLIENT.$deviceid:$cpeid";
},

'adddeviceid' => sub # query for new device id
{
  # list of allowed CPEs
  my %cpeids; $cpeids{$_->[0]}=$_->[1] for @{$DB->selectall_arrayref('select cpeid, id from cpe order by cpeid')}; 
  
  html "Enter a new Device ID and CPE:<p>
        <form method=post>
          &nbsp;$CLIENT.<input type=text name='deviceid' value=''>:
          <select name='cpe_id'>", 
            (map opt($_,undef,$cpeids{$_}), sort keys %cpeids),
         "</select>
          <input type=submit value=OK>
          <input type=hidden name=state value=doadddeviceid>
          <input type=hidden name=client value='$CLIENT'>
          <input type=hidden name=pass value='$PASS'>
         </form>";
},

doadddeviceid => sub # create the device id 
{
  my $deviceid = param 'deviceid' or die "You didn't specify a Device ID\n";
  $deviceid =~ tr/A-Z/a-z/;
  $deviceid =~ /[^-a-z0-9]/ && die "Only letters, numbers, and '-' can be used in the Device ID.\n";
  my $cpe_id = supplied 'cpe_id';
  my $cpeid=$DB->selectrow_array("select cpeid from cpe where id=?",undef,$cpe_id) or die "Invalid cpe_id '$cpe_id'\n";
  
  my $existing;
  $existing = $DB->selectrow_array("select device.deviceid from alias, device 
                                    where alias.alias=? and device.id = alias.device_id and device.cpe_id=? and device.client_id=?",
                                    undef,$deviceid, $cpe_id, $CLIENT_ID) and die "Alias '$CLIENT.$deviceid->$existing:$cpeid' already exists\n";

  $DB->do("lock tables device write"); # atomic test-and-create

  $DB->selectrow_array("select 1 from device where deviceid=? and cpe_id=? and client_id=?",undef,$deviceid, $cpe_id, $CLIENT_ID) 
                        and die "Device '$CLIENT.$deviceid:$cpeid' already exists\n";

  $DB->do("insert device (deviceid, client_id, cpe_id) values (?,?,?)", undef, $deviceid, $CLIENT_ID, $cpe_id);
  $DB->do("unlock tables");

  logger "added deviceid $CLIENT.$deviceid:$cpeid";
},

'Alias' => sub # query for new alias
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;
  
  html "Please enter an alias for <b>$CLIENT.$deviceid:$cpeid</b>:<p>
	<form method=post>
	  &nbsp;$CLIENT.<input type=text name='alias' value=''>=>$deviceid:$cpeid
	  <input type=submit value=OK>
	  <input type=hidden name=state value=domakealias>
	  <input type=hidden name=device_id value='$device_id'>
	  <input type=hidden name=client value='$CLIENT'>
	  <input type=hidden name=pass value='$PASS'>
	</form>";
},

domakealias => sub # create the symlink to actual device 
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $alias = (param 'alias') or die "You didn't specify an alias.\n";
  $alias =~ tr/A-Z/a-z/;
  $alias =~ /[^-a-z0-9]/ && die "Only letters, numbers, and '-' can be used in the alias.\n";
  my $device_id = supplied 'device_id';

  my ($deviceid, $cpeid, $cpe_id)=($DB->selectrow_array("select device.deviceid, cpe.cpeid, cpe.id from device, cpe 
                                                        where device.client_id=? and device.id=? and cpe.id=device.cpe_id",
                                                        undef,$CLIENT_ID, $device_id));
  $deviceid or die "Invalid device_id $device_id\n";
 
  my $existing;
  $existing = $DB->selectrow_array("select device.deviceid from alias, device 
                                    where alias.alias=? and device.id = alias.device_id and device.cpe_id=? and device.client_id=?",
                                    undef, $alias, $cpe_id, $CLIENT_ID) and die "Alias '$CLIENT.$alias=>$existing:$cpeid' already exists\n";

  # check for existing device id
  $DB->do("lock tables device write, alias write");
  $DB->selectrow_array("select 1 from device where deviceid=? and cpe_id=? and client_id=?",undef,$alias, $cpe_id, $CLIENT_ID) 
     and die "Device '$CLIENT.$alias:$cpeid' already exists\n";

  $DB->do("insert alias (alias, device_id) values (?,?)", undef, $alias, $device_id);
  $DB->do("unlock tables");

  logger "added alias $CLIENT.$alias=>eviceid:$cpeid";
},

'Delete' => sub # query, delete deviceid?
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;

  html "<h3>You are about to delete device '$CLIENT.$deviceid:$cpeid' and associated aliases.</h3><p>
        <form method=post>
          Click <input type=submit value='YES, DELETE DEVICE!'> to continue.  
          <input type=hidden name=state value=dodeldevice>
	  <input type=hidden name=client value='$CLIENT'>
	  <input type=hidden name=pass value='$PASS'>
	  <input type=hidden name=device_id value='$device_id'>
	</form>";
},
 
dodeldevice => sub # yes, delete it
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;

  $DB->do("delete from device where client_id=? and id=?", undef, $CLIENT_ID, $device_id);
  $DB->do("delete from alias where device_id=?", undef, $device_id);
  logger "deleted $CLIENT.$deviceid:$cpeid\n";
},

'Delete Alias' => sub # query, delete deviceid?
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $alias_id = supplied 'alias_id';
  my ($alias, $deviceid, $cpeid)=getalias $alias_id;

  html "<h3>You are about to delete alias '$CLIENT.$alias=>$deviceid:$cpeid'.</h3><p>
        <form method=post>
          Click <input type=submit value='YES, DELETE ALIAS!'> to continue.  
          <input type=hidden name=state value=dodelalias>
          <input type=hidden name=client value='$CLIENT'>
          <input type=hidden name=pass value='$PASS'>
          <input type=hidden name=alias_id value='$alias_id'>
        </form>";
},
 
dodelalias => sub # yes, delete it
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $alias_id = supplied 'alias_id';
  my ($alias, $deviceid, $cpeid)=getalias $alias_id;

  $DB->do("delete from alias where id=?", undef, $alias_id);
  logger "deleted alias $CLIENT.$alias=>$deviceid:$cpeid\n";
},

'Global Properties' => sub # edit global mailto list
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my ($mailto, $message, $msg_priority, $ftpto, $pgpkey, $ftpstyle) = 
                    ($DB->selectrow_array("select mailto, message, msg_priority, ftpto, pgpkey, ftpstyle from client where id=?",undef,$CLIENT_ID));

  html "Global Properties:<p>
        <form method=post>
        <table border=0 cellpadding=1 cellspacing=0>
        <tr><td>Mailto:<tr><td colspan=2><textarea name=mailto rows=6 cols=80>",(escapeHTML $mailto),"</textarea> 
            <td><td valign=bottom><small><i>
            Enter one address per line in format <b>user\@tld.dom</b>.  Do not use
            quotes, <b>&lt;</b>, <b>&gt;</b>, etc. Invalid lines will be quietly ignored
            (including those that start with <b>#</b>). 

        <tr><td><hr>
        
        <tr><td>Global Message:<tr><td colspan=2><textarea name=message rows=6 cols=80>",(escapeHTML $message),"</textarea>
            <td><td valign=bottom><small><i>XX03 and later only.

        <tr><td>Global message has priority:<td><input type=checkbox name=msg_priority".($msg_priority?" checked":"").">
            <td><td valign=bottom><small><i>
            If checked, only the global message will be delivered to the device, even if blank.  
      
        <tr><td><hr>
        
        <tr><td>FTPto:<td><input type=text name=ftpto value='$ftpto' size=60>
            <td><td valign=bottom><small><i>
            FTP delivery host in format <b>user:pass\@host</b>, with optional
            <b>/path</b>. Disabled if starts with <b>#</b>.
        
        <tr><td>Send as CSV:<td><input type=checkbox name=ftpstyle".($ftpstyle?" checked":"").">
            <td><td valign=bottom><small><i>
            If checked, FTP data will be sent in CSV format. Note, requires
            special scripting.  
        
        <tr><td>PGP public key:<tr><td colspan=2><textarea name=pgpkey rows=16 cols=80>",(escapeHTML $pgpkey),"</textarea>
            <td>&nbsp;<td valign=bottom><small><i>
            Base-64 portion of armored PGP public key (without BEGIN, END,
            Version: lines), for FTP delivery. Leaave blank to deliver plain-text.   
        
        </table>
        <br>
	<input type=submit value=Save>
	<input type=hidden name=state value=saveglobalprops>
	<input type=hidden name=client value='$CLIENT'>
	<input type=hidden name=pass value='$PASS'>
        </form>";
},

saveglobalprops => sub # save global message
{ 
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  (my $mailto = param 'mailto') =~ s/^\s+|\s$//gm; 
  my $message = wrap param 'message';
  length $message > 1023 and die "Message cannot exceed 1023 characters\n";
  my $msg_priority = defined(param 'msg_priority')?1:0; 
  (my $ftpto = param 'ftpto') =~ s/^\s+|\s$//g; 
  $ftpto && $ftpto !~ /^\S+:\S+\@[-_.a-z0-9]+([\/\\]\S+)?$/ix and die "FTPto: is invalid\n";
  
  (my $pgpkey = param 'pgpkey') =~ s/^\s+|\s$//gm; 
  $pgpkey =~/Version:|Comment:|PUBLIC KEY BLOCK/m and die "Please strip headers and comments from the PGP key."; 
  
  my $ftpstyle = defined(param 'ftpstyle')?1:0; 
  
  $DB->do("update client set mailto=?, message=?, msg_priority=?, ftpto=?, pgpkey=?, ftpstyle=? where id=?",
    undef,$mailto,$message,$msg_priority,$ftpto,$pgpkey,$ftpstyle,$CLIENT_ID);
  logger "changed client properties";
},

'Properties'=> sub # edit device properties
{
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;
  my ($mailto, $message, $changed, $update_delay, $private_key, $public_key)=
    ($DB->selectrow_array("select mailto, message, date_format(time,'%a, %d %b %Y %T'), 
                           update_delay, private_key, public_key from device 
                           where client_id=? and id=?",undef,$CLIENT_ID,$device_id));
  my ($SENDMSG, $CRYPT)=($DB->selectrow_array("select sendmsg, crypt from cpe where cpeid=?",undef,$cpeid));

  my $html = "Properties for <b>$CLIENT.$deviceid:$cpeid</b>:<p>
              <form method=post>
              <table border=0 cellpadding=1 cellspacing=0>
              <tr><td>Mailto:<tr>
                  <td><textarea name=mailto rows=6 cols=80>".(escapeHTML $mailto)."</textarea>
                  <td>&nbsp;
                  <td valign=bottom><small><i>
	                Enter one address per line on format 'user\@tld.dom'.  Do not use
                  quotes, '&lt;', '&gt;', etc. Invalid lines will be quietly ignored (including
                  those that start with '#'). <b>Any</b> entry in this field will override global
                  mailto.";

  $SENDMSG and 
  $html .= "<tr><td>Message:<tr>
                <td><textarea name=message rows=6 cols=80>".(escapeHTML $message)."</textarea>
                <td>&nbsp;
                <td valign=bottom><small><i><b>Any</b> entry in this field will override global message.";

  $html .= "<tr><td>Update Delay: <input type=text name=update_delay value='$update_delay'> 
                  <td>&nbsp;
                  <td valign=bottom><small><i>
		              Maximum age in seconds of a new script before guaranteed
                  update. Range 0 (update on next call) to 86400 (update sometime within next 24
                  hours).";

  $CRYPT and
  $html .= "<tr><td>Capture private (D) key: <input type=text name=private_key value='$private_key' size=50>
                  <td>&nbsp;
                  <td valign=bottom><small><i>
                      If defined, must be paired with the public key, 32 hex chars.
            <tr><td>Capture public (E) key: <input type=text name=public_key value='$public_key' size=50>
                  <td>&nbsp;
                  <td valign=bottom><small><i>
                      If defined, must be paired with the private key, 32 chars (this is the key that also goes in the script).";
  

  $html .= "</table>
            <br>
            <input type=submit value=Save>
            <input type=hidden name=state value=savedevprops>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
            <input type=hidden name=device_id value='$device_id'>
            </form><hr><small><small><i>Last changed $changed";

  html $html;
 },

savedevprops => sub # save device properties
{ 
  $CLIENT eq 'system' and die "Operation not allowed for 'system'\n";

  my $device_id = supplied 'device_id';
  my ($deviceid, $cpeid)=getdevice $device_id;
  (my $mailto = param 'mailto') =~ s/^\s+|\s$//gm; 
  my $message = wrap param 'message';
  length $message > 1023 and die "Message cannot exceed 1023 characters\n";
  
  my $update_delay = (param 'update_delay')||0;
  $update_delay =~ /\D/ and $update_delay = 0;
  $update_delay > 86400 and $update_delay = 86400; 

  my $private_key = uc(param 'private_key');
  my $public_key = uc(param 'public_key');
 
  if ($private_key xor $public_key) { die "Must supply both or neither of the capture keys.\n"; }
  
  if ($private_key)
  {
    $private_key !~ /^[0-9A-F]{32}$/ and die "Capture private key must be 32 hex digits.\n";
    $public_key !~ /^[0-9A-F]{32}$/ and die "Capture public key must be 32 hex digits.\n";
    # XXX test keys here!
  } 

  $DB->do("update device set mailto=?, message=?, update_delay=?, private_key=?, public_key=? where id=?", 
        undef, $mailto, $message, $update_delay, $private_key, $public_key, $device_id);
  logger "changed properties for $CLIENT.$deviceid:$cpeid";
},

'New Client' => sub # query for new client id
{
  $CLIENT eq 'system' or die "Forbidden\n";
   
  html "Please enter the new client name. Only letters, numbers and '-' can be used.<p>
        <form method=post>
         Client name <input type=text name='addclient' value=''>
         <input type=submit value=OK>
         <input type=hidden name=state value=doaddclient>
         <input type=hidden name=client value='$CLIENT'>
         <input type=hidden name=pass value='$PASS'>
        </form>";
},

doaddclient => sub # create client 
{
  $CLIENT eq 'system' or die "Forbidden\n";

  my $addclient = param 'addclient' or die "You didn't specify a client name\n";
  $addclient =~ tr/A-Z/a-z/; $addclient =~ /[^-a-z0-9]/ && die "Only letters, numbers, and '-' can be used in the client name.\n";
  $DB->do("lock tables client write");
  $DB->selectrow_array("select 1 from client where client=?",undef,$addclient) and die "Client already exists\n";
  $DB->do("insert client (client) values (?)",undef,$addclient);
  $DB->do("unlock tables");
  logger "added client $addclient";
},

'Client Properties' => sub # client password
{
  $CLIENT eq 'system' or die "Forbidden\n";
  
  my $client_id = supplied 'client_id';
  my ($client, $password, $permission)=getclient $client_id;
  
  html "Properties for <b>$client</b>:<p>
        <form method=post>
        <table border=0 cellpadding=1 cellspacing=0>
         <tr><td>Password: 
             <td><input type=text name='newpasswd' value='",(escapeHTML $password),"'>
             <td>&nbsp;
             <td><small><i>Delete the password entirely to disable logins as this client.
         <tr><td>Permission: 
             <td><select name='permission'>", 
                   opt("XML and Client messages",$permission==0,0), 
                   opt("Device Access",$permission==1,1), 
                   opt("Full Access",$permission==2,2), 
                "</select>
         </table>   
         <br>
         <input type=submit value=OK>
         <input type=hidden name=state value=dosetaccess>
         <input type=hidden name=client_id value='$client_id'>
         <input type=hidden name=client value='$CLIENT'>
         <input type=hidden name=pass value='$PASS'>
        </form>";
        
},

dosetaccess => sub # create client password 
{
  $CLIENT eq 'system' or die "Forbidden\n";

  my $client_id = supplied 'client_id';
  my ($client) = getclient $client_id;
  my $newpasswd = (param 'newpasswd')||"";
  my $permission = (param 'permission')||0;
  $DB->do('update client set passwd=?, permission=? where id=?',undef,$newpasswd, $permission, $client_id);
  logger "changed access for '$client'";
},

'Delete Client' => sub # query, delete deviceid?
{
  $CLIENT eq 'system' or die "Forbidden\n";

  my $client_id = supplied 'client_id';
  my ($client) = getclient $client_id;
    
  html "<h3>You are about to delete client '$client', including all devices and aliases. XML and session records will not be affected.</h3><p>
        <form method=post>
           Click <input type=submit value='YES, DELETE CLIENT'> to continue.  
           <input type=hidden name=state value=dodelclient>
           <input type=hidden name=client value='$CLIENT'>
           <input type=hidden name=pass value='$PASS'>
           <input type=hidden name=client_id value='$client_id'>
         </form>"
},
 
dodelclient => sub # yes, delete it
{
  $CLIENT eq 'system' or die "Forbidden\n";

  my $client_id = supplied 'client_id';
  my ($client) = getclient $client_id; # verify
  $client eq 'system' and die "can't delete system\n";

  $DB->do("delete alias from alias, device where device.client_id = ? and device.id = alias.device_id",undef,$client_id);
  $DB->do("delete from device where client_id = ?",undef,$client_id);
  $DB->do("delete from client where id = ?",undef,$client_id);
  logger "deleted client $client";
},

'Become' => sub # become new client 
{
  $CLIENT eq 'system' or die "Forbidden\n";
  param client => (getclient supplied 'client_id')[0];
  Delete 'client_id'; # remove from parameter list
  # fall thru to refresh
},

# end of state table
);

if (!$PERMISSION) 
{
  # no permissions, just allow xml access
  html "<table border=0 cellpadding=1 cellspacing=0>
        <tr>
        <td><form action='dbrecords.cgi' method=get>
            <input type=submit value='Records'>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
            </form>

        <td><form action='dbmsgs.cgi' method=get>
            <input type=submit value='Device Messages'>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
        </form>
        <td><form method=get>
            <input type=submit value='Logout'>
            </form>
         </table>";
}

# here, process submitted state
my $n = param 'state'; 	# get state

if (defined $n)
{
  die "Illegal state '$n'\n" unless exists $states{$n};		# and valid	
  $states{$n}->(); 						# execute state
}

# on exit or if no state, refresh top
param(state => 'top');
$HEAD .= "<meta http-equiv='refresh' content='0;url=".self_url."'>";
html "Please wait...";

