#!/usr/bin/perl
# show databug xml records for client 
use warnings;
use strict;
use lib '/crux/dbug/pm';
use dbCGI::init;
use dbCGI::gui;

use CGI qw(param Delete escapeHTML self_url -oldstyle_urls);

# 'die' will show error to user instead of barfing apache  
local $SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

$CLIENT_ID or die "Forbidden\n";

# process parameters
my @filter;

sub like($$)
{
  my ($thing, $is) = @_;
  my $not = ($is =~ s/^!//);
  $is ne '' and push @filter, $thing.(($is =~ /[^\][%_]/)?($not?" not like ":" like "):($not?" != ":" = ")).$DB->quote($is);
}

my $deviceid = param('deviceid')||'';
if ($CLIENT ne 'system')
{
  if ($deviceid eq '')
  {
    like "substring_index(deviceid,'.',1)", $CLIENT;
  } else
  {
    like("deviceid","$CLIENT.$deviceid");
  }
} else
{
 if ($deviceid =~ /^([^.%_]+)(\.%?)?$/)
 {
   like "substring_index(deviceid,'.',1)", $1;
 } else
 {
   like("deviceid",$deviceid);
 }
}

my ($ayear, $amonth, $aday, $ahour, $aminute, $asecond) = timestamp 0,qw{ayear amonth aday ahour aminute asecond};
my ($byear, $bmonth, $bday, $bhour, $bminute, $bsecond) = timestamp 1,qw{byear bmonth bday bhour bminute bsecond};

my $after = param('after')||0 and push @filter, "time >= '$ayear-$amonth-$aday $ahour:$aminute:$asecond'";
my $before = param('before')||0 and push @filter,"time <= '$byear-$bmonth-$bday $bhour:$bminute:$bsecond'";

my $recordid = param('recordid') || "";
{
  my $s = $recordid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter,"recordid $not like ".$DB->quote("\%$s\%");
}

my $insensitive = param('insensitive') || "";

my $containing = param('containing') || "";
{
  my $s = $containing;
  my $not = ($s =~ s/^!//)?"not":"";
  if ($s ne '')
  {
    if ($insensitive) { push @filter,"lcase(body) $not like lcase(".$DB->quote("\%$s\%").")" }
                 else { push @filter,"body $not like ".$DB->quote("\%$s\%") }
  }              
}

my $recstate = param('recstate') || "All";
for ($recstate)
{
  /^Deleted/i and push(@filter,"state = 2"), last;
  /^Not Deleted/i and push(@filter,"state < 2"), last;
  /^Pulled/i and push(@filter,"state = 1"), last;
  /^Not Pulled/i and push(@filter, "state = 0"), last;  
  $recstate = 'All';
}
  
my $show=param('show') || "Last 1";
my $rev = 0;
my $order = "order by id asc"; 
 
if ($show =~ /^Last (10?0?)$/)
{
  $order = "order by id desc limit $1";
  $rev = 1;
} elsif ($show =~ /^First (10?0?)$/)
{ 
  $order = "order by id asc limit $1";
} 

my $filter = ""; @filter and $filter = "where ".join ' and ', @filter;

# this is the state table, a named array of anonymous subroutines
my %states = ( 
  
top => sub # top level display 
{
  my $query = "select recordid, id, state, time, deviceid from xml $filter $order";
#  die $query; 
  my $r = $DB->selectall_arrayref($query);

  my $html .= "<table cellpadding=0 cellspacing=0><tr><td>
               <form method=get>
               <table border=1 cellpadding=3 cellspacing=0>";

  $html .= sprintf '<tr><td rowspan=2>Time<td><input type=checkbox name=after%s>On or after<td>',$after?" checked":"";
  $html .= "<select name=ayear>"; $html .= opt(sprintf("%.2d", $_), $_==$ayear) for 2000..2050; $html .= "</select>-";
  $html .= "<select name=amonth>"; $html .= opt(sprintf("%.2d", $_), $_==$amonth) for 1..12; $html .= "</select>-";
  $html .= "<select name=aday>"; $html .= opt(sprintf("%.2d", $_), $_==$aday) for 1..31; $html .= "</select>";
  $html .= "&nbsp;<select name=ahour>"; $html .= opt(sprintf("%.2d", $_), $_==$ahour) for 0..23; $html .= "</select>:";
  $html .= "<select name=aminute>"; $html .= opt(sprintf("%.2d", $_), $_==$aminute) for 0..59; $html .= "</select>:";
  $html .= "<select name=asecond>"; $html .= opt(sprintf("%.2d", $_), $_==$asecond) for 0..59; $html .= "</select>";

  $html .= sprintf '<tr><td><input type=checkbox name=before%s>On or before<td>',$before?" checked":"";
  $html .= "<select name=byear>"; $html .= opt(sprintf("%.2d", $_), $_==$byear) for 2000..2050; $html .= "</select>-";
  $html .= "<select name=bmonth>"; $html .= opt(sprintf("%.2d", $_), $_==$bmonth) for 1..12; $html .= "</select>-";
  $html .= "<select name=bday>"; $html .= opt(sprintf("%.2d", $_), $_==$bday) for 1..31; $html .= "</select>";
  $html .= "&nbsp;<select name=bhour>"; $html .= opt(sprintf("%.2d", $_), $_==$bhour) for 0..23; $html .= "</select>:";
  $html .= "<select name=bminute>"; $html .= opt(sprintf("%.2d", $_), $_==$bminute) for 0..59; $html .= "</select>:";
  $html .= "<select name=bsecond>"; $html .= opt(sprintf("%.2d", $_), $_==$bsecond) for 0..59; $html .= "</select>";

  $html .= "<tr><td>State<td colspan=2><select name=recstate>";
  $html .= opt($_,$_ eq $recstate) for "All","Deleted","Not Deleted","Pulled","Not Pulled"; $html .= "</select>";


  $html .= "<tr><td>Device ID<td colspan=2>"; $CLIENT ne 'system' and $html .= "$CLIENT."; $html .= "<input type=text name=deviceid value='$deviceid'>
            <tr><td>Record ID<td colspan=2><input type=text name=recordid value='$recordid'>
            <tr><td>Containing<td colspan=2><input type=text name=containing value='$containing'>
                                            &nbsp;
                                            (ignore case <input type=checkbox name=insensitive".($insensitive?" checked":"").">)";

  $html .= "<tr><td>Show<td colspan=2><select name=show>"; 
  $html .= opt($_, $_ eq $show) for "All", "Last 1", "Last 10", "Last 100", "First 1", "First 10", "First 100"; $html .= "</select>";

  $html .= "</table>
            <td valign=bottom>
            <input type=submit value='Filter'>
            <input type=hidden name=client value='$CLIENT'>
            <input type=hidden name=pass value='$PASS'>
            </form>
            <td valign=bottom>
	    <form action='dbman.cgi' method=get>
	    <input type=submit value='Done'>
	    <input type=hidden name=client value='$CLIENT'>
	    <input type=hidden name=pass value='$PASS'>
            </form>
            </table>";

  if (@$r)
  {
    $rev and  $r = [reverse @$r];
    $html .= "<hr><table border=1 cellpadding=1 cellspacing=0>";
    for (@$r)			
    {
      $html .= "<tr><td class=data>$_->[3]<td class=data>$_->[4]<td class=data ";
      $html .= "bgcolor=yellow" if $_->[2] == 2; 	# fetched and deleted 
      $html .= "bgcolor=blue" if $_->[2] == 1;    # fetched but not deleted
      $html .= ">$_->[0]"; 
      $html .= "    
	        <td><form method=get>
	          <input type=submit value='View'>
	          <input type=hidden name=state value='view'>
	          <input type=hidden name=id value='$_->[1]'>
	          <input type=hidden name=client value='$CLIENT'>
	          <input type=hidden name=pass value='$PASS'>
                  <select name=as>
                   <option selected value=0>As HTML</option> 
                   <option value=1>As text/xml</option>
                   <option value=2>As text/plain</option>  
                   <option value=3>As raw data</option>  
                  </select> 
	        </form>";
      $PERMISSION == 100 and   
      $html .= "    
	        <td><form method=get>
          <input type=submit value='".(($_->[2]<2)?"Delete":"Undelete")."'>
	          <input type=hidden name=state value='chone'>
	          <input type=hidden name=id value='$_->[1]'>
	          <input type=hidden name=client value='$CLIENT'>
	          <input type=hidden name=pass value='$PASS'>
	        </form>";

    }
    $html .=  "</table>";

    if ($recstate =~ 'Deleted' && $CLIENT ne 'system' && $PERMISSION == 100)
    {

      param 'state'=>'chstate'; 
      $html .= "<hr>
                <form method=post>"
                .(join '', map "<input type=hidden name=$_ value='".(escapeHTML param $_)."'>", param).
               "<input type=submit value='".($recstate eq "Deleted"?"Undelete":"Delete")."'> ALL matching records (not just those shown).
                </form>";
    }
  } else { $html .= "<hr>No records match the filter"; }
  
  html $html;#."<hr><small><small>Query: $query";
},
  
view => sub # view record
{
  my $id = supplied 'id';
  my $as = param 'as';
  my $r= $DB->selectrow_arrayref("select deviceid, raw, recordid, body from xml where id=?",undef,$id) or die "couldn't access record $id";
  
  $CLIENT eq 'system' || $r->[0] =~ /^$CLIENT\./ or die "Record not owned by $CLIENT\n"; 

  my $xml = "<DB index='$r->[2]'>\n$r->[3]\n</DB>";
  $as or html "<pre>",escapeHTML $xml,"</pre>";
  $as == 3 and html "<pre>",$r->[1]?escapeHTML $r->[1]:"(none)","</pre>";
  
  print "Content-type: text/".(($as==1)?"xml\n\n<?xml version='1.0' encoding='ISO-8859-1'?>":"plain\n"),"\n$xml";
  exit;  
},

chstate => sub
{
  $CLIENT ne 'system' && $recstate =~ 'Deleted' && $PERMISSION == 100 or die "Forbidden\n";

  my $count = $DB->selectrow_array("select count(*) from xml $filter");
  $count or die "No matching records\n";  
   
  my $action = $recstate eq "Deleted"?"undelete":"delete";

  param 'state' => 'dochstate';
  html "<h3>You are about to $action $count records.</h3><p>
        <form>
        Click <input type=submit value='YES, \U$action\E THE RECORDS!'> to continue.",
        map("<input type=hidden name=$_ value='".(escapeHTML param $_)."'>", param),
        "</form>";
},

dochstate => sub
{
  $CLIENT ne 'system' && $recstate =~ 'Deleted' && $PERMISSION == 100 or die "Forbidden\n";
  if ($recstate eq "Deleted")
  {
    $DB->do("update xml set state=0 $filter"); 
    logger("undelete $filter");
  } else
  {
    $DB->do("update xml set state=2 $filter");
    logger ("delete $filter");
  }
  Delete 'recstate';
},

chone => sub
{
  my $id = supplied 'id';
  $PERMISSION == 100 or die "Forbidden\n";
  
  my @r = $DB->selectrow_array("select deviceid, state, recordid from xml where id=?",undef,$id) or die "Invalid record id\n";  
  $CLIENT eq 'system' || $r[0] =~ /^$CLIENT\./ or die "Record not owned by $CLIENT\n"; 

  my $action = $r[1]<2?"delete":"undelete";

  param 'state' => 'dochone';
  html "<h3>You are about to $action record $r[2] from device $r[0].</h3><p>
        <form>
        Click <input type=submit value='YES, \U$action\E THE RECORD!'> to continue.",
        map("<input type=hidden name=$_ value='".(escapeHTML param $_)."'>", param),
        "</form>";
},

dochone => sub
{
  my $id = supplied 'id';
  $PERMISSION == 100 or die "Forbidden\n";
  
  my @r = $DB->selectrow_array("select deviceid, state, recordid from xml where id=?",undef,$id) or die "Invalid record id\n";  
  $CLIENT eq 'system' || $r[0] =~ /^$CLIENT\./ or die "Record not owned by $CLIENT\n"; 
  
  my $action = $r[1]<2?"2":"0";

  $DB->do("update xml set state=? where id=?", undef,$action, $id);
  Delete 'id';
  Delete 'recstate';
},
);

# here, process submitted state
my $n = param 'state'; 						# get state

if (defined $n)							# if defined 
{
  die "Illegal state '$n'\n" unless exists $states{$n};		# and valid	
  $states{$n}->(); 						# execute state
}

param state=>'top';
$HEAD .= "<meta http-equiv='refresh' content='0;url=".self_url."'>";
html "Please wait...";

