# LSP crypto system - public key 
# see also the 'keygen' script
package crypto;

use warnings; 
use strict;
use Math::BigInt;

sub new($$$)
{
    shift;
    my $self=bless {};

    $self->{E}=Math::BigInt->new("0x".shift); # E key
    $self->{D}=Math::BigInt->new("0x".shift); # D key
    $self;
}    

# encrypt hex string using E key
sub encrypt($$)
{
  my $self=shift;
  my $M=Math::BigInt->new("0x".shift);

  $self->{E}->bcmp($M)<0 and die "M must be less than E, aborting!\n";
  
  # Me = Mp ^ 3 % E
  $M->bmodpow(3,$self->{E});

  uc substr $M->as_hex,2;
}  
  

# decrypt hex string using D and E keys
sub decrypt($$$$)
{
  my $self=shift;
  my $M=Math::BigInt->new("0x".shift);

  $self->{E}->bcmp($M)<0 and die "M must be less than E, aborting!\n";
  
  # Mp = Mc ^ D % E
  $M->bmodpow($self->{D},$self->{E});

  uc substr $M->as_hex,2;
}

1;
