# common startup for databug cgi's
package dbCGI::init;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw($CLIENT $PASS $DB $CLIENT_ID $PERMISSION logger getdevice getalias getclient);

use warnings;
use strict;
use Sys::Syslog qw(:DEFAULT setlogsock);
use CGI qw(param escapeHTML);
use DBI;

# global variables
our $CLIENT;   		# logged in client's name
our $PASS;     		# logged in password
our $DB;	     	# database handle
our $CLIENT_ID;		# authenticated client's database id
our $PERMISSION;	# authenticated client's permission

# write to databug log
sub logger(@)
{
  unshift @_,"ip=$ENV{REMOTE_ADDR}" if exists $ENV{REMOTE_ADDR}; 
  unshift @_,"client=$CLIENT" if $CLIENT; # unshift makes this first in the list
  warn("@_\n"), return if -t STDOUT;  
  setlogsock('unix');
  (my $self = $0) =~ s/.*\///;
  openlog($self,'pid','local5'); #local5 is databug log 
  syslog('info','%s',"@_"); 
  closelog();
}

# verify device id, return interesting into
# must be owned by current $CLIENT
sub getdevice($)
{
  my $device_id = shift;  
  my @a = $DB->selectrow_array("select device.deviceid, cpe.cpeid, cpe.id from device, cpe 
                                where device.id=? and device.client_id=? and device.cpe_id=cpe.id",undef,$device_id, $CLIENT_ID);
  defined $a[0] or die "Invalid device_id $device_id";
  @a;
}

# ditto for alias id
# must be owned by current $CLIENT
sub getalias($)
{
  my $alias_id = shift;
  my @a=$DB->selectrow_array("select alias.alias, device.deviceid, cpe.cpeid from alias, device, cpe
                              where device.client_id=? and alias.id=? and alias.device_id=device.id and device.cpe_id=cpe.id",
                              undef, $CLIENT_ID, $alias_id);
  defined $a[0] or die "invalid alias_id $alias_id";
  @a;
}

# ditto for client id
sub getclient($)
{
  my $client_id = shift;
  my @a=$DB->selectrow_array("select client, passwd, permission from client where id=?",undef,$client_id);
  defined $a[0] or die "invalid client_id $client_id";
  @a;
}

# this runs when the module is loaded
$CLIENT = lc((param 'client') || param 'user'); # logged in client's name, force lower case
$PASS = (param 'pass') || param 'pwd';          # logged in password

# connect to database
for (reverse 0..10) # try 10 times
{
  $DB = DBI->connect('DBI:mysql:databug:dbdb','databug',undef,{RaiseError => 1, AutoCommit=>1}) and last;
  $_ or die "can't connect to database\n";
  sleep 1; 
}

# ensure disconnect on exit
END { $DB and $DB->disconnect() }

# authenticate user
if ($CLIENT && $PASS) 
{
  my $system = $DB->selectrow_array("select passwd from client where client='system'") or die "no system password";
  if ($PASS eq $system)
  {
    $CLIENT_ID = $DB->selectrow_array("select id from client where client=?",undef, $CLIENT);
    $PERMISSION = 100;
  } else
  {
    # password in form x@ip.add.re.ss must match REMOTE HOST 
    $PASS !~ /\@(\d+\.\d+\.\d+\.\d+)$/ || $1 eq $ENV{REMOTE_ADDR} and 
    ($CLIENT_ID, $PERMISSION) = ($DB->selectrow_array("select id, permission from client where client=? and passwd=?",undef, $CLIENT, $PASS));
  }
  # for subsequent inclusion into forms
  $CLIENT = escapeHTML $CLIENT;
  $PASS = escapeHTML $PASS;
}

1; 
