#!/usr/bin/perl5.8.5 

# databug xml pull server
use warnings;
use strict;
use lib "/crux/dbug/pm";
use dbCGI::init;
use dbCGI::compile;

use CGI qw(param upload);
use DBI;

# abort http with error code
sub fail(@)
{
  print "Status: @_\n\n@_";
  exit 0;
}

$CLIENT_ID or logger("unauthorized"), fail("401 unauthorized");

# low-level non-gui cgi operations
my $getxml = param 'getxml';
if (defined $getxml)
{
  # return oldest xml as text/xml
  # 204 error if none
  # note filenames contain the timestamp such that the first alphabetical file is the oldest 
  #my $record = $DB->selectall_arrayref("select high_priority id, recordid, body from xml 
  my $record = $DB->selectall_arrayref("select id, recordid, body from xml 
                                        where deviceid like ? and state < 2 order by time limit 1000",undef,"$CLIENT.%"); 
  @$record or logger("getxml: no records"), fail("410 no files"); 

  print "Content-type: text/xml\n\n<?xml version='1.0' encoding='ISO-8859-1'?>\n<GETXML>\n";

  for (@$record)
  {
    chomp $_->[2];
    if ($getxml =~ /^index/i)
    {
      print "<DB index='$_->[1]'>\n$_->[2]\n</DB>\n";  
    } else
    {
      print "<DB.$_->[1]>\n$_->[2]\n</DB.$_->[1]>\n";
    } 
  } 
  print "</GETXML>\n";

  # now mark them all as sent
  my @records = map $_->[0], @$record;
  $DB->do("update xml set state=1 where id in (".(join ',',@records).")"); # don't quote the 'in' set
  @records < 1000 and $DB->do("update client set lastpoll=now() where id=?",undef,$CLIENT_ID);
  logger("getxml:",scalar @records,"records $records[0] to $records[$#records]");
  exit; # success!
}

if (defined param 'deletexml')
{
  # 'delete' retreived xml files
  my $rows=$DB->do('update xml set state=2 where deviceid like ? and state=1',undef,"$CLIENT.\%")+0; 
  logger("deletexml: $rows records");
  fail "200 OK"; # misnamed...
}

if (defined param 'putscript')
{
  # get parameters
  param 'file' or fail "400 missing file";

  my $cpeid = param 'cpeid' or fail "400 missing cpeid";
  my $cpe_id = $DB->selectrow_array("select id from cpe where cpeid=?",undef,$cpeid) or fail "400 bad cpeid"; 

  my $deviceid = param 'script' or fail "400 missing script"; # 'script' parameter is the device name
  $deviceid =~ tr/A-Z/a-z/;
  $deviceid =~ /[^-a-z0-9]/i and fail "400 invalid character in script parameter";

  # find specified device, create it if necessary
  my $device_id;
  $DB->do("lock tables device write"); # atomic test-and-create 
  unless ($device_id = $DB->selectrow_array("select id from device where client_id=? and deviceid=? and cpe_id=?",undef,$CLIENT_ID,$deviceid,$cpe_id))
  {
    # create it 
    $DB->do("insert device (deviceid, cpe_id, client_id) values (?,?,?)",undef,$deviceid,$cpe_id,$CLIENT_ID);
    $device_id = $DB->selectrow_array("select last_insert_id()");
  }
  $DB->do("unlock tables");
  # go compile
  local $/; my $handle=upload 'file';
  my @result=compile($device_id, <$handle>);
  $result[0] == 200 and logger "putscript updated '$deviceid:$cpeid'";
  fail @result; # misnomer if 200
}

if (defined param 'audit')
{
  my $start = param 'start' or fail "400 missing start";
  my $startid = $DB->selectrow_array("select id from xml where deviceid like ? and recordid=?", undef, "$CLIENT.%", $start) or fail "400 invalid recordid";
  my $records =  $DB->selectall_arrayref("recordid, state from xml where deviceid like ? and id >= ? limit 1000", undef, "$CLIENT.%", $startid);

  print "Content-type: text/xml\n\n<?xml version='1.0' encoding='ISO-8859-1'?>\n<AUDIT start='$start' records='".(scalar @$records.)"'>\n";
  print "  <RECORD id='$_->[0]' state='$_->[1]' />\n" for @$record;
  print "</AUDIT>\n";
  logger("audit:", scalar @$records, "records $records->[0] to $records->[$#$records]");
}

logger("invalid query");
fail "400 invalid"; 

