#!/usr/bin/perl
use warnings; use strict;
no utf8; use bytes;  

use HTTP::Request::Common qw(GET);
use LWP::UserAgent;
use XML::Parser;

use CGI qw(param escapeHTML);
use POSIX;

sub error($)
{
  "Sorry, @_.<p>Please try again later.";
}  

sub invalid($)
{
    "Sorry, @_.<p>Try a city and state, like 'San Jose CA' or 'Bismarck, North Dakota', or use a 5-digit zipcode.";
}    

sub lookup($)
{
    my $query=shift;
    $query =~/ / && $query !~/,/ and $query=~s/(.*) /$1, /;

    my $google = URI->new('http://maps.googleapis.com/maps/api/geocode/xml');
    $google->query_form('sensor'=>'false', 'address'=>"$query usa", 'region'=>'us');
    
    my $ua = LWP::UserAgent->new();
    my $rsp = $ua->get($google);
    
    $rsp->is_success or return error "the geocoding servers did not respond";

    my ($addr, $lat, $lng);
    my @stack=();
    my $state=0; # looking for first result, 1=found it, -1=past it 
    my $x = XML::Parser->new(Handlers=>{
        Start => sub {
            $state < 0 and return;
            my ($p, $e, %a)=@_;
            push @stack, $e;
        },
    
        Char => sub {
            $state < 0 and return; 
            my ($p, $t)=@_;
            "@stack" eq "GeocodeResponse result formatted_address" and $addr = $t, $state=1;
            "@stack" eq "GeocodeResponse result geometry location lat" and $lat = $t;
            "@stack" eq "GeocodeResponse result geometry location lng" and $lng = $t;
        },
    
        End => sub {
            $state < 0 and return;
            pop @stack;
            $state == 1 && "@stack" eq "GeocodeResponse" and $state=-1;
        }
    });    
    $x->parse($rsp->content);
     
    ($addr and $lat and $lng) or return invalid "the location you specified is invalid";
    $addr =~ s/, USA$// || $addr =~ s/, United States$// or return invalid "the NWS cannot provide a forecast for '$addr'";

    my $weather=URI->new('http://forecast.weather.gov/MapClick.php');
    # TextType 3 returns XML...
    #$weather->query_form(lat=>$lat,lon=>$lng,TextType=>'3');
    $weather->query_form(lat=>$lat,lon=>$lng,FcstType=>'xml');

    $rsp=$ua->get($weather);
    $rsp->is_success or return error "the NWS servers did not respond";

    my $out="";
    @stack=();
    $x = XML::Parser->new(Handlers=>{
        Start => sub {
            my ($p, $e, %a)=@_;
            push @stack, $e;
        },
    
        Char => sub {
            my ($p, $t)=@_;
            "@stack" eq "Forecast period valid" and $out .= "<p>".escapeHTML($t).": ";
            "@stack" eq "Forecast period text" and $out .= escapeHTML($t);
        },
    
        End => sub {
            pop @stack;
        }
    });    
    $x->parse($rsp->content);
     
    $out or return error "NWS does not have a forecast for '$addr' at this time";
    "NWS forecast for $addr: $out";
}  

my $where = param 'where';

my $content;
if ($where) { $content = lookup $where } 
       else { $content = '<form>Enter a location: <input type=text name=where size=40><input type=submit value="!OK" bind=SK1></form>' }
  
print <<EOT;
Content-type: text/html; charset="ISO-8859-1"

<html>
<head><title>National Weather Service</title>
</head>
<body>
$content
</body>
</html>
EOT
