#!/usr/bin/perl
# send arbitrary email
use warnings;
use strict;
use Mail::Mailer;
use Mail::Address;
use CGI qw(escapeHTML -oldstyle_urls Delete_all cookie);
use MIME::QuotedPrint;
use MIME::Words 'encode_mimewords';
use POSIX;
use bytes;

sub html(@)
{
    local $" = '';
    print "Content-type: text/html

           <html>
             <head>
               <meta http-equiv=pragma content=no-cache>
               <meta http-equiv=cache-control content=no-cache>
               <meta http-equiv='expires' content='-1'>
               <title>Mailer</title>
               <style type='text/css'> form {margin:0} pre{margin:0} table{margin:0}
                                       body {background-color:white; font-family:'Arial', sans-serif; color:black; font-size:12pt}
                                       table {font-family:'Arial', sans-serif; color:black; font-size:12pt}
                                      .title {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 150%; vertical-align: top}
                                      .head {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 100%; vertical-align: top}
               </style>
             </head>
             <body>
               <span class=title>Mailer</span><p>
               @_
             </body>
           </html>\n";
    exit;
}

# 'die' will show error to contact instead of barfing apache
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>",escapeHTML("@_"),"</pre>"};


# return unescaped parameter, or ''
sub param($)
{
    my $p = CGI::param(shift) || '';
    $p =~ s/[\x7f-\xa0]/ /g; # Avoid IE unpleasantness
    $p;
}

# wrap text like a MailBug
sub wrap($)
{
    my $s = shift;
    $s =~ s/[\r\t]//g;
    my $o;
    # get line if shorter than 79, or up to first lf, or up to last space, or block of 79
    $o .= "$+\n" while $s =~ /\G(.{0,78}?)\n|\G(.{1,79})$|\G(.{0,78})\s|\G(.{79}\n)|\G(.{79})/gs;
    $o =~ s/ +$//gm;
    $o;
}

# clean a string
sub clean($)
{
    my $s = shift;
    $s =~ s/^\s+|\s+$//g;
    $s =~ s/\s+/ /g;
    $s;
}

# parse addresses from string, return list of Mail::Address objects
sub addrs($)
{
    my @a=Mail::Address->parse(shift);
    my $c='[-_+.A-Za-z0-9]';
    $_->address() =~ /$c+\@$c+\.$c+/ or die $_->format()," is invalid\n" for @a;
    @a;
}

# get text block, clean ends of lines etc.
sub text($)
{
  my $s = shift;
  $s =~ s/\s+$//g;
  $s =~ s/[ \r\t]+$//mg;
  $s;
}

# collect interesting parameters, whether they exist or not
my $state=param 'state';
my @from = addrs param 'from';
my @to = addrs param 'to'; 
my $subject = clean param 'subject'; 
my @bcc = addrs param 'bcc';
my $message=text param 'message';

if (!$state)
{
    html "<form method=post>
         <table>
         <tr><td class=head>Subject<td><input type=text size=78 name=subject value='",escapeHTML($subject),"'>
         <tr><td class=head>From<td><input type=text size=78 name=from>
         <tr><td class=head>To<td><textarea name=to wrap=soft rows=4 cols=78></textarea>
         <tr><td class=head>Bcc<td><textarea name=bcc wrap=soft rows=4 cols=78></textarea>
         <tr><td class=head>Message<td><textarea name=message wrap=soft rows=30 cols=78>",escapeHTML(wrap $message),"</textarea>
         </table><br>
         <input type=hidden name=state value=1>
         <input type=submit value=OK>
         </form>";
}

# verify params exist
@from==1 or die "Must specify a single from address\n";
@to      or die "Must specify at least one to address\n";
$subject or die "Must specify a subject\n";
$message or die "Must specifiy a message\n";

if ($state eq "1")
{
    html "Verify message below, and click Send:
         <table>
         <tr><td class=head>Subject<td>",escapeHTML($subject),"
         <tr><td class=head>From<td>",escapeHTML($from[0]->format()),"
         <tr><td class=head>To<td>",join("<br>",map escapeHTML($_->format()), @to),"
         <tr><td class=head>Bcc<td>",join("<br>",map escapeHTML($_->format()), @bcc),"
         <tr><td class=head>Message<td><pre>",escapeHTML(wrap $message),"</pre>
         </table><br>
         <form method=post>
         <input type=hidden name=state value=2>
         <input type=hidden name=from value='",escapeHTML(param 'from'),"'>
         <input type=hidden name=to value='",escapeHTML(param 'to'),"'>
         <input type=hidden name=bcc value='",escapeHTML(param 'bcc'),"'>
         <input type=hidden name=subject value='",escapeHTML(param 'subject'),"'>
         <input type=hidden name=message value='",escapeHTML(param 'message'),"'>
         <input type=submit value=Send>
         </form>";
}

if ($state eq "2")
{
    # create headers
    my %headers=(From=>$from[0]->format(), 
                 To=>join(", ",map $_->format(), @to), 
                 Date=>POSIX::strftime("%a, %d %b %Y %H:%M:%S %z", localtime),
                 Subject=>encode_mimewords($subject),
                 'MIME-Version'=>'1.0',
                 'Content-Type'=>'text/plain; charset=ISO-8859-1',
                 'Content-Transfer-Encoding'=>'quoted-printable',
                ); 
                
    # with bcc if any
    @bcc and $headers{Bcc}=join(",", map $_->format(), @bcc);

    my $m=new Mail::Mailer('sendmail','-bm',                                # send from stdin
                                      '-f'.$from[0]->address(),             # set return path
                                      '-ODeliveryMode=deferred',            # queue without dns lookup
                                      '-OErrorMode=mail',                   # mail back errors
                                      '-OIgnoreDots')                       # ignore lone dot
                            ->open(\%headers);                              # open

    $m or die("can't open sendmail\n");                                     # complain if errors
    print $m encode_qp $message or die("can't write sendmail\n");
    $m->close;                                                              # ignore close error

    html "Message has been sent (please don't go 'BACK' from here or you'll send it again!)
         <form method=post>
         <input type=hidden name=state value=0>
         <input type=hidden name=from value='",escapeHTML(param 'from'),"'>
         <input type=hidden name=subject value='",escapeHTML(param 'subject'),"'>
         <input type=hidden name=message value='",escapeHTML(param 'message'),"'>
         <input type=submit value=OK>
         </form>";
} 

die "Invalid state $state\n";

