#!/usr/bin/perl 

# mailbug toll-free access manager

use warnings;
use strict;
use CGI;
use DBI;

# get specified parameter with bogus characters deleted
sub param($)
{
  my $p = CGI::param(shift);
  defined $p and $p =~ s/[\x7f-\xa0]/ /g;
  $p;
}  

# get http parameter 
sub supplied($)
{
  my $p = param $_[0]; 
  defined $p or die "Missing parameter '$_[0]'\n";
  $p;
}  

# output text as html 
sub html(@)
{
  print "Content-type: text/html

         <html>
         <head>
           <title>MailBug Access Manager</title>
           <meta http-equiv=pragma content=no-cache> 
           <meta http-equiv=cache-control content=no-cache>
           <meta http-equiv='expires' content='-1'>
           <style type='text/css'> 
              form {margin:0} pre{margin:0} table{margin:0}
              body {background-color:white; font-family:'Tahoma', sans-serif; color:black; font-size:16pt}
              table {font-family:'Tahoma', sans-serif; color:black; font-size:12pt}
              input {vertical-align: middle}

              .data {font-family:'Courier New', monospace; font-size:100%}
              .head {font-family:'Tahoma', sans-serif; font-weight: bold; color:black; font-sze: 100%; vertical-align: top}
              .note {font-family:'Tahoma', sans-serif; font-style: italic; color:black; font-size: 80%; vertical-align: bottom}
           </style>
         </head><body>
         <b>MailBug Toll-Free Access Manager</b><hr>
         @_</body></html>";
  exit;
}

# return a form string 
sub form($$;%)
{
  my ($text, $button, %hidden) = @_;
  $text or $text="";
  my $wide = $text ne ""?"":"style='width:100%'";
  $button or $button="OK";
  my $hidden=""; $hidden .= "<input type=hidden name='$_' value='$hidden{$_}'>" for keys %hidden;
  "<form>$text<input $wide type=submit value='$button'>$hidden</form>";
}

# database handling
my $DB = DBI->connect('DBI:mysql:mailbug:mbdb','mailbug',undef,{RaiseError => 1, AutoCommit=>1}) or die "can't connect to database\n";

# 'die' will show error to user, doesn't barf apache  
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

# find list of interesting domains
my %domain;
$domain{$_->[0]}=$_->[1] for @{$DB->selectall_arrayref("select a.id, a.domain
                                                        from domains as a left join domains as b on a.parent=b.id 
                                                        where a.domain='mailbug.com' or b.domain='mailbug.com'")};


my %states = (
        
  default => sub {
    html"<form method=post style='vertical-align: middle'>
         MailBug username: <input type=text size=20 name=user style='vertical-align: middle'>@<select name=domain_id style='vertical-align: middle'>".
         (join '', map "<option value=$_".(($domain{$_} eq 'mailbug.com')?' selected':'').">$domain{$_}</option>", 
         sort {$domain{$a} cmp $domain{$b}} keys %domain)."</select>
         <input type=submit value=OK> 
         <input type=hidden name=state value=lookup>
         </form>";
  },
        
  lookup => sub {
    my $user = lc param 'user'; $user =~ s/[^-._a-z0-9]//g; $user or die "Must specify a username.\n";
    my $domain_id = supplied 'domain_id'; $domain{$domain_id} or die "invalid domain_id $domain_id\n";
    
    my $u .= "$user@$domain{$domain_id}"; # true for mailbug.com and descendents
    my $exists = $DB->selectrow_array('select 1 from mblog where user=? and domain_id=? and error="ok" limit 1',undef,$u,$domain_id);
    my $n = $DB->selectrow_array('select count(*) from triggers where user=? and domain_id=? and phone="800303217"', undef, $u, $domain_id);
    if ($n > 1) die "Sorry, there are multiple toll-free triggers for $user (call Bryan or Rich)\n";
    my ($phone_trigger_id) = $DB->selectrow_array('select id from triggers where user=? and domain_id=? and phone="8003032107"', undef, $u, $domain_id);
    $n = $DB->selectrow_array('select count(*) from triggers where user=? and domain_id=? and allow_fax is not null', undef, $u, $domain_id);
    if ($n > 1) die "Sorry, there are multiple allow_fax triggers for $user (call Bryan or Rich)\n";
    my ($fax_trigger_id, $allow_fax) = $DB->selectrow_array('select id, allow_fax from triggers where user=? and domain_id=? and allow_fax is not null', undef, $u, $domain_id);

    $phone_trigger_id ||= 0;
    $fax_trigger_id ||= 0;
    $allow_fax = int($allow_fax/5)*5;
    $allow_fax ||= 5;
    
    my $html = "<b>Access for $u</b></p>";

    if (!$exists) $html = "<i>Warning: '$u' does not appear in the session
    log. If this is an established user then you have entered the name
    incorrectly, please go back and fix it.</i><p>";

    $html .= "<form method=post><table><tr><td>Toll-free access?<td>
                                               Yes<input type=radio name=tollfree value=1".($phone_trigger_id?" checked":"")."> 
                                               No<input type=radio name=tollfree value=0".($phone_trigger_id?"":" checked").">
                                       <tr><td>Max fax pages<td>".join('', map "<option value=$_."(($_=$allow_fax)?' selected':'').">$_</option>".map($_*5, 1..30))."</select>
                                </table>
                                <input type=submit value=OK>
                                <input type=hidden name=state value=update>
                                <input type=hidden name=user value='".escapeHTML($user).">
                                <input type=hidden name=domain_id value='".escapeHTML($domain_id).">
                                </form>";     
    html $html;
  },

  update = sub {
    my $user = lc param 'user'; $user =~ s/[^-._a-z0-9]//g; $user or die "Must specify a username.\n";
    my $domain_id = supplied 'domain_id';  $domain{$domain_id} or die "invalid domain_id $domain_id\n";
    my $tollfree = param 'tollfree';
    my $allow_fax = param 'allow_fax';
    
    $allow_fax = int($allow_fax/5)*5;
    $allow_fax ||=5;
    
    my $u .= "$user@$domain{$domain_id}"; # true for mailbug.com and descendents
    my ($fax_trigger_id, $allow_fax) = $DB->selectrow_array('select id, allow_fax from triggers where user=? and domain_id=? and allow_fax is not null', undef, $user, $domain_id);
    my $n = $DB->selectrow_array('select count(*) from triggers where user=? and domain_id=? and phone="800303217"', undef, $u, $domain_id);
    if ($n > 1) die "Sorry, there are multiple toll-free triggers for $user (call Bryan or Rich)\n";
    my ($phone_trigger_id) = $DB->selectrow_array('select id from triggers where user=? and domain_id=? and phone="8003032107"', undef, $u, $domain_id);
    $n = $DB->selectrow_array('select count(*) from triggers where user=? and domain_id=? and allow_fax is not null', undef, $u, $domain_id);
    if ($n > 1) die "Sorry, there are multiple allow_fax triggers for $user (call Bryan or Rich)\n";
    my ($fax_trigger_id, $allow_fax) = $DB->selectrow_array('select id, allow_fax from triggers where user=? and domain_id=? and allow_fax is not null', undef, $u, $domain_id);
    
    if ($phone_trigger_id && !$tollfree) { $DB->do('delete from triggers where id = ?', $phone_trigger_id 
    
    
  
    
  
my $action = supplied 'action';

my $who = $user.'@'.$domain{$domain_id};

$DB->do("lock tables triggers write");

my ($trigger_id) = $DB->selectrow_array("select id, phone from triggers where user=? and domain_id=? and phone='8003032107'", undef, $who, $domain_id);

if ($action eq 'Allow')
{
  $trigger_id and html form "<b>$who</b> already has toll-free access.", "OK";
  $DB->do("insert triggers set user=?, domain_id=?, disable=0, phone='8003032107', message=''",undef, $who, $domain_id); 
  $DB->do("unlock tables");
  html form "<b>$who</b> toll-free access is now allowed.","OK";
}  
    
if ($action eq 'Disallow')
{
  $trigger_id or html form "<b>$who</b> does not currently have toll-free access.", "OK";
  $DB->do("delete from triggers where id = ?",undef,$trigger_id);
  $DB->do("unlock tables");
  html form "<b>$who</b> toll-free access is no longer allowed.","OK";
}  
  
die "Invalid action '$action'\n";
