#!/usr/bin/perl
# mailbug email support
use warnings;
use strict;
use CGI qw(param escapeHTML self_url -oldstyle_urls Delete_all cookie);
use DBI;
use POSIX;

# get text block, clean ends of lines etc.
sub text($)
{
  my $s = shift; defined $s or $s = '';
  $s =~ s/\s+$//g;
  $s =~ s/[ \r\t]+$//mg;
  $s;
}
                         
sub need($) { my $p = param $_[0] or die "'$_[0]' is missing\n"; 
              $p =~ s/\s+$//; $p ne '' or die "'$_[0]' is empty\n"; 
              $p 
            }

my $HTTP = "Content-type: text/html\n";

my $HEAD = ""; # = "<meta http-equiv=pragma content=no-cache> 
               #    <meta http-equiv=cache-control content=no-cache>
               #    <meta http-equiv='expires' content='-1'>";

sub html(@)
{
  local $" = '';
  print "$HTTP\n
         <html>
           <head>
             $HEAD
             <title>FAQS</title>
             <style type='text/css'> form {margin:0} pre{margin:0} table{margin:0}
                                     body {background-color:white; font-family:'Arial', sans-serif; color:black; font-size:12pt}
                                     table {font-family:'Arial', sans-serif; color:black; font-size:12pt}

                                      
                                    .title {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 150%; vertical-align: top}
                                    .subtitle {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 110%; vertical-align: top}
                                    .data {font-family:'Courier New', monospace; font-size:100%}
                                    .head {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 100%; vertical-align: top}
             </style>
           </head> 
           <body> 
             <span class=title>MailBug FAQs</span><p> 
             @_ 
           </body>
         </html>\n";
  exit;
}

# create footer with multiple elements
sub foot(@)
{
  '<hr>'.join ' &nbsp; ',@_;
}  

# wrap text like a MailBug
sub wrap($)
{
  my $s = shift;
  $s =~ s/[\r\t]//g;
  my $o;
  # get line if shorter than 79, or up to first lf, or up to last space, or block of 79 
  $o .= "$+\n" while $s =~ /\G(.{0,78}?)\n|\G(.{1,79})$|\G(.{0,78})\s|\G(.{79}\n)|\G(.{79})/gs;
  $o =~ s/ +$//gm;
  escapeHTML $o;
}

# return url current page with given parameters
sub url(;@)
{
  Delete_all;
  param splice(@_,0,2) while @_ > 1;
  self_url;
}  

# return link with specified text and parameters
sub href($;@)
{
  my $text=escapeHTML(shift);
  '<a href='.url(@_).">$text</a>";
}  

sub nparam($)
{
  my $p = param $_[0];
  no warnings; $p+=0; use warnings;
  $p;
} 

# 'die' will show error to contact instead of barfing apache 
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>"};

# same as above, for recoverable errors
sub err { html "Sorry, an error has occurred:<p><pre>@_</pre>",foot(href('Show Open Threads')) };


# database handle
my $DB;


my %states = (
  top=>sub
  {
    html map(href($_->[1],state=>'view',id=>$_->[0]).'<br>',@{$DB->selectall_arrayref('select id, title from faqs order by title')});
  },
  
  view=>sub 
  {
    my $id = need 'id';
    my ($title, $body) = $DB->selectrow_array('select title, body from faqs where id = ?',undef,$id);
    defined $title or err "Invalid faq $id";
    html '<span class=subtitle>',escapeHTML($title),'</span><p><pre>'.wrap($body).'</pre>';
  }, 
  
  # administrative stuff

  'admin' => sub
  {
    html join '',
         "Faqs (available for use during reply)
         <table border=1 rules=rows>",
                               map('<tr><td>'.$_->[1].
                               '<td>'.href('Edit',state=>'editfaq',id=>$_->[0]).
                               '<td>'.href('Delete',state=>'delfaq',id=>$_->[0]),
                            @{$DB->selectall_arrayref('select id, title from faqs order by title')}),'</table><p>',
         foot href('Add FAQ',state=>'editfaq');
  },

  'editfaq' => sub
  {
    my $id=nparam 'id'; my $really = nparam 'really';

    unless ($really)
    {
      my ($title, $body)=('','');
      $id and ($title, $body)=$DB->selectrow_array('select title, body from faqs where id = ?',undef,$id);
      defined $title or die "Invalid faq $id";
    
      html "<form method=post>
            Edit FAQ and click ok.<p> 
            Title: <input type=text size=80 name=title value='".escapeHTML($title)."'><br>
            <textarea name=body wrap=soft rows=20 cols=75>".escapeHTML($body)."</textarea>
            <input type=submit value='OK'>
            <input type=hidden name=really value=1>
            <input type=hidden name=id value=$id>
            <input type=hidden name=state value=editfaq>
            </form>";
     }       

     # update faq
     my $title = text need 'title';
     my $body = text need 'body';
     $DB->do('lock tables faqs write');
     if ($id)
     {
       $DB->do('update faqs set title=?, body=? where id=?',undef,$title,$body,$id);
     } else
     {  
       $DB->do('insert faqs set title=?, body=?',undef,$title, $body);
     }  
     $DB->do('unlock tables');
     (state=>'admin');
   },              

  delfaq => sub
  {
    my $id=need 'id'; my $really = nparam 'really';
    $really or html 'Really delete faq?', foot(href('No',state=>'admin'),href('Yes',state=>'delfaq', really=>1, id=>$id)); 
    $DB->do('delete from faqs where id=?',undef,$id);
     (state=>'admin');
  },  
); # end of state table

my $state = param 'state';
my %next=(state=>'top');
if (defined $state)
{
  $states{$state} or die "invalid state $state\n";
  
  for (reverse 0..10) # try 10 times
  {
    $DB = DBI->connect('DBI:mysql:support:mbdb','support',undef,{RaiseError => 1, AutoCommit=>1}) and last;
    $_ or die "can't connect to database\n";
    sleep 1; 
  }
  
  %next=$states{$state}->();
}


# on exit or if no state, refresh top
$HEAD .= "<meta http-equiv='refresh' content='0;url=".url(%next)."'>\n";
html 'Please wait...';

