#!/usr/bin/perl 

use strict;
use LWP::UserAgent;
use HTTP::Response;
use HTTP::Request;

{
  package XRC;

  sub new
  {
    my $class = shift;

    my $self = { distID => shift,
                 passwd => shift,
                 url => 'http://xrc.everyone.net/ccc/xrc', 
               };
    bless $self, $class;
    $self;
  }  

  # convert arguments to xrc list
  sub list
  {
    shift;
    "( @_ )";
  }

  # convert arguments to xrc string
  sub string
  {
    shift;
    s/\\/\\\\/g, s/"/\\"/g for @_;
    @_;
  }

  # decode xrc response 
  my $decode;
  sub decode
  {
    shift;
    $decode = shift;
    my $nest = 0;
    my @result; 

    while(length $decode)
    {
      $nest < 0 and die "xrc is malformed\n";
      s/^(\s*// and $nest++, next;
      if (s/^)\s*//) 
      {
        $nest-- or die "Unexpected ) in xrc\n";

    
      if (s/^((?:\\\\|\\"|[^"])*)"\s*$//s)
      {
        (my $s = $1) =~ s/\\(.)/$1/sg;
        push @result, $s;
        next;
      }
    }
    

  # turn any method into XRC call, return verbatim response
  sub AUTOLOAD 
  {
    our $AUTOLOAD; # what are we trying to load?
    (my $method = $AUTOLOAD) =~ s/.*://;
    $method eq 'DESTROY' and exit;
    
    my $self = shift;
    my $request = HTTP::Request->new(POST => $self->{url});
    $request->content_type('application/x-eon-xrc-request');
    my $content = "version: 1\nclientID: ".$self->{distID}."\npassword: ".$self->{passwd}."\n\n$method( @_ )\n";
    $request->content($content);
    my ($head, $body) = split /\n *\n/, LWP::UserAgent->new->request($request)->content(), 2;
    my %rsp;
    /(.+):[ \t]+(.+)\s*/ and $rsp{$1}=$2 for split /[ \r\t]*\n/, $head; 
    defined $rsp{status} && $rsp{status}==0 or die "XRC ERROR:\nQuery:$method ( @_ )\n$head\n";
    $body =~ s/^\s+|\s+$//gs; $body;
  }
}  

# XRC commands
# createEmailAlias ( clientID : int, name : string, recipients : list ) : void
# createEmailClient ( offerIDs : list, emailDomain : string, webmailHostname : string ) : int
# createUser ( clientID : int, offerIDs : list, username : string, password : string ) : int
# deleteClient ( clientID : int ) : void
# deleteEmailAlias ( clientID : int, name : string ) : void
# deleteUser ( clientID : int, username : string ) : void
# getClientOffers ( clientID : int ) : list
# getClientPassword ( clientID : int ) : string
# getEmailAliasRecipients ( clientID : int, name : string ) : List
# getEmailServiceCatchAll ( clientID : int ) : string
# getQuotaWarningLetter ( clientID : int ) : Letter
# getUnreadCountForUserMailbox ( clientID : int, username : string ) : int
# getUserOffers ( clientID : int, username : string ) : list
# getUserPassword ( clientID : int, username : string ) : string
# getWebmailPresentation ( clientID : int ) : WebmailPresentation
# getWelcomeLetter ( clientID : int ) : Letter
# isAccountNameAvailable ( clientID : int, name : string ) : boolean
# listClientIDsOfDistributor ( ) : list
# listEmailAliasInfoOfClient ( clientID : int ) : List
# listUserNamesOfClient ( clientID : int ) : List
# loginWebmail ( clientID : int, username : string ) : string
# logoutWebmail ( cv : string ) : void
# lookupMXReadyClientIDByEmailDomain ( emailDomain : string ) : int
# noop ( ) : void
# purgeSuspendedUserAccount ( clientID : intm username : string ) : void
# setClientOffers ( clientID : int, offerIDs : List ) : void
# setClientPassword ( clientID : int, password : string ) : void
# setEmailAliasRecipients ( clientID : int, name : string, recipients : list ) : void
# setEmailServiceCatchAll ( clientID : int, catchall : string ) : void
# setQuotaWarningLetter ( clientID : int, letter : Letter ) : void
# setUserOffers ( clientID : int, username : string, offerIDs : List ) : void
# setUserPassword ( clientID : int, username : string, password : string ) : void
# setWebmailLogos ( clientID : int, smallLogoImage : bytes, largeLogoImage : bytes ) : void
# setWebmailNavbarImages ( clientID : int, inboxImage : bytes, composeImage : bytes, addressesImage : bytes, foldersImage : bytes, searchImage : bytes,
#                          optionsImage : bytes, helpImage : bytes, logoutImage : bytes ) : void
# setWebmailPresentation ( clientID : int, presentation : WebmailPresentation ) : void
# setWelcomeLetter ( clientID : int, letter : Letter ) : void
# summarizeEmailClient ( clientID : int ) : EmailClientSummary
# summarizeEmailClientsOfDistributor ( ) : List
# suspendUser ( clientID : int, username : string ) : void
# unsuspendUser ( clientID : int, username : string ) : void                

my $xrc = new XRC('1544191', 'password');
print "Client IDs: @{[$xrc->listClientIDsOfDistributor()]}\n";
print "Client Password: ",$xrc->getClientPassword('1546498'),"\n";
print "Client QWL: ",$xrc->getQuotaWarningLetter('1546498'),"\n";
print "Client Offers: @{[$xrc->getClientOffers('1546498')]}\n";
print "Client Usernames: @{[$xrc->listUserNamesOfClient('1546498')]}\n";
