#!/usr/bin/perl 
# show idle devices, allow delete
use warnings;
use strict;
use lib '/crux/dbug/pm';
use dbCGI::init;
use dbCGI::gui;
use CGI qw(param escapeHTML self_url -oldstyle_urls upload);
use DBI;


# 'die' will show error to user instead of barfing apache  
local $SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

#$PERMISSION > 1 or die "Forbidden\n";

# mysql> desc message;
# +------------+------------+------+-----+---------+----------------+
# | Field      | Type       | Null | Key | Default | Extra          |
# +------------+------------+------+-----+---------+----------------+
# | id         | bigint(20) |      | PRI | NULL    | auto_increment |
# | deviceid   | tinytext   |      | MUL |         |                |
# | msg        | blob       |      |     |         |                |
# | updated    | datetime   | YES  |     | NULL    |                |
# | downloaded | datetime   | YES  |     | NULL    |                |
# +------------+------------+------+-----+---------+----------------+

# for 'post' method
sub fail(@) { print "Status: @_\n\n@_"; exit 0; }

# this is the state table, a named array of anonymous subroutines
my %states;
%states = ( 

top => sub  
{
  html "<table cellpadding=0 cellspacing=0 border=0>  
        <tr><td>  
	<form method=get>
        <input type=hidden name=client value='$CLIENT'>
        <input type=hidden name=pass value='$PASS'>
        <input type=hidden name=state value='edit'>
        <tr><td>Specify message for unit ".(($CLIENT ne 'system')?"$CLIENT.":'')."<input type=text name=suffix>
        <td valign=bottom>
        <input type=submit value='OK'>    
        </form>
        <td valign=bottom>
        <form action='dbman.cgi' method=get>
        <input type=submit value='Done'>
        <input type=hidden name=client value='$CLIENT'>
        <input type=hidden name=pass value='$PASS'>
        </form>
        </table>";
},
  
edit => sub
{
  my $suffix = param('suffix') or die "Must specify a unit";
  my $deviceid=$suffix;
  $CLIENT ne 'system' and $deviceid = "$CLIENT.$suffix";

  $DB->selectrow_array('select 1 from session where deviceid = ? limit 1', undef, $deviceid) or die "Sorry, no record for '$deviceid' in the last 90 days.\n";
  my ($id, $msg, $updated, $downloaded) = $DB->selectrow_array('select id, msg, updated, downloaded from message where deviceid = ?', undef, $deviceid);
  $updated ||= "never";
  $downloaded ||= "never";
  my $html .= 
    "<form method=get>
     <input type=hidden name=client value='$CLIENT'>
     <input type=hidden name=pass value='$PASS'>
     <table border=1 cellpadding=3 cellspacing=0>
       <tr><td>Unit<td>".(escapeHTML $deviceid).
    "  <tr><td>Last Updated<td>$updated
       <tr><td>Last Downloaded<td>$downloaded
       <tr><td valign=top>Message<td><textarea name=msg rows=6 cols=80>".(escapeHTML $msg)."</textarea>
     </table>  
     <hr>
     <table cellpadding=0 cellspacing=0>
       <tr><td><input type=submit name=state value='Update'>";
  $id and $html .=        
    "      <td><input type=submit name=state value='Delete'>
               <input type=hidden name=id value='$id'>";
  $html .= 
    "      <td><input type=submit value='Back'>
     </table>   
     <input type=hidden name=suffix value='".(escapeHTML $suffix)."'>
     <input type=hidden name=deviceid value='".(escapeHTML $deviceid)."'>
     </form>";

  html $html;
},

Update => sub
{
  my $deviceid = param('deviceid') or die "Missing deviceid in update\n";

  (my $msg = (param 'msg') || '') =~ s/\s+$//gm;
  
  length $msg > 1023 and die "Message cannot exceed 1023 characters\n";

  my $id = param 'id';
  
  if ($id)
  {
    $DB->do("update message set msg=?, updated=now(), downloaded=NULL where id=?", undef, $msg, $id);    
  } else 
  {
    # make sure it's legal
    $DB->selectrow_array('select 1 from session where deviceid = ? limit 1', undef, $deviceid) or die "Sorry, no record for '$deviceid' in the last 90 days.\n";
    $DB->do("insert into message set deviceid=?, msg=?, updated=now(), downloaded=NULL", undef, $deviceid, $msg);
  }
  logger "Updated message for $deviceid";
  $states{'edit'}->();
},

Delete => sub
{
	
  my $id = param 'id' or die "Missing ID in delete\n";
  my $deviceid = param 'deviceid' or die "Missing deviceid in delete\n";
  $DB->do('delete from message where id = ?', undef, $id);
  logger "Deleted message for $deviceid (id=$id)";
},

# automated message post

post => sub
{
    my $unit = param 'unit' or fail "400 missing unit";

    my $deviceid = "$CLIENT.$unit";
    $DB->selectrow_array('select 1 from session where deviceid = ? limit 1', undef, $deviceid) or fail "400 no record for $deviceid";

    (my $msg = (param 'msg') || '') =~ s/\s+$//gm;
    length $msg > 1023 and fail "400 message exceeds 1023 chars";

    $DB->do('lock tables message write');
    my $id = $DB->selectrow_array('select id from message where deviceid = ?', undef, $deviceid);
    if ($id)
    {
	if (length $msg) 
        {
           $DB->do("update message set msg=?, updated=now(), downloaded=NULL where id=?", undef, $msg, $id);    
	   $DB->do('unlock tables');
           logger "updated message $id for $deviceid";
        } else
        {
	   $DB->do("delete from message where id = ?", undef, $id);
    	   $DB->do('unlock tables');
           logger "delete message $id for $deviceid";
        }
    }
    else 
    {
      if (length $msg)
      {
        $DB->do("insert into message set deviceid=?, msg=?, updated=now(), downloaded=NULL", undef, $deviceid, $msg);
        $DB->do('unlock tables');
        logger "create message for $deviceid";
      } else
      { 
      	$DB->do('unlock tables');
	logger "ignoring empty message for $deviceid";
      }
    }
    fail "200 OK"
}

);

defined $PERMISSION or fail "401 unauthorized";

# here, process submitted state
my $n = param 'state';

if (defined $n)							# if exists
{
  die "Illegal state '$n'\n" unless exists $states{$n};		# and valid	
  $states{$n}->();						# maybe it returns...
}

param state=>'top';						# refresh to 'top'
$HEAD .= "<meta http-equiv='refresh' content='0;url=".self_url."'>";
html "Please wait...";

