#!/usr/bin/perl
# databug session log 
use warnings;
use strict;
use lib '/crux/dbug/pm';
use dbCGI::init;
use dbCGI::gui;
use CGI qw(param escapeHTML self_url -oldstyle_urls);
use Time::HiRes;

my $start = Time::HiRes::time;

# 'die' will show error to user instead of barfing apache  
local $SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

$PERMISSION > 1 or die "Forbidden\n";

unless (param 'state')
{
  param 'state'=>'top';
  $HEAD .= "<meta http-equiv='refresh' content='0;url=".self_url."'>";
  html "Please wait...";
}

my @filter;

# create filter string
my $duration = (param 'duration') || 0; 
$duration =~ /\D/ and die "Non-digit in 'Duration' filter\n";  
my $durop = (param 'durop') || '>=';
$duration != 0 || $durop ne '>=' and push @filter, "duration $durop ".$DB->quote($duration);

my $records = (param 'records') || 0; 
$records =~ /\D/ and die "Non-digit in 'Records' filter\n";  
my $recop = (param 'recop') || '>=';
$records != 0 || $recop ne '>=' and push @filter, "records $recop ".$DB->quote($records);

my ($ayear, $amonth, $aday, $ahour, $aminute, $asecond) = timestamp 0,qw{ayear amonth aday ahour aminute asecond};
my ($byear, $bmonth, $bday, $bhour, $bminute, $bsecond) = timestamp 1,qw{byear bmonth bday bhour bminute bsecond};

my $after = defined param 'after' and push @filter, "time >= '$ayear-$amonth-$aday $ahour:$aminute:$asecond'";
my $before = defined param 'before' and push @filter,"time <= '$byear-$bmonth-$bday $bhour:$bminute:$bsecond'";

my $not;
my $deviceid = param('deviceid')||'';
if ($CLIENT ne 'system')
{
  my $s = "$CLIENT.".(($deviceid ne '')?$deviceid:'%');
  push @filter, "deviceid like ".$DB->quote($s);
} elsif ($deviceid ne '')
{
  my $s = $deviceid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter, "deviceid $not like ".$DB->quote($s);
} 

my $cpeid = (param 'cpeid') || '';  
if ($cpeid ne '')
{
  my $s = $cpeid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter, "cpeid $not like ".$DB->quote($s);
}

my $scriptid = (param 'scriptid') || ''; 
if ($scriptid ne '')
{
  my $s = $scriptid;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter,"scriptid $not like ".$DB->quote($s);
}

my $phone = (param 'phone') || '';
if ($phone ne '')
{
  $phone =~ /[^!0-9*#%]/ and die  "Invalid character in 'Phone' filter\n";
  my $s = $phone;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter,"phone $not like ".$DB->quote($s);
}

my $remotehost = (param 'remotehost') || '';
if ($remotehost ne '')
{
  $remotehost =~ /[^!0-9.%]/ and die "Invalid character in 'IPt' filter\n";  
  my $s = $remotehost;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter, "remotehost $not like ".$DB->quote($s);
}

my $result = (param 'result') || '';
if ($result ne '')
{ 
  my $s = $result;
  my $not = ($s =~ s/^!//)?"not":"";
  $s ne '' and push @filter, "result $not like ".$DB->quote($s);
}

my $sessions = param 'sessions'; defined $sessions or $sessions = ''; 
$sessions =~ /\D/ and die "Non-digit in 'Sessions' filter\n";  
my $sessions_op = (param 'sessions_op') || '>=';
$sessions ne '' and push @filter, "sessions $sessions_op $sessions";

my $halted = param 'halted'; defined $halted or $halted = '';
$halted =~ /\D/ and die "Non-digit in 'Halted' filter\n";  
my $halted_op = (param 'halted_op') || '>=';
$halted ne '' and push @filter, "halted $halted_op $halted";

my $busy = param 'busy'; defined $busy or $busy = ''; 
$busy =~ /\D/ and die "Non-digit in 'Busy' filter\n";  
my $busy_op = (param 'busy_op') || '>=';
$busy ne '' and push @filter, "busy $busy_op $busy";

my $nocarrier = param 'nocarrier'; defined $nocarrier or $nocarrier = ''; 
$nocarrier =~ /\D/ and die "Non-digit in 'No Carrier' filter\n";  
my $nocarrier_op = (param 'nocarrier_op') || '>=';
$nocarrier ne '' and push @filter, "nocarrier $nocarrier_op $nocarrier";

my $dropped = param 'dropped'; defined $dropped or $dropped = ''; 
$dropped =~ /\D/ and die "Non-digit in 'Dropped' filter\n";  
my $dropped_op = (param 'dropped_op') || '>=';
$dropped ne '' and push @filter, "dropped $dropped_op $dropped";

my $link = param 'link'; defined $link or $link = ''; 
$link =~ /\D/ and die "Non-digit in 'Link' filter\n";  
my $link_op = (param 'link_op') || '>=';
$link ne '' and push @filter, "link $link_op $link";

my $chat = param 'chat'; defined $chat or $chat = ''; 
$chat =~ /\D/ and die "Non-digit in 'Chat' filter\n";  
my $chat_op = (param 'chat_op') || '>=';
$chat ne '' and push @filter, "chat $chat_op $chat";

my $timeout = param 'timeout'; defined $timeout or $timeout = '';
$timeout =~ /\D/ and die "Non-digit in 'Timeout' filter\n";  
my $timeout_op = (param 'timeout_op') || '>=';
$timeout ne '' and push @filter, "timeout $timeout_op $timeout";

my $lsp = param 'lsp'; defined $lsp or $lsp = '';
$lsp =~ /\D/ and die "Non-digit in 'LSP' filter\n";  
my $lsp_op = (param 'lsp_op') || '>=';
$lsp ne '' and push @filter, "lsp $lsp_op $lsp";

my $server = param 'server'; defined $server or $server = ''; 
$server =~ /\D/ and die "Non-digit in 'Server' filter\n";  
my $server_op = (param 'server_op') || '>=';
$server ne '' and push @filter, "server $server_op $server";

my $show=param('show') || "Last 10";

# build it
my $html .= "<form method=get>
                <table border=1 cellpadding=3 cellspacing=0>";

# device id
$html .= sprintf "<tr><td>Device&nbsp;ID<td>%s<input type=text size=40 name=deviceid value='%s'>",($CLIENT ne 'system')?"$CLIENT.":'',(escapeHTML $deviceid);

# time
$html .= sprintf '<td><input type=checkbox name=after%s>On or after<td>',$after?" checked":"";
$html .= "<select name=ayear>"; $html .= opt(sprintf("%.2d", $_), $_==$ayear) for 2000..2050; $html .= "</select>-";
$html .= "<select name=amonth>"; $html .= opt(sprintf("%.2d", $_), $_==$amonth) for 1..12; $html .= "</select>-";
$html .= "<select name=aday>"; $html .= opt(sprintf("%.2d", $_), $_==$aday) for 1..31; $html .= "</select>";
$html .= "&nbsp;<select name=ahour>"; $html .= opt(sprintf("%.2d", $_), $_==$ahour) for 0..23; $html .= "</select>:";
$html .= "<select name=aminute>"; $html .= opt(sprintf("%.2d", $_), $_==$aminute) for 0..59; $html .= "</select>:";
$html .= "<select name=asecond>"; $html .= opt(sprintf("%.2d", $_), $_==$asecond) for 0..59; $html .= "</select>";

# cpeid 
$html .= "<tr><td>CPE&nbsp;ID<td><input type=text size=40 name=cpeid value='".(escapeHTML $cpeid)."'>";

$html .= sprintf '<td><input type=checkbox name=before%s>On or before<td>',$before?" checked":"";
$html .= "<select name=byear>"; $html .= opt(sprintf("%.2d", $_), $_==$byear) for 2000..2050; $html .= "</select>-";
$html .= "<select name=bmonth>"; $html .= opt(sprintf("%.2d", $_), $_==$bmonth) for 1..12; $html .= "</select>-";
$html .= "<select name=bday>"; $html .= opt(sprintf("%.2d", $_), $_==$bday) for 1..31; $html .= "</select>";
$html .= "&nbsp;<select name=bhour>"; $html .= opt(sprintf("%.2d", $_), $_==$bhour) for 0..23; $html .= "</select>:";
$html .= "<select name=bminute>"; $html .= opt(sprintf("%.2d", $_), $_==$bminute) for 0..59; $html .= "</select>:";
$html .= "<select name=bsecond>"; $html .= opt(sprintf("%.2d", $_), $_==$bsecond) for 0..59; $html .= "</select>";

# scriptid
$html .= "<tr><td>Script&nbsp;ID<td><input type=text size=40 name=scriptid value='".(escapeHTML $scriptid)."'>";

# seconds
$html .= "<td>Seconds<td><select name=durop>".opt('&gt;=',$durop eq '>=').opt('&lt;',$durop eq '<').
         "</select><input type=text size=4 value=$duration name=duration> (per session)" ;

# records 
$html .= "<tr><td>Records<td><select name=recop>".opt('&gt;=',$recop eq '>=').opt('&lt;',$recop eq '<').
         "</select><input type=text size=4 value=$records name=records>";
# phone
$html .= "<td>Phone<td><input type=text size=40 name=phone value='$phone'>";

# remotehost
$html .= "<tr><td>IP<td><input type=text size=40 name=remotehost value='$remotehost'>";

# result 
$html .= "<td>Result<td><input type=text size=40 name=result value='".(escapeHTML $result)."'>";

# show
$html .= "<tr><td>Show<td><select name=show>";
$html .= opt($_, $_ eq $show) for "All", "Last 1", "Last 10", "Last 100", "First 1", "First 10", "First 100", "Summary"; 
$html .= "</select>";

$html .= "<td colspan=2>&nbsp;";

$html .= "</table><p><table border=1 cellpadding=2 cellspacing=0>";

# sessions  
$html .= "<tr><td class=head>Sessions<td><select name=sessions_op>"
	 .(join '',map opt($_,$sessions_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$sessions' name=sessions>" ;

# halted  
$html .= "<td class=head>Halted<td><select name=halted_op>"
	 .(join '',map opt($_,$halted_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$halted' name=halted>" ;

# busy  
$html .= "<td class=head>Busy<td><select name=busy_op>"
	 .(join '',map opt($_,$busy_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$busy' name=busy>" ;

# nocarrier  
$html .= "<td class=head>No Carrier<td><select name=nocarrier_op>"
	 .(join '',map opt($_,$nocarrier_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$nocarrier' name=nocarrier>" ;

# dropped  
$html .= "<td class=head>Dropped<td><select name=dropped_op>"
	 .(join '',map opt($_,$dropped_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$dropped' name=dropped>" ;

# link  
$html .= "<tr><td class=head>Link<td><select name=link_op>"
	 .(join '',map opt($_,$link_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$link' name=link>" ;

# chat  
$html .= "<td class=head>Chat<td><select name=chat_op>"
	 .(join '',map opt($_,$chat_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$chat' name=chat>" ;

# timeout  
$html .= "<td class=head>Timeout<td><select name=timeout_op>"
	 .(join '',map opt($_,$timeout_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$timeout' name=timeout>" ;

# lsp  
$html .= "<td class=head>Protocol<td><select name=lsp_op>"
	 .(join '',map opt($_,$lsp_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$lsp' name=lsp>" ;

# server  
$html .= "<td class=head>Abort<td><select name=server_op>"
 	 .(join '',map opt($_,$server_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$server' name=server>" ;

#
$html .= "</table><p>
          <table border=0>
	  <tr><td>
          <input type=submit value='Filter'>
          <input type=hidden name=client value='$CLIENT'>
          <input type=hidden name=pass value='$PASS'>
          </form>
          <td>
          <form>
          <input type=submit value='Reset'>
          <input type=hidden name=client value='$CLIENT'>
          <input type=hidden name=pass value='$PASS'>
          </form>
          <td>
	  <form action='dbman.cgi' method=get>
          <input type=submit value='Done'>
          <input type=hidden name=client value='$CLIENT'>
          <input type=hidden name=pass value='$PASS'>
          </form>
          </table>
          <p>";

$html .= "<table border=1 cellpadding=2 cellspacing=0><tr>";

my $query;
my $total = 0;

if ($show ne 'Summary')
{ 
  my $rev = 0;
  my $order = "order by id asc"; 

  if ($show =~ /^Last (10?0?)$/)
  {
    $order = "order by id desc limit $1";
    $rev = 1;
  } elsif ($show =~ /^First (10?0?)$/)
  { 
    $order = "order by id asc limit $1";
  }

  # list view, items to fetch
  my @items = qw(time deviceid cpeid scriptid phone remotehost duration records result sessions halted busy nocarrier dropped link chat timeout lsp server); 
  $query = "select high_priority ".join(",",@items)." from session ignore index (deviceid) ";
  @filter and $query .= " where ".join(" and ",@filter);
  $query .= " $order";
#  die $query;
  my $array = $DB->selectall_arrayref($query);
  $rev and $array= [reverse @$array]; 

  # headings for each item
  my %headings = (time=>'Time',deviceid=>'Device ID',cpeid=>'CPE ID',scriptid=>,'Script ID',duration=>'Seconds',
                  records=>'Records',phone=>'Phone',remotehost=>'IP', result=>'Result',
                  sessions=>'S', halted=>'H', busy=>'B', nocarrier=>'N', dropped=>'D', link=>'L', chat=>'C', timeout=>'T', lsp=>'P', server=>'A');
 
  # items to sum
  my %sum = (duration=>0, records=>0, sessions=>0, halted=>0, busy=>0, nocarrier=>0, dropped=>0, link=>0, chat=>0, timeout=>0, lsp=>0, server=>0);

  $html .= "<td class=data><b>$headings{$_}</b>" for @items;
  for my $ref (@$array)
  {
    $total++;        # sum records
    $html .= "<tr>";
    for (0..$#items)
    {
      $html .= sprintf("<td class=data>%s",($ref->[$_] eq "")?"&nbsp;":$ref->[$_]) if exists $headings{$items[$_]};
      $sum{$items[$_]}+=$ref->[$_] if exists $sum{$items[$_]};
    }
  }
  $html .= "<tr>";
  for (0..$#items)
  {
    $html .= sprintf("<td class=data><b>%s</b>",(exists $sum{$items[$_]})?$sum{$items[$_]}:"&nbsp;") if exists $headings{$items[$_]};
  }
  $html .= "</table>";
}
else
{
  # summary view items to fetch
  my @items = qw(count(id) deviceid cpeid phone remotehost sum(duration) sum(records));
  $sessions  ne '' and push @items, 'sum(sessions)';
  $halted ne '' and push @items, 'sum(halted)'; 
  $busy ne '' and push @items, 'sum(busy)'; 
  $nocarrier ne '' and push @items, 'sum(nocarrier)';
  $dropped ne '' and push @items, 'sum(dropped)';
  $link ne '' and push @items, 'sum(link)'; 
  $chat ne '' and push @items, 'sum(chat)';
  $timeout ne '' and push @items, 'sum(timeout)';
  $lsp ne '' and push @items, 'sum(lsp)';
  $server ne '' and push @items, 'sum(server)';
  my $query = "select high_priority ".join(",",@items)." from session where ".join(" and ",@filter)." group by deviceid, cpeid, phone order by deviceid";

  my $array = $DB->selectall_arrayref($query);

  # headings for each item
  my %headings = (deviceid=>'Device ID',cpeid=>'CPE ID','sum(duration)'=>'Seconds',phone=>'Phone',remotehost=>'IP',
		  'count(id)'=>"&nbsp;",'sum(records)'=>"Records",
                  'sum(sessions)'=>'S', 'sum(halted)'=>'H', 'sum(busy)'=>'B', 'sum(nocarrier)'=>'N', 'sum(dropped)'=>'D', 
                  'sum(link)'=>'L', 'sum(chat)'=>'C', 'sum(timeout)'=>'T', 'sum(lsp)'=>'P', 'sum(server)'=>'A'); 
 
  # items to sum
  my %sum = ('sum(duration)'=>0,'count(id)'=>0,'sum(records)'=>0,'sum(sessions)'=>0, 'sum(halted)'=>0, 'sum(busy)'=>0, 'sum(nocarrier)'=>0, 'sum(dropped)'=>0, 
             'sum(link)'=>0, 'sum(chat)'=>0, 'sum(timeout)'=>0, 'sum(lsp)'=>0, 'sum(server)'=>0);

  $html .= "<td class=data><b>$headings{$_}</b>" for @items;
  for my $ref (@$array)
  {
    $total++;        # sum records
    $html .= "<tr>";
    for (0..$#items)
    {
      $html .= sprintf("<td class=data>%s%s%s",$_?'':'<b>',($ref->[$_] eq "")?"&nbsp;":$ref->[$_],$_?'':'</b>') if exists $headings{$items[$_]};
      $sum{$items[$_]}+=$ref->[$_] if exists $sum{$items[$_]};
    }
  }
  $html .= "<tr>";
  for (0..$#items)
  {
    $html .= sprintf("<td class=data><b>%s</b>",(exists $sum{$items[$_]})?$sum{$items[$_]}:"&nbsp;") if exists $headings{$items[$_]};
  }
} 

#$html .= "</table><hr><div style='font-size: 7pt'>$query<br>Processing time: ".(sprintf "%.3f",Time::HiRes::time-$start)." seconds</div>";
$html .= "</table><hr><div style='font-size: 7pt'>Returned $total lines in ".(sprintf "%.3f",Time::HiRes::time-$start)." seconds</div>";

html $html;
