# LSP cryptosystem - stream cipher
package cipher;
use warnings;
use strict;

my %transform = (

     # transform legal LSP chars SOH=1, EOT=4, ACK=6, BS=8, TAB=9, LF=10,
     # CR=13, ESC=27, ASCII=32-127 to state index 0-103 and back
    lsp => {
        c2i => [ 
            undef, 0,     undef, undef, 1,     undef, undef, 2,     
            3,     4,     5,     undef, undef, 6,     undef, undef,
            undef, undef, undef, undef, undef, undef, undef, undef,
            undef, undef, undef, 7,     undef, undef, undef, undef, 
            8,     9,     10,    11,    12,    13,    14,    15,
            16,    17,    18,    19,    20,    21,    22,    23,  
            24,    25,    26,    27,    28,    29,    30,    31,
            32,    33,    34,    35,    36,    37,    38,    39,  
            40,    41,    42,    43,    44,    45,    46,    47,
            48,    49,    50,    51,    52,    53,    54,    55,  
            56,    57,    58,    59,    60,    61,    62,    63,
            64,    65,    66,    67,    68,    69,    70,    71,  
            72,    73,    74,    75,    76,    77,    78,    79,
            80,    81,    82,    83,    84,    85,    86,    87, 
            88,    89,    90,    91,    92,    93,    94,    95,
            96,    97,    98,    99,    100,   101,   102,   103 ],
        
        # convert state index 0-103 to LSP character
        i2c => [ 
            1,     4,     7,     8,     9,     10,    13,    27,   
            32,    33,    34,    35,    36,    37,    38,    39,
            40,    41,    42,    43,    44,    45,    46,    47,  
            48,    49,    50,    51,    52,    53,    54,    55,
            56,    57,    58,    59,    60,    61,    62,    63,  
            64,    65,    66,    67,    68,    69,    70,    71,
            72,    73,    74,    75,    76,    77,    78,    79,  
            80,    81,    82,    83,    84,    85,    86,    87,
            88,    89,    90,    91,    92,    93,    94,    95,  
            96,    97,    98,    99,    100,   101,   102,   103,
            104,   105,   106,   107,   108,   109,   110,   111, 
            112,   113,   114,   115,   116,   117,   118,   119,
            120,   121,   122,   123,   124,   125,   126,   127 ] 
    },


    # transform legal file file chars ASCII 32-126, ISO 160-255 to index 0-191
    # and back
    file => {
        c2i => [ 
            undef, undef, undef, undef, undef, undef, undef, undef, 
            undef, undef, 0,     undef, undef, undef, undef, undef,
            undef, undef, undef, undef, undef, undef, undef, undef, 
            undef, undef, undef, undef, undef, undef, undef, undef,
            1,     2,     3,     4,     5,     6,     7,     8,     
            9,     10,    11,    12,    13,    14,    15,    16,  
            17,    18,    19,    20,    21,    22,    23,    24,    
            25,    26,    27,    28,    29,    30,    31,    32,  
            33,    34,    35,    36,    37,    38,    39,    40,    
            41,    42,    43,    44,    45,    46,    47,    48,  
            49,    50,    51,    52,    53,    54,    55,    56,    
            57,    58,    59,    60,    61,    62,    63,    64,  
            65,    66,    67,    68,    69,    70,    71,    72,    
            73,    74,    75,    76,    77,    78,    79,    80,  
            81,    82,    83,    84,    85,    86,    87,    88,    
            89,    90,    91,    92,    93,    94,    95,    undef,
            undef, undef, undef, undef, undef, undef, undef, undef, 
            undef, undef, undef, undef, undef, undef, undef, undef,
            undef, undef, undef, undef, undef, undef, undef, undef, 
            undef, undef, undef, undef, undef, undef, undef, undef,
            96,    97,    98,    99,    100,   101,   102,   103,   
            104,   105,   106,   107,   108,   109,   110,   111, 
            112,   113,   114,   115,   116,   117,   118,   119,   
            120,   121,   122,   123,   124,   125,   126,   127, 
            128,   129,   130,   131,   132,   133,   134,   135,   
            136,   137,   138,   139,   140,   141,   142,   143, 
            144,   145,   146,   147,   148,   149,   150,   151,   
            152,   153,   154,   155,   156,   157,   158,   159, 
            160,   161,   162,   163,   164,   165,   166,   167,   
            168,   169,   170,   171,   172,   173,   174,   175, 
            176,   177,   178,   179,   180,   181,   182,   183,   
            184,   185,   186,   187,   188,   189,   190,   191 ],
    
        i2c => [ 
            10,    32,    33,    34,    35,    36,    37,    38,    
            39,    40,    41,    42,    43,    44,    45,    46,  
            47,    48,    49,    50,    51,    52,    53,    54,    
            55,    56,    57,    58,    59,    60,    61,    62,  
            63,    64,    65,    66,    67,    68,    69,    70,    
            71,    72,    73,    74,    75,    76,    77,    78,  
            79,    80,    81,    82,    83,    84,    85,    86,    
            87,    88,    89,    90,    91,    92,    93,    94,  
            95,    96,    97,    98,    99,    100,   101,   102,   
            103,   104,   105,   106,   107,   108,   109,   110, 
            111,   112,   113,   114,   115,   116,   117,   118,   
            119,   120,   121,   122,   123,   124,   125,   126, 
            160,   161,   162,   163,   164,   165,   166,   167,   
            168,   169,   170,   171,   172,   173,   174,   175,
            176,   177,   178,   179,   180,   181,   182,   183,   
            184,   185,   186,   187,   188,   189,   190,   191,
            192,   193,   194,   195,   196,   197,   198,   199,   
            200,   201,   202,   203,   204,   205,   206,   207,
            208,   209,   210,   211,   212,   213,   214,   215,   
            216,   217,   218,   219,   220,   221,   222,   223,
            224,   225,   226,   227,   228,   229,   230,   231,   
            232,   233,   234,   235,   236,   237,   238,   239,
            240,   241,   242,   243,   244,   245,   246,   247,   
            248,   249,   250,   251,   252,   253,   254,   255 ]
    }
);

sub new($$)
{
  shift;
  my @k=unpack "C*", shift;  # generate key from individual characters
  my $table=$transform{shift||'lsp'};
  $table or die "Invalid cipher conversion method\n";
  my $rngsize=scalar @{$table->{i2c}};
  
  my $state=[0..$rngsize-1]; # init the state table
  my $a = 0; my $b=0;
  
  for (0..$rngsize-1)
  {
    $a++; 
    $a%=$rngsize; 
    $b+=$state->[$a]+$k[$_%@k];
    $b%=$rngsize;
    ($state->[$a],$state->[$b])=($state->[$b],$state->[$a]);
  } 
  
  # init the object. 
  my $self=bless [$a,$b,$state, $table];
  $self;
} 
 
# pull next random number from sequence
# some attempt has been made to optimize this
sub rng($)
{
  my $self=shift; 
  my ($a, $b, $state, $table) = @$self; 
  my $rngsize=scalar @{$table->{i2c}};
  
  #printf "A:%.2X B:%.2X\n",$a,$b;
  $a+=1, $a%=$rngsize; my $A=$state->[$self->[0]=$a];
  $b+=$A, $b%=$rngsize; my $B=$state->[$self->[1]=$b];
  ($state->[$a],$state->[$b])=($B,$A);
  $state->[($A+$B)%$rngsize];
} 

sub rngchar($)
{
    my $self=shift;
    my ($a, $b, $state, $table) = @$self; 
    $table->{i2c}->[$self->rng];
}    
              
# encipher lsp string
sub encipher($@)
{
  my $self=shift;
  my ($a, $b, $state, $table) = @$self; 
  my $inpsize=scalar @{$table->{c2i}};
  my $rngsize=scalar @{$table->{i2c}};

  pack "C*", map 
  {
    $_ < $inpsize && defined(my $n = $table->{c2i}->[$_]) or die "Invalid character $_ in encipher stream\n";  
    $table->{i2c}->[($n+$self->rng)%$rngsize];
  } unpack "C*", shift;  
}

# decipher lsp string
sub decipher($@)
{
  my $self=shift;
  my ($a, $b, $state, $table) = @$self; 
  my $inpsize=scalar @{$table->{c2i}};
  my $rngsize=scalar @{$table->{i2c}};

  pack "C*", map
  {
    $_ < $inpsize && defined(my $n = $table->{c2i}->[$_]) or die "Invalid character $_ in decipher stream\n";  
    $table->{i2c}->[($n+$rngsize-$self->rng)%$rngsize]; 
  } unpack "C*", shift;   
}  

1;
