#!/usr/bin/perl
# mailbug homepage  
use warnings;
use strict;
use CGI qw(param escapeHTML self_url -oldstyle_urls Delete_all cookie);
use DBI;
use POSIX;

# get text block, clean ends of lines etc.
sub text($)
{
  my $s = shift; defined $s or $s = '';
  $s =~ s/\s+$//g;
  $s =~ s/[ \r\t]+$//mg;
  $s;
}
                         
sub need($) { my $p = param $_[0]; defined $p or die "'$_[0]' is missing\n"; 
              $p =~ s/\s+$//; $p ne '' or die "'$_[0]' is empty\n"; 
              $p 
            }

my $HEAD = ""; 

sub html(@)
{
  local $" = '';
  print "Content-type: text/html\nPragma: no-cache\nCache-control: no-cache\nExpires: -1\n\n
         <html>
           <head>
             $HEAD
             <title>Mailbug Homepage Editor</title>
             <style type='text/css'> form {margin:0} pre{margin:0} table{margin:0}
                                     body {background-color:white; font-family:'Arial', sans-serif; color:black; font-size:12pt}
                                     table {font-family:'Arial', sans-serif; color:black; font-size:12pt}

                                      
                                    .title {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 150%; vertical-align: top}
                                    .subtitle {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 110%; vertical-align: top}
                                    .data {font-family:'Courier New', monospace; font-size:100%}
                                    .head {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 100%; vertical-align: top}
             </style>
           </head> 
           <body> 
             <span class=title>MailBug Homepage Editor</span><p> 
             @_ 
           </body>
         </html>\n";
  exit;
}

# create footer with multiple elements
sub foot(@)
{
  '<hr>'.join ' &nbsp; ',@_;
}  

# return url current page with given parameters
sub url(;@)
{
  Delete_all;
  param splice(@_,0,2) while @_ > 1;
  self_url;
}  

# return link with specified text and parameters
sub href($;@)
{
  my $text=escapeHTML(shift);
  '<a href='.url(@_).">$text</a>";
}  

# 'die' will show error to contact instead of barfing apache 
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>"};

# same as above, for recoverable errors
sub err { html "Sorry, an error has occurred:<p><pre>@_</pre>",foot(href('Show Open Threads')) };

# mysql> desc news.homepage;;
# +-------+------------+------+-----+---------+----------------+
# | Field | Type       | Null | Key | Default | Extra          |
# +-------+------------+------+-----+---------+----------------+
# | id    | bigint(20) |      | PRI | NULL    | auto_increment |
# | name  | tinytext   | YES  |     | NULL    |                |
# | title | tinytext   | YES  |     | NULL    |                |
# | page  | text       | YES  |     | NULL    |                |
# +-------+------------+------+-----+---------+----------------+
# 4 rows in set (0.00 sec)

# database handle
my $DB;


my %states = (
  top=>sub
  {
    html join '',
         "<table border=1 cellpadding=6 rule=rows>
         <tr><th>Key<th>Title<td colspan=3>&nbsp",
         map("<tr><td>$_->[1]<td>$_->[2]<td>".href('Edit',state=>'edit',id=>$_->[0]).
                                       '<td>'.href('View',state=>'view',id=>$_->[0]).
                                       '<td>'.href('Delete',state=>'delete',id=>$_->[0]),
             @{$DB->selectall_arrayref('select id, name, title from homepage order by name')}),
         '</table><p>',
         foot href('Add homepage',state=>'edit');
  },

  'edit' => sub
  {
    my $id = param 'id';
    no warnings; $id+=0; use warnings;

    my ($name, $title, $page)=('','');
    $id and ($name, $title, $page)=$DB->selectrow_array('select name, title, page from homepage where id = ?',undef,$id);
    defined $name or die "Invalid homepage id $id";
    
    html "<form method=post>
          Edit homepage and click ok.<p> 
          <table>
          <tr><td>Page key:<td><input type=text size=20 name=name value='".escapeHTML($name)."'>
          <tr><td>Title:<td><input type=text size=80 name=title value='".escapeHTML($title)."'>
          <tr><td colspan=2><textarea name=page wrap=soft rows=20 cols=120>".escapeHTML($page)."</textarea>
          </table>
          <input type=submit value='OK'>
          <input type=hidden name=id value=$id>
          <input type=hidden name=state value=save>
          </form>";
  },       

  'save' => sub
  {
    my $id=need 'id'; 
    my $name = text need 'name';
    $name =~ /^[-_0-9a-z]+$/i or die "Page key can contain only letters, numbers, underscore and hyphen.\n";
    my $title = text need 'title';
    my $page = text need 'page';
    $DB->do('lock tables homepage write');
    if ($id)
    {
      $DB->selectrow_array('select 1 from homepage where name=? and id != ?', undef, $name, $id) and die "Page key '$name' is already in use.\n";
      $DB->do('update homepage set name=?, title=?, page=? where id=?',undef,$name,$title,$page,$id);
    } else
    {  
      $DB->selectrow_array('select 1 from homepage where name=?', undef, $name) and die "Page key '$name' is already in use.\n";
      $DB->do('insert homepage set name=?, title=?, page=?',undef,$name, $title, $page);
    }  
    $DB->do('unlock tables');
    (state=>'top')
   },              

  'view' => sub
  {
    my $id=need 'id'; 
    my ($name)=$DB->selectrow_array('select name from homepage where id = ?',undef,$id);
    print "Location: https://www.mailbug.net:446/einfo/einfo.cgi?key=$name\n\n";
    exit;
  },  

  'delete' => sub
  {
    my $id=need 'id'; 
    my $name = $DB->selectrow_array('select name from homepage where id=?', undef, $id); 
    $name eq 'default' and die "Please don't delete the default homepage!\n"; 
    html "Really delete '$name'?", foot(href('No'),href('Yes',state=>'really', id=>$id)); 
  },

  'really' => sub
  {
    my $id=need 'id'; 
    $DB->do('delete from homepage where id=?',undef,$id);
  },  
); # end of state table

my $state = param 'state';
my %next=(state=>'top');
if (defined $state)
{
  $states{$state} or die "invalid state $state\n";
  
  for (reverse 0..10) # try 10 times
  {
    $DB = DBI->connect('DBI:mysql:news:db','news',undef,{RaiseError => 1, AutoCommit=>1}) and last;
    $_ or die "can't connect to database\n";
    sleep 1; 
  }
  
  %next=$states{$state}->();
}


# on exit or if no state, refresh top
$HEAD .= "<meta http-equiv='refresh' content='0;url=".url(%next)."'>\n";
html 'Please wait...';

