#!/usr/bin/perl
# mailbug session log 
use warnings;
use strict;

use Sys::Syslog qw(:DEFAULT setlogsock);
use CGI qw(param escapeHTML self_url -oldstyle_urls);
use DBI;
#use Carp;
use Time::HiRes;

# enclose text in appropriate html, using specified head, and exit

sub html(@)
{
  local $" = ""; 
  print "Content-type: text/html\n\n
         <html>
           <head>
            <meta http-equiv=pragma content=no-cache> 
             <meta http-equiv=cache-control content=no-cache>
             <meta http-equiv='expires' content='-1'>
             <title>MailBug Session Log</title>
             <style type='text/css'> form {margin:0} pre{margin:0} table{margin:0}
                                     body {background-color:white; font-family:'Tahoma', sans-serif; color:black; font-size:12pt}
                                     table {font-family:'Tahoma', sans-serif; color:black; font-size:12pt}
                                     td.data {font-family:'Courier New', monospace; font-size: 10pt}
                                     td.head {font-family:'Tahoma', sans-serif; font-weight: bold; color:black; font-size: 10pt}
                                       

             </style>
           </head> 
           <body> 
           <big><b>MailBug Session Log</b></big><p>
           @_ </body>
           </html>\n";
  exit;
}

# 'die' will show error to user instead of barfing apache  
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };

my $start = Time::HiRes::time();
# connect to database
my $DB;
for (reverse 0..10) # try 10 times
{
  $DB = DBI->connect('DBI:mysql:mailbug:mbdbro','mailbug',undef,{RaiseError => 1, AutoCommit=>1}) and last;
  $_ or die "can't connect to database\n";
  sleep 1; 
}

# return html option string with default
sub opt($$;$) 
{ 
  my ($s, $selected, $value)=@_; 
  my $o = ""; $value and $o .= " value='".escapeHTML($value)."'"; $selected and $o .= " selected";
  "<option$o>".escapeHTML($s)."</option>";  
}

# given time and params in order year, month, day, hour, minute, second (or undef for any) 
# return list of params but if any undefined the return specified time instead
# whence 0 = today at midnight, 1=today at 11:59:59PM, 2=80 days ago midnight, 3 = now
sub timestamp($;@)
{
  my $whence = shift;
  my @time;
  if ($whence==0) {@time=((localtime)[5,4,3],0,0,0)}
  elsif ($whence==1) {@time=((localtime)[5,4,3],23,59,59)}
  elsif ($whence==2) {@time=((localtime time-(86400*80))[5,4,3],0,0,0)}
  else { @time = (localtime)[5,4,3,2,1,0] }
  $time[0] += 1900;  # fix year
  $time[1] +=1;      # fix month
  @_ or return @time;
  my @a;
  for (@_)
  {
    defined $_ or push(@a,undef), next;
    my $p = param $_;
    defined $p or return @time;
    push @a, $p;
  }
  return @a;
}

my @filter;

#sub like($)
#{
#  my $s = shift;
#  my $not = ($s =~ s/^!//);  
#  $s eq '' || $s eq '%' and return undef;
#  my $op = ($s eq '_' || $s =~ /(^|[^\\])[%_]/)?$not?"not like ":"like ":$not?"!= ":"= ";
#  $op.$DB->quote($s);
#}

# create filter string
my $s;
my $user = param 'user'; defined $user or $user = ''; 
my $user_op = (param 'user_op') || 'starts with';
my %user_op = ( 'like' => "user like ".$DB->quote($user),
                'not like' => "user not like ".$DB->quote($user),
                'starts with' => "user like".$DB->quote("$user%") );
$user and push @filter,$user_op{$user_op};

# make domain -> id
my %host=(''=>0); $host{$_->[0]}=$_->[1] for @{$DB->selectall_arrayref("select domain, id from domains")};
my $domainid = param 'domainid'; defined $domainid or $domainid = $host{'mailbug.com'};
my $domainid_op = (param 'domainid_op') || '=';
$domainid and push @filter, "domainid $domainid_op $domainid";

my $phone = param 'phone'; defined $phone or $phone = '';
my $phone_op = (param 'phone_op') || 'contains';
my %phone_op = ( 'like' => "phone like ".$DB->quote($phone),
                 'not like' => "phone not like ".$DB->quote($phone),
                 'contains' => "phone like".$DB->quote("%$phone%") );
$phone ne '' and push @filter,$phone_op{$phone_op};

my $seconds = param 'seconds'; defined $seconds or $seconds = ''; 
$seconds =~ /\D/ and die "Non-digit in 'seconds' filter\n";  
my $seconds_op = (param 'seconds_op') || '>=';
$seconds ne '' and push @filter, "time $seconds_op $seconds";

# msent and mrec are reversed in the database 
my $mrec = param 'mrec'; defined $mrec or $mrec = ''; 
$mrec =~ /\D/ and die "Non-digit in 'Received' filter\n";  
my $mrec_op = (param 'mrec_op') || '>=';
$mrec ne '' and push @filter, "msent $mrec_op $mrec";

my $msent = param 'msent'; defined $msent or $msent=''; 
$msent =~ /\D/ and die "Non-digit in 'Sent' filter\n";  
my $msent_op = (param 'msent_op') || '>=';
$msent ne '' and push @filter, "mrec $msent_op $msent";

my $mfax = param 'mfax'; defined $mfax or $mfax=''; 
$mfax =~ /\D/ and die "Non-digit in 'Faxed' filter\n";  
my $mfax_op = (param 'mfax_op') || '>=';
$mfax ne '' and push @filter, "mfax $mfax_op $mfax";

#my @diag = qw 'sessions halted busy nocarrier dropped link chat timeout lsp session';

#for (@diag)
#{
#  defined param "${_}_op" or param "${_}_op" => '=';
#  if (defined param $_)
#  {
#    param $_ =~ /\D/ and die "Non-digit in '$_' filter\n";
#    push @filter, "$_ \$${_}_op \$$_";
#  } else { param $_ => '' }
#}
  
my $sessions = param 'sessions'; defined $sessions or $sessions = ''; 
$sessions =~ /\D/ and die "Non-digit in 'Sessions' filter\n";  
my $sessions_op = (param 'sessions_op') || '>=';
$sessions ne '' and push @filter, "sessions $sessions_op $sessions";

my $halted = param 'halted'; defined $halted or $halted = '';
$halted =~ /\D/ and die "Non-digit in 'Halted' filter\n";  
my $halted_op = (param 'halted_op') || '>=';
$halted ne '' and push @filter, "halted $halted_op $halted";

my $busy = param 'busy'; defined $busy or $busy = ''; 
$busy =~ /\D/ and die "Non-digit in 'Busy' filter\n";  
my $busy_op = (param 'busy_op') || '>=';
$busy ne '' and push @filter, "busy $busy_op $busy";

my $nocarrier = param 'nocarrier'; defined $nocarrier or $nocarrier = ''; 
$nocarrier =~ /\D/ and die "Non-digit in 'No Carrier' filter\n";  
my $nocarrier_op = (param 'nocarrier_op') || '>=';
$nocarrier ne '' and push @filter, "nocarrier $nocarrier_op $nocarrier";

my $dropped = param 'dropped'; defined $dropped or $dropped = ''; 
$dropped =~ /\D/ and die "Non-digit in 'Dropped' filter\n";  
my $dropped_op = (param 'dropped_op') || '>=';
$dropped ne '' and push @filter, "dropped $dropped_op $dropped";

my $link = param 'link'; defined $link or $link = ''; 
$link =~ /\D/ and die "Non-digit in 'Link' filter\n";  
my $link_op = (param 'link_op') || '>=';
$link ne '' and push @filter, "link $link_op $link";

my $chat = param 'chat'; defined $chat or $chat = ''; 
$chat =~ /\D/ and die "Non-digit in 'Chat' filter\n";  
my $chat_op = (param 'chat_op') || '>=';
$chat ne '' and push @filter, "chat $chat_op $chat";

my $timeout = param 'timeout'; defined $timeout or $timeout = '';
$timeout =~ /\D/ and die "Non-digit in 'Timeout' filter\n";  
my $timeout_op = (param 'timeout_op') || '>=';
$timeout ne '' and push @filter, "timeout $timeout_op $timeout";

my $lsp = param 'lsp'; defined $lsp or $lsp = '';
$lsp =~ /\D/ and die "Non-digit in 'LSP' filter\n";  
my $lsp_op = (param 'lsp_op') || '>=';
$lsp ne '' and push @filter, "lsp $lsp_op $lsp";

my $server = param 'server'; defined $server or $server = ''; 
$server =~ /\D/ and die "Non-digit in 'Server' filter\n";  
my $server_op = (param 'server_op') || '>=';
$server ne '' and push @filter, "server $server_op $server";

my ($ayear, $amonth, $aday, $ahour, $aminute, $asecond) = timestamp 0,qw{ayear amonth aday ahour aminute asecond};
my ($byear, $bmonth, $bday, $bhour, $bminute, $bsecond) = timestamp 1,qw{byear bmonth bday bhour bminute bsecond};

my $after = defined param 'after' and push @filter, "starttime >= unix_timestamp('$ayear-$amonth-$aday $ahour:$aminute:$asecond')";
my $before = defined param 'before' and push @filter,"starttime <= unix_timestamp('$byear-$bmonth-$bday $bhour:$bminute:$bsecond')";

my $cpeid = param 'cpeid'; defined $cpeid or $cpeid = '';  
my $cpeid_op = (param 'cpeid_op') || 'contains';
my %cpeid_op = ( 'like' => "cpeid like ".$DB->quote($cpeid),
                 'not like' => "cpeid not like ".$DB->quote($cpeid),
                 'starts with'=>"cpeid like".$DB->quote("$cpeid\%"),
                 'contains'=>"cpeid like".$DB->quote("\%$cpeid\%") );
$cpeid ne '' and push @filter, $cpeid_op{$cpeid_op};

my $result = param 'result'; defined $result or $result = '';
my $result_op = (param 'result_op') || 'starts with';
my %result_op = ('like' => "error like ".$DB->quote($result),
                 'not like' => "error not like ".$DB->quote($result),
                 'starts with'=>"error like".$DB->quote("$result\%"),
                 'contains'=>"error like".$DB->quote("\%$result\%") );
$result ne '' and push @filter, $result_op{$result_op};

# sessiontype bits from mailbug session.h
# #define cEXTERNAL       0x0004          // xsession required
# #define cOUTGOING       0x0008          // outgoing mail (automatic)
# #define cMANUAL         0x0010          // manually initiated
# #define cCHECK          0x0040          // explicit mail check
# #define cSETTIME        0x0080          // time needs to be set (automatic)


my %sessiontypes= ( 'Set Clock/Deliver'=>[0x88, 0x88,'#ccff99'],
                    'Set Clock'=>[0x88,0x80,'#00ff00'], 
                    'Browse'=>[0x4,0x4,'#00ffff'], 
                    'Manual Deliver'=>[0x9C,0x18,'#ffcc99'], 
                    'Manual Check'=>[0x9C,0x10,'#ffff00'], 
                    'Auto Deliver'=>[0x9C,0x08,'#cccccc'],
                    'Auto Check'=>[0x9C,0x00,'#ffffff']);

my $sessiontype = (param 'sessiontype') || '';
my $sessiontype_op = (param 'sessiontype_op') || '=';
$sessiontypes{$sessiontype} and push @filter, "sessiontype & $sessiontypes{$sessiontype}->[0] $sessiontype_op $sessiontypes{$sessiontype}->[1]";

my $filter = join ' and ', grep defined $_, @filter;
$filter and $filter = "where $filter";

my $show=param('show') || "Last 10";

# build it
my $html .= "<form method=get>
             <table border=1 cellpadding=2 cellspacing=0>";


$html .= "<tr>"; 
# user
$html .= "<td class=head>User<td><select name=user_op>".(join '',map opt($_,$user_op eq $_), sort keys %user_op)."</select>
          <input type=text name=user value='".(escapeHTML $user)."'>";

# domain
$html .= "<td class=head>Domain<td><select name=domainid_op>".(join '',map opt($_,$domainid_op eq $_), ('=', '!='))."</select>
          <select name=domainid>".(join '', map opt($_,($domainid && $domainid eq $host{$_}), $host{$_}),sort keys %host)."</select>";
%host = reverse %host; # now id -> domain

$html .= "<tr>"; 
# phone
$html .= "<td class=head>Phone<td><select name=phone_op>".(join '',map opt($_,$phone_op eq $_), sort keys %phone_op)."</select>
          <input type=text name=phone value='$phone'>";

# seconds
$html .= "<td class=head>Seconds<td><select name=seconds_op>"
	 .(join '',map opt($_,$seconds_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text value='$seconds' name=seconds>" ;

$html .= "<tr>"; 
# cpeid 
$html .= "<td class=head>CPEID<td><select name=cpeid_op>".(join '',map opt($_,$cpeid_op eq $_), sort keys %cpeid_op)."</select>
          <input type=text name=cpeid value='".(escapeHTML $cpeid)."'>";

# result 
$html .= "<td class=head>Result<td><select name=result_op>".(join '',map opt($_,$result_op eq $_), sort keys %result_op)."</select>
          <input type=text name=result value='".(escapeHTML $result)."'>";

$html .= "<tr>"; 
# messages received  
$html .= "<td class=head>Received<td><select name=mrec_op>"
	 .(join '',map opt($_,$mrec_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text value='$mrec' name=mrec>" ;

# messages sent  
$html .= "<td class=head>Sent<td><select name=msent_op>"
	 .(join '',map opt($_,$msent_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text value='$msent' name=msent>" ;

$html .= "<tr>"; 
# messages faxed
$html .= "<td class=head>Faxed<td><select name=mfax_op>"
	 .(join '',map opt($_,$mfax_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text value='$mfax' name=mfax>" ;

# reason for connection 
$html .= "<td class=head>Reason<td><select name=sessiontype_op>"
	 .(join '',map opt($_,$sessiontype_op eq $_), ('=', '!=',))
         ."</select><select name=sessiontype>"
         .(join '',map opt($_, $_ eq $sessiontype), "", sort keys %sessiontypes).
         "</select>";

$html .= "<tr>"; 
# show
$html .= "<td class=head>Show<td><select name=show>";
$html .= opt($_, $_ eq $show) for "All", "Last 1", "Last 10", "Last 100", "Last 1000", "First 1", "First 10", "First 100", "First 1000", "Summary"; 
$html .= "</select>";

# filler
$html .= "<td colspan=2>&nbsp;";

$html .= "<tr>"; 

# time
$html .= sprintf '<td class=head><input type=checkbox name=after%s align=middle>After<td>',$after?" checked":"";
$html .= "<select name=ayear>"; $html .= opt(sprintf("%.2d", $_), $_==$ayear) for 2000..2050; $html .= "</select>-";
$html .= "<select name=amonth>"; $html .= opt(sprintf("%.2d", $_), $_==$amonth) for 1..12; $html .= "</select>-";
$html .= "<select name=aday>"; $html .= opt(sprintf("%.2d", $_), $_==$aday) for 1..31; $html .= "</select>";
$html .= "&nbsp;<select name=ahour>"; $html .= opt(sprintf("%.2d", $_), $_==$ahour) for 0..23; $html .= "</select>:";
$html .= "<select name=aminute>"; $html .= opt(sprintf("%.2d", $_), $_==$aminute) for 0..59; $html .= "</select>:";
$html .= "<select name=asecond>"; $html .= opt(sprintf("%.2d", $_), $_==$asecond) for 0..59; $html .= "</select>";

$html .= sprintf '<td class=head><input type=checkbox name=before%s align=middle>Before<td>',$before?" checked":"";
$html .= "<select name=byear>"; $html .= opt(sprintf("%.2d", $_), $_==$byear) for 2000..2050; $html .= "</select>-";
$html .= "<select name=bmonth>"; $html .= opt(sprintf("%.2d", $_), $_==$bmonth) for 1..12; $html .= "</select>-";
$html .= "<select name=bday>"; $html .= opt(sprintf("%.2d", $_), $_==$bday) for 1..31; $html .= "</select>";
$html .= "&nbsp;<select name=bhour>"; $html .= opt(sprintf("%.2d", $_), $_==$bhour) for 0..23; $html .= "</select>:";
$html .= "<select name=bminute>"; $html .= opt(sprintf("%.2d", $_), $_==$bminute) for 0..59; $html .= "</select>:";
$html .= "<select name=bsecond>"; $html .= opt(sprintf("%.2d", $_), $_==$bsecond) for 0..59; $html .= "</select>";

$html .= "</table><p><table border=1 cellpadding=2 cellspacing=0>";

# sessions  
$html .= "<tr><td class=head>Sessions<td><select name=sessions_op>"
	 .(join '',map opt($_,$sessions_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$sessions' name=sessions>" ;

# halted  
$html .= "<td class=head>Halted<td><select name=halted_op>"
	 .(join '',map opt($_,$halted_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$halted' name=halted>" ;

# busy  
$html .= "<td class=head>Busy<td><select name=busy_op>"
	 .(join '',map opt($_,$busy_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$busy' name=busy>" ;

# nocarrier  
$html .= "<td class=head>No Carrier<td><select name=nocarrier_op>"
	 .(join '',map opt($_,$nocarrier_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$nocarrier' name=nocarrier>" ;

# dropped  
$html .= "<td class=head>Dropped<td><select name=dropped_op>"
	 .(join '',map opt($_,$dropped_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$dropped' name=dropped>" ;

# link  
$html .= "<tr><td class=head>Link<td><select name=link_op>"
	 .(join '',map opt($_,$link_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$link' name=link>" ;

# chat  
$html .= "<td class=head>Chat<td><select name=chat_op>"
	 .(join '',map opt($_,$chat_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$chat' name=chat>" ;

# timeout  
$html .= "<td class=head>Timeout<td><select name=timeout_op>"
	 .(join '',map opt($_,$timeout_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$timeout' name=timeout>" ;

# lsp  
$html .= "<td class=head>Protocol<td><select name=lsp_op>"
	 .(join '',map opt($_,$lsp_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$lsp' name=lsp>" ;

# server  
$html .= "<td class=head>Abort<td><select name=server_op>"
 	 .(join '',map opt($_,$server_op eq $_), ('=', '!=', '>=', '<=')).
         "</select><input type=text size=4 value='$server' name=server>" ;



#
$html .= "</table><p>
          <table border=0>
            <tr><td><input type=submit value='Filter'>
            </form>
            <td>
            <form> <input type=submit value='Reset'><input type=hidden name=x></form>
            </table><p>";
#           <td>
#           <form action='/'> <input type=submit value='Done'> </form>

$html .= "<table border=1 cellpadding=2 cellspacing=0><tr>";

my $query;

my $array;

my $csv;

# return array as csv string
sub csv(@)
{
  my @i = @_;
  
  for (@i) 
  {
    defined $_ or $_ = '';
    $_ =~ s/\"/\"\"/g;
    $_ = qq{"$_"};
  }
  
  (join ',',@i)."\n";
}  

if ($show ne 'Summary')
{ 
  my $rev = 0;
  my $order = "order by starttime asc"; 

  if ($show =~ /^Last (10*)$/)
  {
    $order = "order by starttime desc limit $1";
    $rev = 1;
  } elsif ($show =~ /^First (10*)$/)
  { 
    $order = "order by starttime asc limit $1";
  }

  # list view, items to fetch (note msent, mrec from servers perspective, ie logically reversed)
  my @items = qw(from_unixtime(starttime) user domainid phone time msent mrec mfax cpeid sessions halted busy nocarrier dropped link chat timeout lsp server error);   
  
  # sessiontype is last!
  $query = "select high_priority ".(join ',',@items,'sessiontype')." from mblog $filter $order";
  $array = $DB->selectall_arrayref($query);

  # headings for each item
  my @headings = qw(Time User Domain Phone Secs Rcvd Sent Fax CPEID S H B N D L C T P A Result);  
  
  # items to sum
  my %sum = (time=>0, mrec=>0, msent=>0, mfax=>0, sessions=>0, halted=>0, busy=>0, nocarrier=>0, dropped=>0, link=>0, chat=>0, timeout=>0, lsp=>0, server=>0);

  $rev and $array= [reverse @$array]; 

  $html .= "<td class=head>$_" for @headings;
  $csv = csv(@headings,"sessiontype");

  my $total = 0;
  for my $ref (@$array)
  {
    $total++;
    $ref->[2] = $host{$ref->[2]}||"";
    my $st = $ref->[@items]; # get session type
    $html .= "<tr";
    for (keys %sessiontypes)
    {
      ($st & $sessiontypes{$_}->[0]) == $sessiontypes{$_}->[1] and $html .= " bgcolor='$sessiontypes{$_}->[2]'", last;
    }
    $html .= ">";
    for (0..$#items)
    {
      $html .= "<td class=data>".($ref->[$_] eq ""?"&nbsp;":escapeHTML($ref->[$_]));
      exists $sum{$items[$_]} and $sum{$items[$_]}+=$ref->[$_];
    }
    $csv .= csv(@$ref); 
  }
  $html .= "<tr><td class=data><b>$total sessions</b>";
  for (1..$#items) # skip the first possiblity
  {
    $html .= sprintf("<td class=data><b>%s</b>",(exists $sum{$items[$_]})?$sum{$items[$_]}:"&nbsp;");
  }
  $html .= "</table>
            <p><table border=1 cellpadding=3 cellspacing=0>
            <tr><td>Connect Reason:";
  $html .= "<td bgcolor='$sessiontypes{$_}->[2]'>$_" for sort keys %sessiontypes;
  #</table> is below
}
else
{
  # summary view items to fetch
  my @items = ('count(*)','sum(time) as x');
  my @group = ('domainid');
  $user ne '' and push(@items, 'user'), push(@group, 'user');
  push @items, 'domainid'; 
  $phone ne '' and push(@items, 'phone'), push(@group,'phone');
  $cpeid ne '' and push(@items, 'cpeid'), push(@group,'cpeid');
  $mrec ne '' and push @items, 'sum(msent)';
  $msent ne '' and push @items, 'sum(mrec)';
  $mfax ne '' and push @items, 'sum(mfax)';
  $sessions  ne '' and push @items, 'sum(sessions)';
  $halted ne '' and push @items, 'sum(halted)'; 
  $busy ne '' and push @items, 'sum(busy)'; 
  $nocarrier ne '' and push @items, 'sum(nocarrier)';
  $dropped ne '' and push @items, 'sum(dropped)';
  $link ne '' and push @items, 'sum(link)'; 
  $chat ne '' and push @items, 'sum(chat)';
  $timeout ne '' and push @items, 'sum(timeout)';
  $lsp ne '' and push @items, 'sum(lsp)';
  $server ne '' and push @items, 'sum(server)';
  $result ne '' and push(@items, 'error'), push (@group, 'error');

  $query = "select high_priority ".join(",",@items)." from mblog $filter group by ".join(',',@group)." order by x";
  #die $query;
  $array = $DB->selectall_arrayref($query);

  # headings for each item
  my %headings = ($items[0]=>'Sessions', $items[1]=>'Seconds', user=>'User', domainid=>'Domain', cpeid=>'CPEID', phone=>'Phone',
                  'sum(msent)'=>'Rcvd', 'sum(mrec)'=>'Sent', 'sum(mfax)'=>'Fax', 
                  'sum(sessions)'=>'S', 'sum(halted)'=>'H', 'sum(busy)'=>'B', 'sum(nocarrier)'=>'N', 'sum(dropped)'=>'D', 
                  'sum(link)'=>'L', 'sum(chat)'=>'C', 'sum(timeout)'=>'T', 'sum(lsp)'=>'P', 'sum(server)'=>'A', 
                  error=>'Result'); 
 
  # items to sum
  my %sum = ($items[0]=>0, $items[1]=>0, 'sum(mrec)'=>0, 'sum(msent)'=>0, 'sum(mfax)'=>0,
             'sum(sessions)'=>0, 'sum(halted)'=>0, 'sum(busy)'=>0, 'sum(nocarrier)'=>0, 'sum(dropped)'=>0, 
             'sum(link)'=>0, 'sum(chat)'=>0, 'sum(timeout)'=>0, 'sum(lsp)'=>0, 'sum(server)'=>0); 

  $html .= "<td>&nbsp;";
  $html .= "<td class=head>$headings{$_}" for @items;
  
  $csv = csv(map $headings{$_}, @items); 
  my $total = 0;
  for (0..$#$array)
  {
    my $ref = $array->[$_];
    $html .= "<tr><td class=head>".($_+1);
    for (0..$#items)
    {
      $items[$_] eq 'domainid' and $ref->[$_] = $host{$ref->[$_]}||"";
      $html .= sprintf("<td class=data>%s",($ref->[$_] eq "")?"&nbsp;":escapeHTML($ref->[$_]));
      $sum{$items[$_]}+=$ref->[$_] if exists $sum{$items[$_]};
    }
    $csv .= csv(@$ref);
  }
  $html .= "<tr><td>&nbsp;";
  for (0..$#items) # skip the first possiblity
  {
    $html .= sprintf("<td class=data><b>%s</b>",(exists $sum{$items[$_]})?$sum{$items[$_]}:"&nbsp;");
  }
}

my $stop=Time::HiRes::time();
param 'csv' or param('csv',1), 
               html $html, "</table>
                            <hr>
                            <table width=100% borderwidth=0>
                            <tr style='font-size: xx-small'>
                              <td>Processing time: ",sprintf("%.3f", $stop-$start)," seconds
                              <td align=right><a href='".self_url."'>Download CSV</a>
                            </table>";

print "Content-type: text/plain\nContent-Disposition: attachment; filename=mblog.",time,".csv\n\n$csv";

