#!/usr/bin/perl 

# mailbug domain manager

use warnings;
use strict;
use CGI;
use DBI;

sub param($)
{
  my $p = CGI::param(shift);
  defined $p and $p =~ s/[\x7f-\xa0]/ /g;
  $p;
}  

# get http parameter 
sub supplied($)
{
  my $p = param $_[0]; 
  defined $p or die "Missing parameter '$_[0]'\n";
  $p;
}  

# output text as html 
sub html(@)
{
  print "Content-type: text/html

         <html>
         <head>
           <title>MailBug System Manager</title>
           <meta http-equiv=pragma content=no-cache> 
           <meta http-equiv=cache-control content=no-cache>
           <meta http-equiv='expires' content='-1'>
           <style type='text/css'> 
	      form {margin:0} pre{margin:0} table{margin:0}
              body {background-color:white; font-family:'Tahoma', sans-serif; color:black; font-size:16pt}
              table {font-family:'Tahoma', sans-serif; color:black; font-size:12pt}
              input {vertical-align: middle}

              .data {font-family:'Courier New', monospace; font-size:100%}
              .nodata {font-family:'Courier New', monospace; font-size:100%; background-color:lightgray}
              .head {font-family:'Tahoma', sans-serif; font-weight: bold; color:black; font-sze: 100%; vertical-align: top}
              .note {font-family:'Tahoma', sans-serif; font-style: italic; color:black; font-size: 80%; vertical-align: bottom}
           </style>
         </head><body>
         <b>MailBug System Manager</b><hr>
         @_</body></html>";
  exit;
}

# return a form string 
sub post($$;%)
{
  my ($text, $button, %hidden) = @_;
  $text or $text="";
  $button or $button="OK";
  my $hidden=""; $hidden .= "<input type=hidden name='$_' value='$hidden{$_}'>" for keys %hidden;
  "<form method=post>$text<input type=submit value='$button'>$hidden</form>";
}

sub form($$;%)
{
  my ($text, $button, %hidden) = @_;
  $text or $text="";
  my $wide = $text ne ""?"":"style='width:100%'";
  $button or $button="OK";
  my $hidden=""; $hidden .= "<input type=hidden name='$_' value='$hidden{$_}'>" for keys %hidden;
  "<form>$text<input $wide type=submit value='$button'>$hidden</form>";
}

# wrap text like a MailBug
sub wrap($;$)
{
  my ($s,$max) = @_;
  $s =~ s/[\r\t]//g;
  my $o = "";
  # get line if shorter than 79, or up to first lf, or up to last space, or block of 79 
  $o .= "$+".(" "x(79-length $+))."\n" while $s =~ /\G(.{0,78}?)\n|\G(.{1,79})$|\G(.{0,78})\s|\G(.{79}\n)|\G(.{79})/gs;
  $o = CGI::escapeHTML($o);
  $max and $o =~ s/^((?:.*?\n){$max}).+/$1.../s; # clip long messages
  $o =~ s/\n/<BR>/g; 
  $o;
}

# convert minutes to hh:mm format
sub hhmm($)
{
  sprintf '%d:%.2d',int($_[0]/60),$_[0]%60;
}

# clean a string
sub clean($)
{
  my $s = shift; defined $s or $s = "";
  $s =~ s/^\s+|\s+$//g;
  $s =~ s/\s+/ /g;
  $s;
}

# really really clean a string
sub word($)
{
  my $s = shift; defined $s or $s = "";
  $s =~ s/\s+//g;
  $s;
}


# escape html in a string
sub escape($;$)
{
  my ($s,$nobr) = @_;
  defined $s or $s = "";
  $s = CGI::escapeHTML($s);
  $nobr and $s =~ s/\s/&nbsp;/g;
  $s;
}

# get text block, clean ends of lines etc.
sub text($)
{
  my $s = shift; defined $s or $s = "";
  $s =~ s/\s+$//g; 
  $s =~ s/[ \r\t]+$//mg;
  $s;
}


# truncate string for display purposes
sub trunc($$)
{
  my $s = clean(shift);
  my $n = shift;
  $s =~ /^(.{$n})./ and return escape($1,1)."&hellip;"; 
  escape($s,1);
}

sub any($)
{
  my $s = shift;
  ($s && $s ne "")?$s:"<i>any</i>"; 
}

# escape % and _ in username to \% and \_, then convert * and ? to % and _
sub ue($)
{
  my $u = shift;
  $u =~ s/([%_])/\\$1/g;
  $u =~ y/*?/%_/;
  $u;
}

# undo the above
sub unue($)
{
  my $u = shift;
  $u =~ s/(?<!\\)%/*/g;
  $u =~ s/(?<!\\)_/?/g;
  $u =~ s/\\([%_])/$1/g;
  $u;
}  
  

# database handling
my $DB = DBI->connect('DBI:mysql:mailbug:mbdb','mailbug',undef,{RaiseError => 1, AutoCommit=>1}) or die "can't connect to database\n";

# 'die' will show error to user, doesn't barf apache  
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>@_</pre>" };


# this is the state table, a named array of anonymous subroutines
my %states = ( 

##### domains
  
# mysql> desc domains;
#+--------------+-------------+------+-----+---------+----------------+
#| Field        | Type        | Null | Key | Default | Extra          |
#+--------------+-------------+------+-----+---------+----------------+
#| id           | bigint(20)  |      | PRI | NULL    | auto_increment |
#| domain       | tinytext    |      |     |         |                |
#| pop          | tinytext    |      |     |         |                |
#| parent       | bigint(20)  |      |     | 0       |                |
#| appendix     | tinyint(1)  |      |     | 0       |                |
#| title        | varchar(35) |      |     |         |                |
#| welcome_body | text        |      |     |         |                |
#| welcome_from | tinytext    |      |     |         |                |
#| welcome_subj | tinytext    |      |     |         |                |
#+--------------+-------------+------+-----+---------+----------------+

domains => sub
{
  "Domains:<ul>".
  (join "", map "<li><a href='?state=domain_view&id=$_->[0]'>$_->[1]</a>", 
                @{$DB->selectall_arrayref("select domains.id, domain from domains order by domain")}). 
  "</ul><hr>".form(" ","Create",state=>'domain_edit');   
},  
 

domain_view => sub
{
  my $id = supplied 'id';
  my $user = clean param 'user';
  my $phone = clean param 'phone';

  my ($domain, $pop, $appendix, $parent, $title, $welcome_from, $welcome_subj, $welcome_body)=
          $DB->selectrow_array("select domain,pop,appendix,parent,title,welcome_from,welcome_subj,welcome_body from domains where id=?",undef,$id);

  $domain or die "Invalid domain $id\n";
  
  my $pname=$parent?$DB->selectrow_array("select domain from domains where id=?",undef,$parent):""; 
  $appendix = $appendix?"yes":"no";
  $_=escape $_ for $domain, $title, $pop, $pname; 

  my $welcome = '';
  $welcome_from && $welcome_subj && $welcome_body 
    and $welcome = wrap "-- From   : $welcome_from\n-- Subject: $welcome_subj\n\n$welcome_body", 12;

  my $where = 'where domain_id='.$DB->quote($id);
  $user and $where .= " and (user='' or ".$DB->quote(ue($user))." like user)";
  $phone and $where .= " and if (left(phone,1)='!', locate(substring(phone,2),".$DB->quote($phone).")=0, locate(phone,".$DB->quote($phone)."))";
  $where .= "order by ";
  $where .= $user?"length(user) desc":"user='',user";
  $where .= ",length(phone) desc";
                                   
  "<table width=100% border=1 rules=rows>
     <tr><td class=head>Domain:<td class=data>$domain
     <tr><td class=head>Title:<td class=data>$title
     <tr><td class=head>Pop Server:<td class=data>$pop
     <tr><td class=head>Add Domain:<td class=data>$appendix
     <tr><td class=head>Parent:<td class=data>$pname
     <tr><td class=head>Welcome:<td class=data><pre>$welcome</pre>
   </table>
   <hr>
   <table border=0>
     <tr><td>".form('','Edit',state=>'domain_edit',id=>$id).
        "<td>".form('','Delete',state=>'domain_delete',id=>$id).
        "<td>".form('','Done',state=>'domains').
  "</table>
   <hr>
   Triggers: <ul>".
     (join " ", map "<li><a href='?state=trigger_view&id=$_->[0]&domain_id=$id'>".any(unue $_->[1]).':'.any($_->[2])."</a>",
               @{$DB->selectall_arrayref("select id, user, phone from triggers $where")}).
  "</ul><hr><table><tr><td>".
    form("Trigger list for user <input type=text name=user value='".escape($user)."'> 
                         calling <input type=text name=phone value='".escape($phone)."'>",'Select',state=>'domain_view',id=>$id).
  "<td>".form('','Reset',state=>'domain_view',id=>$id)."</table>";
  
},

domain_edit => sub
{
  my $id = param('id') || 0;

  my ($domain, $pop, $appendix, $parent, $title, $welcome_from, $welcome_subj, $welcome_body)=("","",0,0,"MailBug Online","","","");

  $id and ($domain, $pop, $appendix, $parent, $title, $welcome_from, $welcome_subj, $welcome_body)=
          $DB->selectrow_array("select domain,pop,appendix,parent,title,welcome_from,welcome_subj,welcome_body from domains where id=?",undef,$id);

  my $opts = join '', map "<option value=$_->[0]".(($parent==$_->[0])?' selected':'').">$_->[1]</option>", 
                      @{$DB->selectall_arrayref("select id, domain from domains where parent=0 and id != ?",undef,$id)}; 

  $_ = escape $_ for $domain, $title, $pop, $welcome_from, $welcome_subj, $welcome_body;
  $appendix = $appendix?"checked":"";

  post (($id?"Edit":"Create")." domain and click 'OK':<p>
       <table border=1 rules=rows>
         <tr><td class=head>Domain:
             <td class=data><input class=data type=text size=80 name=domain value='$domain'>
             <td class=note>&nbsp;
         <tr><td class=head>Title:
             <td class=data><input class=data type=text size=30 maxlength=35 name=title value='$title'>
             <td class=note>Appears in title bar when connected (35 chars max)
         <tr><td class=head>Pop Server:
             <td class=data><input class=data type=text size=80 name=pop value='$pop'>
             <td class=note>&nbsp;
         <tr><td class=head>Add Domain:
             <td class=data><input type=checkbox value=1 name=appendix $appendix>
             <td class=note>If set, log in as \"user\@domain\".  
         <tr><td class=head>Parent:
             <td class=data><select name=parent><option value=0>primary</option>$opts</select>
             <td class=note>&nbsp;
         <tr><td class=head>Welcome From:
             <td class=data><input class=data type=text size=80 name=welcome_from value='$welcome_from'>
             <td class=note>&nbsp;
         <tr><td class=head>Welcome Subject:
             <td class=data><input class=data type=text size=80 name=welcome_subj value='$welcome_subj'>
             <td class=note>&nbsp;
         <tr><td class=head>Welcome Body:
             <td class=data><textarea class=data name=welcome_body cols=79 rows=12>$welcome_body</textarea>
             <td class=note>&nbsp;
       </table><hr>",
       "OK",state=>'domain_save',id=>$id);
},

domain_save => sub
{
  my $id = param 'id';
  my $domain = word param 'domain'; $domain or die "Must specify a domain name\n"; $domain =~ tr/A-Z/a-z/;
  my $pop =  word param 'pop'; $pop or die "Must specify a pop server\n"; $pop =~ tr/A-Z/a-z/; 
  my $appendix = (param 'appendix') || 0; 
 (my $parent = (param 'parent') || 0) =~ s/\D//g;
  my $title = clean param 'title';
  length $title > 35 and die "Title cannot exceed 35 characters\n";
  my $welcome_from = clean param 'welcome_from';
  my $welcome_subj = clean param 'welcome_subj';
  my $welcome_body = text param 'welcome_body';

  length($welcome_from)+length($welcome_subj)+length($welcome_body) > 8000 and die "Combined 'Welcome' elements cannot exceed 8000 characters.\n";

  if ($id)
  {
    $DB->do("update domains set domain=?, pop=?, parent=?, appendix=?, title=?, welcome_from=?, welcome_subj=?, welcome_body=? where id=?",
             undef,$domain,$pop,$parent,$appendix,$title,$welcome_from,$welcome_subj,$welcome_body,$id);

  } else
  {  
    $DB->do("insert domains set domain=?, pop=?, parent=?, appendix=?, title=?, welcome_from=?, welcome_subj=?, welcome_body=?",
             undef,$domain,$pop,$parent,$appendix,$title,$welcome_from,$welcome_subj,$welcome_body);
             
    $id = $DB->selectrow_array("select last_insert_id()");         
    $DB->do("insert triggers set domain_id=?, user=?, phone=?, disable=?, disable_message=?, message=?, support_phrase=?,
                               active_delay=?, idle_delay=?, sleep_start=?, sleep_length=?, browser_key=?, 
                               browser_max=?, browser_home=?, tagline=?, allow_fax=?",
                               undef, 
                               $id, '', '',0,'','','For assistance please email support@mailbug.com or call 408-360-0490 Mon-Fri 9AM to 5PM Pacific Time.',
                               60,240,1320,480, 'eInfo',
                               0,'http://localhost/einfo.cgi','', 0);
  }
  if ($parent)
  {
    form "Operation complete. ","OK", state=>'domain_view',id=>$id;
  } else
  {
    form "Operation complete.<p><b>Note: primary domains require xinetd server
          configuration, and possibly radius support and a custom ROM. If this
          isn't what you wanted, please specify a parent for this domain.</b><p>","OK",
          state=>'domain_view',id=>$id;
  }
},

domain_delete => sub
{
  my $id = supplied 'id';
  my $domain = $DB->selectrow_array("select domain from domains where id=?",undef,$id);
  defined $domain or die "Invalid domain id $id\n";
  my $s = "Delete domain $domain";
  my @subs = @{$DB->selectcol_arrayref("select id from domains where parent=?",undef,$id)};
  @subs and $s .= " and ".(scalar @subs)." subdomains";
  $DB->selectrow_array("select 1 from mblog where domainid in (".(join ',',$id,@subs).") limit 1") 
   and $s = "<h2>WARNING: deleting this domain will break the MailBug Session Log</h2><hr>$s";
  form "$s? ","Yes",state=>'domain_dodelete',id=>$id;
},

domain_dodelete => sub
{
  my $id = supplied 'id';
  $id or die "Invalid id $id\n";
  $DB->do("delete from triggers using triggers, domains where triggers.domain_id = domains.id and (domains.id = ? or domains.parent = ?)", undef,$id, $id);
  $DB->do("delete from domains where id=? or parent=?",undef,$id,$id);
  form "Operation complete. ","OK", state=>'domains';
},

##### triggers

# mysql> desc triggers;
# +----------------+-------------+------+-----+---------+----------------+
# | Field          | Type        | Null | Key | Default | Extra          |
# +----------------+-------------+------+-----+---------+----------------+
# | id             | bigint(20)  |      | PRI | NULL    | auto_increment |
# | user           | tinytext    |      |     |         |                |
# | phone          | tinytext    |      |     |         |                |
# | domain_id      | bigint(20)  |      |     | 0       |                |
# | disable        | tinyint(1)  |      |     | 0       |                |
# | active_delay   | smallint(6) |      |     | 0       |                |
# | idle_delay     | smallint(6) |      |     | 0       |                |
# | sleep_start    | smallint(6) |      |     | 0       |                |
# | sleep_length   | smallint(6) |      |     | 0       |                |
# | browser_key    | varchar(8)  |      |     |         |                |
# | message        | mediumtext  |      |     |         |                |
# | browser_max    | int(11)     |      |     | 0       |                |
# | browser_home   | text        |      |     |         |                |
# | support_phrase | text        |      |     |         |                |
# | tagline        | text        |      |     |         |                |
# +----------------+-------------+------+-----+---------+----------------+
# 16 rows in set (0.01 sec)

trigger_view => sub
{
  my $domain_id = supplied 'domain_id';
  my $domain = $DB->selectrow_array("select domain from domains where id = ?",undef,$domain_id);
  $domain or die "Invalid domain id $domain_id\n";
  my $id = supplied 'id';
  
  my $r = $DB->selectrow_arrayref("select user, phone, disable, message, support_phrase,
                                          active_delay, idle_delay, sleep_start, sleep_length,
                                          browser_key, browser_max, browser_home, tagline, 
                                          allow_fax, disable_message
                                   from triggers where id=? and domain_id=?",undef,$id,$domain_id);
                                   
  $r or die "Invalid trigger id=$id domain_id=$domain_id\n";
 
  sub tdg($$)  { "<td class=".(defined $_[0]?"data>$_[1]":"nodata>&nbsp;") } 
  sub tdgt($$) { "<td class=".(defined $_[0]?"data>".(($_[0] ne '')?$_[1]:"<i>(blank)</i>"):"nodata>&nbsp;") } 
  
  "<b>".escape($domain)."</b> trigger $id<p>
  <table border=1 width=100% rules=rows>
    <tr><td class=head width=15%>Username:<td class=data>".(any escape(unue $r->[0])).
   "<tr><td class=head>Phone contains:<td class=data>".(any escape($r->[1])).
   "<tr><td class=head>Disable Service?" .tdg($r->[2],$r->[2]?"yes":"no").
   "<tr><td class=head>Disabled Message:".tdgt($r->[14],"<pre>".(wrap($r->[14])||"&nbsp;")."</pre>").
   "<tr><td class=head>Enabled Message:" .tdgt($r->[3],"<pre>".(wrap($r->[3])||"&nbsp")."</pre>").
   "<tr><td class=head>Support Phrase:"  .tdgt($r->[4],escape $r->[4]).
   "<tr><td class=head>Idle Delay:"      .tdg($r->[6],hhmm $r->[6]). # note swapped!
   "<tr><td class=head>Active Delay:"    .tdg($r->[5],hhmm $r->[5]).
   "<tr><td class=head>Sleep Start:"     .tdg($r->[7],hhmm $r->[7]).
   "<tr><td class=head>Sleep Length:"    .tdg($r->[8],hhmm $r->[8]).
   "<tr><td class=head>Browser Key:"     .tdgt($r->[9],escape $r->[9]).
   "<tr><td class=head>Browser Max:"     .tdg($r->[10],hhmm $r->[10]).
   "<tr><td class=head>Browser Home:"    .tdgt($r->[11],escape $r->[11]).
   "<tr><td class=head>Allow Fax:"       .tdg($r->[13], $r->[13]).
   "<tr><td class=head>Tagline:"         .tdgt($r->[12], "<pre>".escape($r->[12])."</pre>").
 "</table><hr>
  <table border=0>
  <tr><td>".form("","Edit ",state=>'trigger_edit',id=>$id,domain_id=>$domain_id).
     "<td>".form("","Clone",state=>'trigger_edit',clone_id=>$id,domain_id=>$domain_id).
     "<td>".form("","Delete",state=>'trigger_delete',id=>$id,domain_id=>$domain_id).
     "<td>".form("","Done",state=>'domain_view',id=>$domain_id).
 "</table>";  
},
  
trigger_edit => sub
{
  my $id = param('id') || 0;
  my $clone_id; $id or $clone_id = supplied('clone_id');
  my $domain_id = supplied 'domain_id';
  my $domain = $DB->selectrow_array("select domain from domains where id = ?",undef,$domain_id);
  $domain or die "Invalid domain id $domain_id\n";

  my ($user, $phone, $disable, $disable_message, $message, $support_phrase, 
      $active_delay, $idle_delay, $sleep_start, $sleep_length, 
      $browser_key, $browser_max, $browser_home, $tagline, 
      $allow_fax, $isdefault) 
        = $DB->selectrow_array("select user, phone, disable, disable_message, message, support_phrase,
                                       active_delay, idle_delay, sleep_start, sleep_length,
                                       browser_key, browser_max, browser_home, tagline, 
                                       allow_fax, user = '' and phone = ''
                                from triggers where id=? and domain_id=?",undef,$id||$clone_id,$domain_id);

  $id or $isdefault = 0;

  $_=defined $_?hhmm $_:'' for $idle_delay, $active_delay, $sleep_start, $sleep_length, $browser_max;
  $_=defined $_?($_ eq ''?'.':escape $_):'' for $disable_message, $message, $support_phrase, $browser_key, $browser_home, $tagline;
  sub yesno($$) { "<select name=$_[0]><option></option>
                                      <option value=1".($_[1]?" selected":"").">Yes</option>
                                      <option value=0".((!$_[1] && defined $_[1])?" selected":"").">No</option>
                   </select>" };
   
  post (($id?"Edit":"Create")." <b>".escape($domain)."</b> trigger and click 'OK'.<br> 
                               <i>Leave fields empty unless you wish to
                               specifically set their values for this trigger.
                               Empty fields will 'inherit' settings from lower
                               priority triggers. To specify a <u>blank</u>
                               field, set it to a single '.' character.<p>
         <table border=1 rules=rows>
           <tr><td class=head>Username:
               <td class=data>".($isdefault?'':"<input class=data type=text size=30 name=user value='".escape(unue $user)."'>").
              "<td class=note>Must match exactly, as shown in the MailBug Session Log. '?' matches single character, '*' matches any characters. 
               
           <tr><td class=head>Phone Contains:
               <td class=data>".($isdefault?'':"<input class=data type=text size=30 name=phone value='".escape($phone)."'>").
              "<td class=note>Must be a fragment of the phone number dialed by the MailBug. 

           <tr><td class=head>Disable Service?
               <td class=data>".yesno('disable',$disable)."
               <td class=note>If disabled, the connection will be immediately terminated.
               
           <tr><td class=head>Disabled Message:
           <td class=data><textarea class=data name=disable_message cols=79 rows=6>$disable_message</textarea>
               <td class=note>Appears on the MailBug idle screen if service is disabled.
               
           <tr><td class=head>Enabled Message:
               <td class=data><textarea class=data name=message cols=79 rows=6>$message</textarea>
               <td class=note>Appears on MailBug idle screen if service is enabled. 

           <tr><td class=head>Support Phrase:
               <td class=data><textarea class=data rows=2 cols=79 name=support_phrase>$support_phrase</textarea>
               <td class=note>Appended to error messages displayed on the MailBug by the server.
              
           <tr><td class=head>Idle Delay:
               <td class=data><input class=data type=text name=idle_delay size=6 maxlength=6 value='$idle_delay'> 
               <td class=note>Callback delay if no mail sent or received.

           <tr><td class=head>Active Delay:
               <td class=data><input class=data type=text name=active_delay size=6 maxlength=6 value='$active_delay'>
               <td class=note>Callback delay if mail sent or received.

           <tr><td class=head>Sleep Start:
               <td class=data><input class=data type=text name=sleep_start size=6 maxlength=6 value='$sleep_start'>
               <td class=note>Start time of sleep period.

           <tr><td class=head>Sleep Length:
               <td class=data><input class=data type=text name=sleep_length size=6 maxlength=6 value='$sleep_length'>
               <td class=note>Length of sleep period, 0=no sleep.
           
           <tr><td class=head>Browser Key: 
               <td class=data><input class=data type=text name=browser_key maxlength=8 size=8 value='$browser_key'> 
               <td class=note>Key label displayed by MailBug after normal disconnect, delete to disable browser.

           <tr><td class=head>Browser Max:
               <td class=data><input class=data type=text name=browser_max size=6 maxlength=6 value='$browser_max'>
               <td class=note>Maximum time allowed per browser session, 0 = no limit.
 
           <tr><td class=head>Browser Home:
               <td class=data><input class=data type=text size=30 name=browser_home value='$browser_home'>
               <td class=note>Fully qualfied URL for browser's home page.

           <tr><td class=head>Allow Fax:
               <td class=data><input class=data type=text name=allow_fax size=6 maxlength=6 value='$allow_fax'>
               <td class=note>Number of pages allowed to be sent to 'fax:name\@number' per month.
               
           <tr><td class=head>Tagline:
               <td class=data><textarea class=data name=tagline cols=79 rows=3>$tagline</textarea>
               <td class=note>Appended, <b>exactly</b> as shown, to outgoing mail. However note recipient may view it with proportional font.

         </table>",
         'OK',state=>'trigger_save',id=>$id, clone_id=>$clone_id,domain_id=>$domain_id);
},

trigger_save => sub
{
  # return undef, '', or data
  sub trist($) { !defined $_[0] || $_[0] eq '' and return undef; $_[0] eq '.' and return ''; $_[0] }

  # return numer of minutes specified, or undef
  sub minutes($;$) 
  { 
    my ($d, $max)=@_; 
    defined $d && $d =~ /^(?:(\d*):)?(\d*)/ or return undef; 
    $d = (($1|0)*60)+($2|0);
    defined $max && $d > $max and $d = $max; 
    $d;
  }
  
  my $domain_id = supplied 'domain_id'; $DB->selectrow_array("select 1 from domains where id = ?",undef,$domain_id) or die "Invalid domain id $domain_id\n";
  my $id = param 'id';
  my $user = word param 'user'; $user =~ tr/A-Z/a-z/;
 (my $phone = (param 'phone') || ''); my $not = $phone =~ s/^!//; $phone =~ s/\D//g; $not && $phone and $phone = "!$phone";
  my $disable = param 'disable';
  my $disable_message = trist text param 'disable_message';
  my $message = trist text param 'message';
  my $support_phrase = trist clean param 'support_phrase'; 
  my $active_delay = minutes trist word param 'active_delay',1440;
  my $idle_delay = minutes trist word param 'idle_delay', 1440;
  my $sleep_start = minutes trist word param 'sleep_start', 1439;
  my $sleep_length = minutes trist word param 'sleep_length', 720;
  my $browser_key = trist clean param 'browser_key'; 
  my $browser_max = minutes trist word param 'browser_max'; 
  my $browser_home = trist word param 'browser_home';
  my $tagline = trist text param 'tagline';
  my $allow_fax = param 'allow_fax';
  defined $_ && $_ ne '' or $_ = undef for $disable, $allow_fax;   
  
  length $browser_key > 8 and die "Browser Key cannot exceed 8 characters\n"; 
  length $disable_message > 1014 and die "Disabled Message cannot exceed 1014 characters.\n"; 
  length $message > 1014 and die "Enabled Message cannot exceed 1014 characters.\n"; 
  length $support_phrase > 512 and die "Support Phrase cannot exceed 512 characters.\n"; 

  my $who = any($user).":".any($phone); 
  $user = ue $user;

  if ($id)
  {
    ($user ne '' || $phone ne '') && $DB->selectrow_array("select user = '' and phone = '' from triggers where id = ?", undef, $id)
      and die "Can't change user/phone of default trigger\n";
        
  
    $DB->selectrow_array("select 1 from triggers where id != ? and user=? and phone=? and domain_id=?",undef,$id,$user,$phone,$domain_id)
      and die "Trigger <b>$who</b> already exists.\n";

    $DB->do("update triggers set user=?,phone=?,disable=?,disable_message=?,message=?,support_phrase=?,
                                 active_delay=?,idle_delay=?,sleep_start=?,sleep_length=?,
                                 browser_key=?,browser_max=?, browser_home=?, tagline=?, 
                                 allow_fax=? where id=? and domain_id=?",
                                 undef,
                                 $user, $phone, $disable, $disable_message, $message, $support_phrase, 
                                 $active_delay, $idle_delay, $sleep_start, $sleep_length, 
                                 $browser_key, $browser_max, $browser_home, $tagline, 
                                 $allow_fax, $id, $domain_id); 
    return form "Updated trigger <b>$who</b>. ",'OK', state=>'trigger_view', id=>$id, domain_id=>$domain_id;
  }  
  
  $DB->selectrow_array("select 1 from triggers where user=? and phone=? and domain_id=?",undef,$user,$phone,$domain_id)
    and die "Trigger <b>$who</b> already exists.\n";

  $DB->do("insert triggers set domain_id=?,
                               user=?,phone=?,disable=?,disable_message=?,message=?,support_phrase=?,
                               active_delay=?,idle_delay=?,sleep_start=?,sleep_length=?,
                               browser_key=?,browser_max=?, browser_home=?, tagline=?, 
                               allow_fax=?",
                               undef, $domain_id, 
                               $user, $phone, $disable, $disable_message, $message, $support_phrase, 
                               $active_delay, $idle_delay, $sleep_start, $sleep_length, 
                               $browser_key, $browser_max, $browser_home, $tagline, 
                               $allow_fax); 
                               
  $id = $DB->selectrow_array("select last_insert_id()");
  form "Created trigger <b>$who</b>. ",'OK', state=>'trigger_view', id=>$id, domain_id=>$domain_id;
},

trigger_delete => sub
{
  my $domain_id = supplied 'domain_id';
  my $id = supplied 'id';
  my $r = $DB->selectrow_arrayref("select user, phone, user='' and phone='' from triggers where id=? and domain_id=?",undef,$id,$domain_id);
  $r or die "Invalid trigger id $id or domain id $domain_id\n";
  $r->[2] and die "Sorry, can't delete the default trigger\n";

  form "Delete trigger <b>".any(unue $r->[0]).":".any($r->[1])."</b>? ","Yes",state=>'trigger_dodelete',id=>$id,domain_id=>$domain_id;
},

trigger_dodelete => sub
{
  my $domain_id = supplied 'domain_id';
  my $id = supplied 'id';
  $DB->do('delete from triggers where id=? and domain_id=?',undef,$id,$domain_id);
  form 'Trigger deleted. ','OK', state=>'domain_view',id=>$domain_id;
},

);

# here, process submitted state
my $n = (param 'state') || 'domains';
die "Illegal state '$n'\n" unless exists $states{$n};           # and valid     
html ($states{$n}->());                                         # display the result 
