package send;
use strict;
use warnings;
use Mail::Mailer;
use MIME::QuotedPrint;
use MIME::Words 'encode_mimewords';

# wrap text like a MailBug
sub wrap($)
{
  my ($s,$max) = @_;
  $s =~ s/[\r\t]//g;
  my $o;
  # get line if shorter than 79, or up to first lf, or up to last space, or block of 79 
  $o .= "$+\n" while $s =~ /\G(.{0,78}?)\n|\G(.{1,79})$|\G(.{0,78})\s|\G(.{79}\n)|\G(.{79})/gs;
  $o =~ s/ +$//gm;
  $o;
}

# create header and send mail 
# must at least specify $return and one of $rto, $rcc or $rbcc
sub mail($$)
{
  my ($headers, $body)=@_;

  #warn "mail headers: ",map("$_->'".(ref($headers->{$_})?join ",",@{$headers->{$_}}:$headers->{$_})."' ",keys %$headers),"\n";

  # make sure there's a date
  $headers->{Date} ||= POSIX::strftime("%a, %d %b %Y %H:%M:%S %z", localtime);
  
  # other required headers
  $headers->{'MIME-Version'}='1.0';
  $headers->{'Content-Type'}='text/plain; charset=ISO-8859-1';   
  $headers->{'Content-Transfer-Encoding'}='quoted-printable';

  # make sure subject is encoded 
  $headers->{Subject}=encode_mimewords($headers->{Subject});

  defined $body or $body = "";

  my $m=new Mail::Mailer('sendmail','-bm',                              # send from stdin                             
                                    '-f'.$headers->{'Return-Path'},     # set return path
                                    '-ODeliveryMode=deferred',          # queue without dns lookup
                                    '-OErrorMode=mail',                 # mail back errors
                                    '-OIgnoreDots')                     # ignore lone dot
                                    ->open($headers);                   # open

  $m or warn("can't open sendmail\n"), return 0;                        # complain if errors
  print $m encode_qp $body or warn("can't write sendmail\n"), return 0; 
  $m->close;                                                            # ignore close error
  1;
}

# send fax through the send2fax.com service
# the number is passed as $headers->{Number}
sub fax($$)
{
  my ($headers, $body)=@_;
  (my $number = $headers->{Number}) =~ s/\D//g;
  $number or return 0;
  
  # build a text header
  my $b = "-- Date   : $headers->{Date}\n".
          "-- From   : $headers->{From}\n".
          "-- Fax To : $headers->{To}\n";

  $headers->{Subject} ne "" and $b .= "-- Subject: $headers->{Subject}\n";

  mail({'Return-Path'   => 'faxagent@mailbug.net',
        'From'          => 'faxagent@mailbug.net',
        'To'            => '1'.$number.'@fax.send2fax.com',
        # 'Bcc'           => 'rich@landel.com',           # for debug
        'Date'          => $headers->{Date},
        'Subject'       => "FaxDeliveryError", 		# this is actually the password
        'Reply-To'      => $headers->{'Return-Path'}},  # confirmations go here
        "$b\n".wrap($body));
}

# send a bounce about an invalid fax
sub faxbounce($$)
{
  my ($sender, $address)=@_;

  mail({'Return-Path' => 'faxagent@mailbug.net',
        'From'       => 'MailBug Fax Service <support@landel.com>',
        'To'          => $sender,
        # 'Bcc'           => 'rich@landel.com',           # for debug
        'Subject'     => 'Fax Addressing Error'},
        <<EOT);
Sorry, the fax address 'fax: $address' is invalid.

A fax address should look like this:

 fax: name \@ number

The 'fax:' keyword identifies this as a fax address (similar to 'cc:' or
'bcc:'). Spaces are allowed after the fax: keyword.

The name is optional. If provided it will appear in the fax exactly as
entered.  It may contain any combination of upper- or lower-case letters,
numbers, '_', '-' or '.'.  If the name is enclosed in quotes then it may
contain any character, including a space.

The '\@' is mandatory. Spaces are allowed between the '\@' and the name or
number.

The number must contain exactly 10 digits and cannot start with '0' or '1'. The
number may also contain letters, '_', '-' or '.' but these will be ignored. 

Right:

 fax: support\@4083600499

 fax:Mailbug.Support \@ 408-360-0499

 fax:\@4083600499

 fax:   S-u.P_p.O-r.T  \@  4-0.8-3_6-0.0-4_9-9

 fax:"Mailbug? It's the greatest!" \@ 408-360-0499


Wrong:

 support\@4083600499 (forgot the fax: keyword)

 fax: Mailbug Support \@ 408-360-0499 (space in the name)

 fax:4083600499 (missing '\@' before the number)

 fax:   S-u.P_p.O-r.T  \@  3_6-0.0-4_9-9 (not enough digits)

 fax:"Mailbug? It's the greatest! \@ 408-360-0499 (forgot " after the name)


The fax address can be used in conjunction with any other email or fax
address, for example:

 To: support\@mailbug.com, fax:support\@408-360-0499, fax:another\@5105551212

The email recipient will not see the fax addresses, and the fax recipients
will not see the email or other fax addresses.

Regards,

Landel Customer Support
EOT
}        

# send a bounce because fax limit was reached
# note, contains variables and so should be unwrapped (let mailbug wrap it correctly)
sub faxlimit($$$)
{
  my ($sender, $address, $limit)=@_;

  mail({'Return-Path' => 'faxagent@mailbug.net',
        'From'       => 'MailBug Fax Service <support@landel.com>',
        'To'          => $sender,
        # 'Bcc'         => 'rich@landel.com',
        'Subject'     => 'Fax Limit Reached'},
        <<EOT);

Sorry, your fax to '$address' could not be sent because you have reached your limit for this month. Please wait until next month and try again. 

Your MailBug Service allows up to $limit fax pages per month. Not enough? Add 5 more pages for only \$1/month, 10 pages for \$2/month, and so on. Reply to this email for more information.

Regards,

Landel Customer Support
EOT
}        

  
1;
