# iptables policy for lx servers
# edit iptables configure here, run on target server to make them permanent

# edit iptables configure here, run to make permanent

die() { echo $* >&2; exit 1; }
 
[[ ${HOSTNAME:0:2} != 'lx' ]] && die "must be lx server" # avoid breaking non-lx servers

internal=eth1
intaddr=`/sbin/ifconfig $internal`; intaddr=${intaddr#*inet addr:}; intaddr=${intaddr%% *};
external=eth0
extaddr=`/sbin/ifconfig $external`; extaddr=${extaddr#*inet addr:}; extaddr=${extaddr%% *};
[[ $intaddr && $extaddr ]] || die $internal and $external must be up
echo "Configuring iptables with internal $internal ($intaddr), external $external ($extaddr)"

# reset
iptables -F # flush everything
iptables -t nat -F
iptables -t mangle -F
iptables -X # delete user chains
iptables -Z # zero counters

# default policies  
iptables -P INPUT DROP 
iptables -P FORWARD DROP 
iptables -P OUTPUT ACCEPT 

# trust lo and eth1, allow eth0 expected 
iptables -A INPUT -i lo -j ACCEPT       
iptables -A INPUT -i eth1 -j ACCEPT     
iptables -A INPUT -i eth0 -m state --state RELATED,ESTABLISHED -j ACCEPT

# everything else is eth0 unsolicited inbound

# allow incoming ping
iptables -A INPUT -p icmp -d $extaddr --icmp-type echo-request -j ACCEPT

# allow incoming ssh ON PORT 9!
iptables -A INPUT -p tcp -d $extaddr --dport 9 --syn -j ACCEPT

# load balanced services follow

# https://dbug.mailbug.net
iptables -A INPUT -p tcp -d $extaddr --dport 444 --syn -j ACCEPT

# https://www.mailbug.net
iptables -A INPUT -p tcp -d $extaddr --dport 446 --syn -j ACCEPT

# allow mailbugs at mailbug.com, etc 
iptables -A INPUT -p tcp -d $extaddr --dport 2011 --syn -j ACCEPT

# allow mailbugs at bls.mailbug.com
iptables -A INPUT -p tcp -d $extaddr --dport 2012 --syn -j ACCEPT

# allow mailbugs at iowatelecom
iptables -A INPUT -p tcp -d $extaddr --dport 3102 --syn -j ACCEPT

# allow databugs
iptables -A INPUT -p tcp -d $extaddr --dport 2001 --syn -j ACCEPT
iptables -A INPUT -p tcp -d $extaddr --dport 15001 --syn -j ACCEPT

# allow lmdp at mailbug.com
iptables -A INPUT -p tcp -d $extaddr --dport 2111 --syn -j ACCEPT

# allow loadwatch from util servers 
iptables -A INPUT -p tcp -d $extaddr --dport 9999 -s 128.242.106.96/29 --syn -j ACCEPT

# open honeypots, 
iptables -A INPUT -p tcp -d $extaddr --dport 23 --syn -j ACCEPT
iptables -A INPUT -p tcp -d $extaddr --dport 6667 --syn -j ACCEPT

# save for boot script 
/etc/init.d/iptables save
