#!/usr/bin/perl
# mailbug email support
use warnings;
use strict;
use Mail::Mailer;
use CGI qw(escapeHTML url self_url -oldstyle_urls Delete_all cookie);
use DBI;
use POSIX;
use MIME::QuotedPrint;
use MIME::Words 'encode_mimewords';
use bytes;

#mysql> desc contact; desc thread; desc msg;
#+---------+--------------+------+-----+---------+----------------+
#| Field   | Type         | Null | Key | Default | Extra          |
#+---------+--------------+------+-----+---------+----------------+
#| id      | bigint(20)   |      | PRI | NULL    | auto_increment |
#| contact | varchar(255) | YES  | MUL | NULL    |                |
#| block   | tinyint(1)   |      |     | 0       |                |
#+---------+--------------+------+-----+---------+----------------+
#3 rows in set (0.00 sec)
#
#+-------------+------------+------+-----+---------------------+----------------+
#| Field       | Type       | Null | Key | Default             | Extra          |
#+-------------+------------+------+-----+---------------------+----------------+
#| id          | bigint(20) |      | PRI | NULL                | auto_increment |
#| contact_id  | bigint(20) |      |     | 0                   |                |
#| date        | datetime   |      |     | 0000-00-00 00:00:00 |                |
#| touched     | datetime   |      |     | 0000-00-00 00:00:00 |                |
#| holdoff     | datetime   |      |     | 0000-00-00 00:00:00 |                |
#| subject     | text       |      |     |                     |                |
#| tolist      | text       |      |     |                     |                |
#| cclist      | text       |      |     |                     |                |
#| resolved    | tinyint(1) |      |     | 0                   |                |
#| comment     | text       |      |     |                     |                |
#| urgent      | tinyint(4) |      |     | 0                   |                |
#+-------------+------------+------+-----+---------------------+----------------+
#8 rows in set (0.00 sec)
#
#+-----------+------------+------+-----+---------------------+----------------+
#| Field     | Type       | Null | Key | Default             | Extra          |
#+-----------+------------+------+-----+---------------------+----------------+
#| id        | bigint(20) |      | PRI | NULL                | auto_increment |
#| thread_id | bigint(20) |      |     | 0                   |                |
#| date      | datetime   |      |     | 0000-00-00 00:00:00 |                |
#| body      | text       |      |     |                     |                |
#| raw       | text       |      |     |                     |                |
#| isrecv    | tinyint(1) |      |     | 0                   |                |
#| issent    | tinyint(1) |      |     | 0                   |                |
#+-----------+------------+------+-----+---------------------+----------------+
#7 rows in set (0.00 sec)

my $SUPPORT = '<support@landel.com>';
my $SUPPORTS = 'Landel Customer Support <support@landel.com>';

# create header and send mail 
# must at least specify $return and one of $rto, $rcc or $rbcc
sub mail($$)
{
  my ($headers, $body)=@_;

  #warn "mail headers: ",map("$_->'".(ref($headers->{$_})?join ",",@{$headers->{$_}}:$headers->{$_})."' ",keys %$headers),"\n";

  # make sure there's a date
  $headers->{Date} ||= POSIX::strftime("%a, %d %b %Y %H:%M:%S %z", localtime);
  
  # other required headers
  $headers->{'Return-Path'}=$SUPPORT;                                   # all msgs from support
  $headers->{'From'}=$SUPPORTS;
  $headers->{'MIME-Version'}='1.0';
  $headers->{'Content-Type'}='text/plain; charset=ISO-8859-1';   
  $headers->{'Content-Transfer-Encoding'}='quoted-printable';

  # make sure subject is encoded 
  $headers->{Subject}=encode_mimewords($headers->{Subject});

  defined $body or $body = "";

  my $m=new Mail::Mailer('sendmail','-bm',	 	                          # send from stdin				
                                    '-f'.$headers->{'Return-Path'},     # set return path
                                    '-ODeliveryMode=deferred',		      # queue without dns lookup
                                    '-OErrorMode=mail',			            # mail back errors
                                    '-OIgnoreDots')			                # ignore lone dot
  				    ->open($headers);                                         # open

  $m or die("can't open sendmail\n");                                   # complain if errors
  print $m encode_qp $body or die("can't write sendmail\n"); 
  $m->close;                                                            # ignore close error
}


sub param($)
{
  my $p = CGI::param(shift);
  defined $p and $p =~ s/[\x7f-\xa0]/ /g; # Avoid IE unpleasantness 
  $p;
}  


sub need($) 
{ 
  my $p = param $_[0] or die "'$_[0]' is missing\n"; 
  $p =~ s/\s+$//; $p ne '' or die "'$_[0]' is empty\n"; 
  $p 
}

sub nparam($)
{
  my $p = param $_[0];
  no warnings; $p+=0; use warnings;
  $p;
} 

my $HTTP = "Content-type: text/html\n";

my $HEAD = "<meta http-equiv=pragma content=no-cache> 
            <meta http-equiv=cache-control content=no-cache>
            <meta http-equiv='expires' content='-1'>";



sub html(@)
{
  local $" = '';
  print "$HTTP\n
         <html>
           <head>
             $HEAD
             <title>MailBug Support</title>
             <style type='text/css'> form {margin:0} pre{margin:0} table{margin:0}
                                     body {background-color:white; font-family:'Arial', sans-serif; color:black; font-size:12pt}
                                     table {font-family:'Arial', sans-serif; color:black; font-size:12pt}

                                      
                                    .title {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 150%; vertical-align: top}
                                    .subtitle {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 110%; vertical-align: top}
                                    
                                    .data {font-family:'Courier New', monospace; font-size:100%}
                                    .tiny {font-family:'Courier New', monospace; font-size:60%}
                                    .stamp {font-family:'Courier New', monospace; vertical-align:middle}
                                    .head {font-family:'Arial', sans-serif; font-weight: bold; color:black; font-size: 100%; vertical-align: top}
             </style>
           </head> 
           <body> 
             <span class=title>MailBug Support</span><p> 
             @_ 
           </body>
         </html>\n";
  exit;
}

# return html option string with default
sub opt($$;$) 
{ 
  my ($s, $selected, $value)=@_; 
  my $o = ""; $value and $o .= " value='".escapeHTML($value)."'"; $selected and $o .= " selected";
  "<option$o>".escapeHTML($s)."</option>";  
}

# create footer with multiple elements
sub foot(@)
{
  '<hr>'.join ' &nbsp; ',@_;
}  

# wrap text like a MailBug
sub wrap($)
{
  my $s = shift;
  $s =~ s/[\r\t]//g;
  my $o;
  # get line if shorter than 79, or up to first lf, or up to last space, or block of 79 
  $o .= "$+\n" while $s =~ /\G(.{0,78}?)\n|\G(.{1,79})$|\G(.{0,78})\s|\G(.{79}\n)|\G(.{79})/gs;
  $o =~ s/ +$//gm;
  escapeHTML $o;
}
              

# clean a string
sub clean($)
{
  my $s = shift; defined $s or $s = '';
  $s =~ s/^\s+|\s+$//g;
  $s =~ s/\s+/ /g;
  $s;
}

# get text block, clean ends of lines etc.
sub text($)
{
  my $s = shift; defined $s or $s = '';
  $s =~ s/\s+$//g;
  $s =~ s/[ \r\t]+$//mg;
  $s;
}
                         
# given string, clean into contact address or return ''
sub addr($)
{
  (my $addr = lc shift) =~ s/\s+//g;
  my $char = '-a-z0-9_+';
  ($addr=~/^[$char]+(?:\.[$char]+)*@[$char]+(?:\.[$char]+)+$/i)?$addr:''; 
}  
                         
# return url current page with given parameters
sub myurl(;@)
{
  Delete_all;
  CGI::param splice(@_,0,2) while @_ > 1;
  self_url;
}  

# return string as valid table data
sub td($)
{
  my $s = clean shift;
  return($s ne ''?escapeHTML($s):'&nbsp;');
}

# return link with specified text and parameters
sub href($;@)
{
  my $text=escapeHTML(shift);
  '<a href='.myurl(@_).">$text</a>";
}  

my $query;
# 'die' will show error to contact instead of barfing apache 
$SIG{__DIE__} = sub { html "Sorry, an error has occurred:<p><pre>".escapeHTML("@_")."</pre><br><pre>".escapeHTML($query)."</pre>"};

# same as above, for recoverable errors
sub err { html "Sorry, an error has occurred:<p><pre>@_</pre>",foot(href('Search')) };


# database handle
my $DB;


my %states = (
  seek=>sub
  {
    my @where; my @having;
    
    # create where string
    my $contact = clean param 'contact'; defined $contact or $contact = ''; 
    my $contact_op = (param 'contact_op') || 'starts with';
    my %contact_op = ('like' => "like ".$DB->quote($contact),
                      'not like' => "not like ".$DB->quote($contact),
                      'starts with' => "like ".$DB->quote("$contact\%"),
                      'is' => "=".$DB->quote($contact),
                      'contains'=>"like ".$DB->quote("\%$contact\%") );
    $contact and push @where,"contact ".$contact_op{$contact_op};
    
#    my $subject = clean param 'subject'; defined $subject or $subject = '';
#    my $subject_op = (param 'subject_op') || 'contains';
#    my %subject_op = ('like' => "like ".$DB->quote($subject),
#                      'not like' => "not like ".$DB->quote($subject),
#                      'starts with' => "like ".$DB->quote("$subject\%"),
#                      'contains'=>"like ".$DB->quote("\%$subject\%") );
#    $subject ne '' and push @where, "subject ".$subject_op{$subject_op};
#    
#    my $comment = clean param 'comment'; defined $comment or $comment = '';
#    my $comment_op = (param 'comment_op') || 'contains';
#    my %comment_op = ('like' => "like ".$DB->quote($comment),
#                      'not like' => "not like ".$DB->quote($comment),
#                      'starts with' => "like ".$DB->quote("$comment\%"),
#                      'contains'=>"like ".$DB->quote("\%$comment\%") );
#    $comment ne '' and push @where, "comment ".$comment_op{$comment_op};
#    
#    my $message = clean param('message'); defined $message or $message = '';
#    my $message_op = (param 'message_op') || 'contains';
#    my %message_op = ('like' => "like ".$DB->quote($message),
#                      'not like' => "not like ".$DB->quote($message),
#                      'starts with' => "like ".$DB->quote("$message\%"),
#                      'contains' => "like ".$DB->quote("\%$message\%"));
#    $message ne '' and push @where, "msg.body ".$message_op{$message_op};
    
    my $thread = clean param('thread'); defined $thread or $thread = '';
    my $thread_op = (param 'thread_op') || 'contains';
    my %thread_op = 
      ('like' =>     "(subject like ".$DB->quote($thread)." or comment like ".$DB->quote($thread)." or msg.body like ".$DB->quote($thread).")",
       'not like' => "(subject not like ".$DB->quote($thread)." and comment not like ".$DB->quote($thread)." and msg.body not like ".$DB->quote($thread).")",
       'contains' => "(subject like ".$DB->quote("\%$thread\%")." or comment like ".$DB->quote("\%$thread\%")." or msg.body like ".$DB->quote("\%$thread\%").")"
      );

    $thread ne '' and push @where, $thread_op{$thread_op};
    
    my %statii = ( All=>['holdoff < now()', undef],
                   Resolved=>['resolved=1 and holdoff < now()',undef],
                   Open=>['resolved=0 and holdoff < now()',undef],
                   New=>['resolved=0 and holdoff < now()','sum(isrecv=0)=0'],
                   Pending=>['resolved=0 and holdoff < now()','sum(isrecv=0)>0'],
                   Locked=>['holdoff >= now()',undef],
                   Urgent=>['urgent=1 and holdoff < now()',undef]);
    
    my $status = (param 'status') || 'Open';
    if ($statii{$status})
    {
      push @where, $statii{$status}->[0];
      $statii{$status}->[1] and push @having, $statii{$status}->[1];
    }  
  
    my %ranges = ( 'all'=>undef,
                   'changed today' =>'touched > curdate()',
                   'changed in last 1 month' =>'touched > DATE_SUB(CURDATE(), INTERVAL 1 MONTH)',    
                   'changed in last 3 months'=>'touched > DATE_SUB(CURDATE(), INTERVAL 3 MONTH)',    
                   'changed in last 6 months'=>'touched > DATE_SUB(CURDATE(), INTERVAL 6 MONTH)',    
                   'changed in last year'=>'touched > DATE_SUB(CURDATE(), INTERVAL 12 MONTH)');
    my $range = (param 'range') || 'all';
    $ranges{$range} and push @where, $ranges{$range};

                 
    my $list = (param 'list') || ''; # default 'By Thread'
       
    # build it
    my $html = '<table border=0 cellpadding=0 cellspace=0>
                <form method=get>
                <tr><td>
                <table border=1 cellpadding=2 cellspacing=0>';
    
    
    $html .= "<tr>
              <td class=head>Contact<td><select name=contact_op>".(join '',map opt($_,$contact_op eq $_), sort keys %contact_op)."</select>
                                    <td><input type=text name=contact size=60 value='".(escapeHTML $contact)."'>";
    
    
#    $html .= "<tr>
#              <td class=head>Subject<td><select name=subject_op>".(join '',map opt($_,$subject_op eq $_), sort keys %subject_op)."</select>
#                                        <input type=text name=subject size=60 value='".(escapeHTML $subject)."'>";

#    $html .= "<tr>
#              <td class=head>Comment<td><select name=comment_op>".(join '',map opt($_,$comment_op eq $_), sort keys %comment_op)."</select>
#                                        <input type=text name=comment size=60 value='".(escapeHTML $comment)."'>";

#    $html .= "<tr>
#              <td class=head>Message<td><select name=message_op>".(join '',map opt($_,$message_op eq $_), sort keys %message_op)."</select>
#                                        <input type=text name=message size=60 value='".(escapeHTML $message)."'>";
              
    $html .= "<tr>
              <td class=head>Thread<td><select name=thread_op>".(join '',map opt($_,$thread_op eq $_), sort keys %thread_op)."</select>
                                   <td><input type=text name=thread size=60 value='".(escapeHTML $thread)."'>";
    $html .= "<tr>
              <td class=head>Show<td><select name=status>".(join '',map opt($_, $_ eq $status), sort keys %statii). "</select>";
    
    $html .= "<td>";
    
    $html .= "<select name=list>".(join '',map opt($_, $_ eq $list), 'Threads, sorted by change', 
                                                                     'Threads, sorted by creation', 
                                                                     'Contacts'). "</select>";
    
    $html .= "<select name=range>".(join '',map opt($_, $_ eq $range), sort keys %ranges). "</select>";
    
    $html .= "</table>
              <td valign=bottom>
                <input type=submit value='Filter'>
                <input type=hidden name=state value=seek>
              </form>
              <td valign=bottom>
               <form><td valign=bottom><input type=submit value='Reset'></form>
              </table><p>";
    

    my $where = join ' and ', "thread.id = msg.thread_id", @where; $where and $where = "where $where";
    my $having = join ' and ', @having; $having and $having = "having $having";

    my $title;
    if ($list =~ /Contact/)
    {
      $query = "select contact, sum(isrecv=0), sum(resolved=0)=0, sum(holdoff>now())
                   from thread, msg $where group by contact $having order by contact";
                
      my @contacts = @{$DB->selectall_arrayref($query)};

      if (@contacts)
      {
        $html .= '<table border=1 cellpadding=2 cellspacing=0>';
        for (@contacts)
        {
          my ($contact, $sent, $resolved, $locked, $count)=@{$_};
          my $color = $locked?'darkred':$resolved?'lime':$sent?'yellow':'white';
          $html .= "<tr bgcolor=$color><td>".href($contact,state=>'seek',contact=>$contact,contact_op=>'is',status=>'All');
        }
        $html .= '</table><p><i>'.scalar(@contacts)." matching contacts</i>"
      } else {
        $html.= "</i>No matching contacts.</i>";
      }
    } else
    {
      my $order; my $col;
      if ($list =~ /creation/)
      {
        $order = 'thread.date';
        $col = 'Created';
      } else
      {
        $order = 'touched';
        $col='Changed';
      }  
      
      $query = "select thread.id, contact, date_format($order,'%a, %d %b %T'), subject, sum(isrecv=0), comment, resolved, urgent, holdoff>now() 
                from thread, msg $where group by thread.id $having order by $order";
                
      my @threads = @{$DB->selectall_arrayref($query)};

      if (@threads)
      {
        $html .= "<table border=1 cellpadding=2 cellspacing=0>
                  <tr><td class=head>$col<td class=head>Contact<td class=head>Subject<td class=head>Comment";
                  
        for (@threads)
        {
          my ($thread_id, $contact, $touched, $subject, $sent, $comment, $resolved, $urgent, $locked)=@{$_};
        
          my $color = $locked?'darkred':$urgent?'orangered':$resolved?'lime':$sent?'yellow':'white';
          $html .= "<tr bgcolor=$color>
                       <td class=stamp nowrap>".td($touched).
                      '<td>'.(($contact !~ /@/)?$contact:href($contact, state=>'seek', contact=>$contact, contact_op=>'is',status=>'All')).
                      '<td>'.href($subject, state=>'view', thread_id=>$thread_id).
                      '<td>'.td($comment);
        }
        $html .= '</table><p><i>'.scalar(@threads)." matching threads";
        $contact && $contact_op eq 'is' && $thread eq '' && $status eq 'Open' and $html .= " - ".href('resolve all', state=>'resolveall', contact=>$contact);
        $html .="</i>";
      } else {
        $html.= "<i>No matching threads.</i>";
      }
    }            

    $HEAD.='<meta http-equiv="refresh" content="60">';
    html $html, foot href('Call Log',state=>'addlog'),          
                    href('Send Mail',state=>'sendmail')
                    #,"<hr><span class=tiny>",escapeHTML($query),"</span>"
                    ;
  },
  
  view=>sub 
  {
    my $thread_id = need 'thread_id'; 
    my $expand = param 'expand';
    
    
    my ($contact, $subject, $cclist, $resolved, $date, $locked, $touched, $sent, $comment, $urgent)
    = $DB->selectrow_array('select contact, subject, cclist, resolved, date_format(thread.date,"%a, %d %b %Y %T"), holdoff>now(),
                                                                       date_format(touched,"%a, %d %b %Y %T"), sum(isrecv=0), comment, urgent
                               from thread, msg 
                               where thread.id = ? and thread_id = thread.id group by thread.id', undef, $thread_id);
                                                              
    defined $contact or err "Broken thread $thread_id\n";

    my $html = "<span class=subtitle>View Thread</span><p>
                <table bgcolor=".($locked?'darkred':$urgent?'orangered':$resolved?'lime':$sent?'yellow':'white').">
                  <tr><td class=head>Contact:<td>".href($contact, state=>'seek', contact=>$contact, contact_op=>'is', status=>'All').
                 '<tr><td class=head>Subject:<td>'.td($subject).
                 '<tr><td class=head>Comment:<td>'.td($comment);
      $cclist and 
        $html .= '<tr><td class=head>Cc:<td>'.td($cclist);
        $html .= "<tr><td class=head>Created:<td>$date";
      $date ne $touched and
        $html .= "<tr><td class=head>Changed:<td>$touched"; 
        $html .= '<tr><td class=head>Status:<td>'.($locked?'Locked':$resolved?'Resolved':$sent?'Pending':'New').
                '</table></table><p>';

    
    my $clipped = 0;
    for(@{$DB->selectall_arrayref('select id, date_format(date,"%a, %d %b %Y %T"), body, isrecv, issent from msg where thread_id = ? order by date',undef,$thread_id)})
    {                            
      my ($msg_id, $date, $body, $isrecv, $issent) = @{$_};
      
      $body = wrap($body);
     
      if ($expand ne 'all')
      {
        if ($expand == $msg_id) 
        { 
            $body .= "\n\n".href('Unexpand', state=>'view', thread_id=>$thread_id);
        } else 
        {
            my $doexpand = href('Expand',state=>'view', expand=>$msg_id, thread_id=>$thread_id);
            $body =~ s/^((?:.*?\n){20})(?:.*?\n){2}.+/$1\n$doexpand/s and $clipped=1;
        }
      }  
        
      $html .= '<hr><div'.($isrecv?'>Received':' style="background-color:yellow">'.($issent?'Sent':'Note added'));
      $html .=" $date:<p><pre>$body</pre></div>";
    }  

    my @f;
    if ($clipped) { push @f, href('Expand All', state=>'view', expand=>'all', thread_id=>$thread_id); }
    elsif ($expand) { push @f, href('Unexpand All', state=>'view', thread_id=>$thread_id); } 
    
    push @f, href('Search');
    unless ($locked)
    {
      push @f,'|',href('Thread Properties', state=>'properties', thread_id=>$thread_id),
                  href('Add Note', state=>'addnote', thread_id=>$thread_id);
               
      addr $contact and push @f, href('Send Reply', state=>'sendreply', thread_id=>$thread_id); 
      push @f, href('Mark as '.($resolved?'Open':'Resolved'), state=>'resolve', resolve=>!$resolved, thread_id=>$thread_id);
    }
    
    html $html, foot @f;            
  },

  properties=>sub
  {
    my $thread_id = need 'thread_id'; 
    my $really = nparam 'really';

    $DB->do('lock tables thread write');
    my ($locked, $contact, $subject, $cclist, $new_holdoff, $resolved, $current_holdoff, $comment, $urgent) 
      = $DB->selectrow_array("select holdoff>now(), contact, subject, cclist, now()+interval 5 minute, resolved, holdoff, comment, urgent
                              from thread
                              where thread.id = ?", undef, $thread_id);

    defined $contact or err "Broken thread $thread_id ";
    
    unless ($really)
    {
      $locked and err "Thread is locked, try again later.\n";                             
      
       
      $DB->do('update thread set holdoff=? where id=?',undef,$new_holdoff,$thread_id);

      $HEAD = '';
      html "<span class=subtitle>Thread Properties</span><p>
            <form method=post>
            Thread will be locked for 5 minutes while you modify the thread settings. Click ",
            href('here', state=>'unlock', new_holdoff=>$new_holdoff, thread_id=>$thread_id)," to unlock and go back,
            <i>do not click BACK or REFRESH!</i><p> 
                     
            Edit thread parameters and click OK. <b>YOU</b> are responsible for making sure email addresses are valid!<p>
                     
            <table>
              <tr><td class=head>Contact:<td><input type=text size=80 name=contact value='".td($contact)."'>
              <tr><td class=head>Subject:<td><input type=text size=80 name=subject value='".td($subject)."'>
              <tr><td class=head>Cc:<td><input type=text size=80 name=cc value='".td($cclist)."'>
              <tr><td class=head>Comment:<td><input type=text size=80 name=comment value='".td($comment)."'>
            </table> 
            <input type=submit value=OK>
            <input type=hidden name=state value=properties>
            <input type=hidden name=really value=1>
            <input type=hidden name=thread_id value=$thread_id>
            <input type=hidden name=new_holdoff value='$new_holdoff'>
            </form>";
    }
    
    $new_holdoff = need 'new_holdoff';
    $new_holdoff eq $current_holdoff or die "Thread lock has expired.\n";
    
    my $newcontact = addr need 'contact';
    unless ($newcontact)
    {
      $contact eq 'nobody' or die "Invalid contact address.\n";
      $newcontact = 'nobody';
    }  
    
    $cclist = clean param 'cc';
    $subject = clean need 'subject';
    $comment = clean param 'comment';
     
    #$DB->do('update thread set touched=now(), holdoff=0, contact=?, subject=?, cclist=?, comment=? where id=?',
    $DB->do('update thread set holdoff=0, contact=?, subject=?, cclist=?, comment=? where id=?',
             undef, $newcontact, $subject, $cclist, $comment, $thread_id);
    $DB->do('unlock tables');
    (state=>'view', thread_id=>$thread_id);
  },  
    
  delete=>sub
  {
    my $thread_id = need 'thread_id';
    $DB->do("lock tables thread write, msg write");
    $DB->selectrow_array('select holdoff<now() from thread where id=?', undef, $thread_id) or err "Thread is locked, try again later.\n";
    
    my $really = nparam 'really';
    
    $really or html 'Permanently delete this thread?', 
                    foot(href('No', state=>'view', thread_id=>$thread_id), 
                         href('Yes',state=>'delete',thread_id=>$thread_id,really=>1));
   
    $DB->do('delete from msg where thread_id=?', undef, $thread_id);
    $DB->do('delete from thread where id=?', undef, $thread_id);
    $DB->do("unlock tables");
    # back to the top 
  },

  resolve=>sub
  {
    my $thread_id = need 'thread_id';
    $DB->do("lock tables thread write");
    $DB->selectrow_array('select holdoff<now() from thread where id=?', undef, $thread_id) or err "Thread is locked, try again later.\n";
    
    my $resolve= (param 'resolve')?1:0;
    
    $DB->do('update thread set touched=now(), resolved=?, urgent=0 where id=?',undef,$resolve,$thread_id);
    $DB->do("unlock tables");
    (state=>'view', thread_id=>$thread_id);
  }, 

  resolveall=>sub
  {
    my $contact = need 'contact';
    $DB->do('update thread set touched=now(), resolved=1, urgent=0 where holdoff < now() and resolved=0 and contact=?', undef, $contact);
    (state=>'seek', contact=>$contact, contact_op=>'is', status=>'Changed Today');
  }, 

  sendreply=>sub
  {
    my $thread_id = need 'thread_id'; my $really = nparam 'really';
    
    $DB->do("lock tables thread write, msg write, sigs read");
    my ($locked, $contact, $subject, $cclist, $current_holdoff, $new_holdoff, $resolved, $now)
      = $DB->selectrow_array("select holdoff>now(), contact, subject, cclist, holdoff, now()+interval 30 minute, resolved, unix_timestamp(now())
                              from thread where id = ?", undef, $thread_id);
           
    defined $contact or err "Broken thread $thread_id ";                                            

    addr $contact or die "Invalid email '$contact'\n";
    
    # fix subject
    $subject =~ s/^\s*(?:re\s*:\s*)?(.*)/RE: $1/i;

    unless ($really)
    {
      $locked and err "Thread is locked, try again later.\n"; 
       
      $DB->do('update thread set holdoff=? where id=?',undef,$new_holdoff,$thread_id);

      (my $orig = $DB->selectrow_array('select body from msg where thread_id=? and (issent = 1 or isrecv = 1) order by date desc limit 1', undef, $thread_id)) 
        =~ s/\s+$//;
      
      if ($orig)
      { 
        $orig = wrap($orig);
        $orig =~ s/^/&gt;&nbsp;/gm;
        $orig = "<hr><br>Original message:<pre>\n$orig<pre>"
      } else {$orig = ''};  
      my $sig = "\n\n\n".wrap($DB->selectrow_array('select body from sigs order by rand() limit 1'));
  
      $HEAD = '';
      html "<form method=post>
            Thread will be locked for 30 minutes while you enter the reply.
            Click ", href('HERE', state=>'unlock', new_holdoff=>$new_holdoff,
            thread_id=>$thread_id)," to unlock and go back, <i>do not click
            BACK or REFRESH!</i><p>
            
            <table><tr><td>From:<td>".td($SUPPORTS)."
                   <tr><td>To:<td>",td($contact).
           ($cclist =~ /\S/ ?
                  '<tr><td>Cc:<td>'.td($cclist):'').
                  '<tr><td>Subject:<td>',td($subject),
           "</table><textarea name=message wrap=soft rows=20 cols=78>$sig</textarea><br>",
            ($resolved?'':'<input type=checkbox name=resolved> Check if this message resolves the thread.<br>'),
           "<input type=submit value='Send'>
            <input type=hidden name=thread_id value='$thread_id'>
            <input type=hidden name=really value=1>
            <input type=hidden name=new_holdoff value='$new_holdoff'>
            <input type=hidden name=state value=sendreply>
            </form>
            $orig",
            foot q{<a href='faq.cgi' 
                      onclick="window.open('faq.cgi','mywindow','width=400, height=200, toolbar=yes, scrollbars=yes, copyhistory=no, resizable=yes'); return false"
                      target='_blank'>Popup FAQs</a>}
    }

    # post reply
    $new_holdoff = need 'new_holdoff';
    $current_holdoff eq $new_holdoff or err "Thread lock has expired.\n";
    my $body = text need 'message'; 
    $resolved = (param 'resolved')?1:0;
    my $date = strftime('%a, %d %b %Y %H:%M:%S %z',localtime $now);

    my %headers = (Date=>$date, To=>$contact, Subject=>$subject);
    $cclist =~ /\S/ and $headers{Cc}=$cclist;
    mail \%headers, $body;
     
    
    # also log it
    my $mail = "Date    : $date\n".
               "From    : $SUPPORTS\n". 
               "To      : $contact\n";
    $cclist =~ /\S/ and $mail .=
               "Cc      : $cclist\n";
    $mail .=   "Subject : $subject\n\n$body";           
                
    $DB->do('insert msg set thread_id=?, date=from_unixtime(?), body=?, issent=1', undef, $thread_id, $now, $mail);
    $DB->do('update thread set touched=now(), holdoff=0, resolved=(resolved|?), urgent=0 where id=?',undef,$resolved,$thread_id);
    $DB->do('unlock tables');
     
    (state=>'view', thread_id=>$thread_id);
  },              

  addnote => sub
  {
    my $thread_id=need 'thread_id'; 
    my $really = nparam 'really';

    $DB->do('lock tables thread write, msg write');
    
    my ($contact, $subject, $locked, $new_holdoff, $resolved, $current_holdoff, $now) 
      = $DB->selectrow_array("select contact, subject, holdoff>now(), now()+interval 30 minute, resolved, holdoff, unix_timestamp(now()) 
                              from thread where id = ?", undef, $thread_id);
    defined $contact or err "Broken thread $thread_id";
    
    unless ($really)
    {
      $locked and err "Thread is locked, try again later.\n"; 
     
      $DB->do('update thread set holdoff=? where id=?',undef,$new_holdoff, $thread_id) != 0 or err "Broken thread $thread_id";
    
      $HEAD = '';
      html "<table>
              <tr><td>Contact:<td>".td($contact).
             '<tr><td>Subject:<td>'.td($subject).
           "</table>
            <form method=post>
              Thread will be locked for 30 minutes while you edit the note.
              Click ",href("here", state=>'unlock', new_holdoff=>$new_holdoff, thread_id=>$thread_id)," to
              unlock the thread and go back, <i>do not click BACK or REFRESH!</i><p>
              <textarea name=message wrap=soft rows=20 cols=78></textarea><br>", 
              ($resolved?'':'<input type=checkbox name=resolved> Check if this note resolves the thread.<br>'),
             "<input type=submit value='OK'>
              <input type=hidden name=thread_id value=$thread_id>
              <input type=hidden name=really value=1>
              <input type=hidden name=new_holdoff value='$new_holdoff'>
              <input type=hidden name=state value=addnote>
            </form>";
    }
    # update note
    $new_holdoff = need 'new_holdoff';
    $new_holdoff eq $current_holdoff or err "Thread lock has expired.\n";
    $resolved = (param 'resolved')?1:0;
    my $body = text need 'message'; 
    
    $DB->do('update thread set holdoff=0, touched=from_unixtime(?), resolved=(resolved or ?), urgent=0 where id=?',undef,$now,$resolved,$thread_id) != 0
      or err "Broken thread $thread_id";

    $DB->do('insert msg set thread_id=?, date=from_unixtime(?), body=?',undef, $thread_id, $now, $body);
    $DB->do('unlock tables');
    (state=>'view', thread_id=>$thread_id);
  },
  
  unlock =>sub 
  {  
    my $thread_id=need 'thread_id';
    my $new_holdoff=need 'new_holdoff';
    $DB->do('update thread set holdoff = 0 where id=? and holdoff=?', undef, $thread_id, $new_holdoff) != 0 or err "Thread lock has expired.\n";  
    (state=>'view', thread_id=>$thread_id);
  },  
    
  'addlog' => sub
  {
    my $contact=clean param 'contact'; my $really = nparam 'really';
                                                                                                     
    $really or 
    html "<span class=subtitle>Call Log</span><p>
          <form method=post>
           
          Enter log contact, subject and message, then click OK. This will
          create a new thread. You may also specify a comment.<p>
          
          <table><tr><td>Contact:<td><input type=text size=80 name=contact value='".escapeHTML(addr($contact)||'nobody')."'>
                 <tr><td>Subject:<td><input type=text size=80 name=subject value='Call Log'>
                 <tr><td>Comment:<td><input type=text size=80 name=comment>
          </table>
          <textarea name=message wrap=soft rows=20 cols=78></textarea><br>
          <input type=checkbox name=resolve> Check if no further action is required.<br>
          <input type=submit value='OK'>
          <input type=hidden name=really value=1>
          <input type=hidden name=state value=addlog>
          </form>";

     # update log
     $contact eq 'nobody' or $contact = addr $contact or die "Invalid contact address\n"; 
     my $subject = (clean param 'subject')||'no subject';
     my $comment = clean param 'comment';
     my $body = text need 'message';
     my $resolve = (param 'resolve')?1:0;
     my $now = $DB->selectrow_array("select unix_timestamp()");
     $DB->do('insert thread set contact=?, date=from_unixtime(?), touched=from_unixtime(?), subject=?, resolved=?, comment=?, urgent=0',
              undef,$contact,$now,$now,$subject,$resolve,$comment);
     my $thread_id = $DB->selectrow_array('select LAST_INSERT_ID()');          
     $DB->do('insert msg set thread_id=?, date=from_unixtime(?), body=?',undef,$thread_id,$now,$body);
     (state=>'seek', contact=>$contact, contact_op=>'is', status=>'Changed Today');
  },              

  'sendmail' => sub
  {
    my $contact=addr param 'contact'; my $really = nparam 'really';
    
    $really or 
    
    html "<span class=subtitle>Send Mail</span><p>
           <form method=post>
           Enter contact, subject, and messages, then click OK. This will
           create a new thread. You may also specify a comment, and a cc list
           for this and all future replies in this thread.<p> 
           
           <table><tr><td>From:<td>".td($SUPPORTS)."
                  <tr><td>To:<td><input type=text name=contact size=80 value='".escapeHTML($contact)."'>
                  <tr><td>Cc:<td><input type=text name=cclist size=80>
                  <tr><td>Subject:<td><input type=text name=subject size=80>
                  <tr><td>Comment:<td><input type=text size=80 name=comment>
           </table>
           <textarea name=message wrap=soft rows=20 cols=78>",
            "\n\n".wrap($DB->selectrow_array('select body from sigs order by rand() limit 1')),
          "</textarea><br>
           <input type=checkbox name=resolve> Check if no further action is required.<br>
           <input type=submit value='Send'>
           <input type=hidden name=really value=1>
           <input type=hidden name=state value=sendmail>
           </form>";

     # send the mail
     $contact or die "Invalid contact address.\n";
     my $subject = clean need 'subject';
     my $cclist = clean param 'cclist';
     my $comment = clean param 'comment';
     my $body = text need 'message';
     my $resolve = (param 'resolve')?1:0;
    
     my $now = $DB->selectrow_array("select unix_timestamp()");
     my $date = strftime('%a, %d %b %Y %H:%M:%S %z',localtime $now);
    
     my %headers = (Date=>$date, To=>$contact, Subject=>$subject); $cclist and $headers{Cc}=$cclist;
     mail \%headers, $body;
    
     # also log it
     my $mail = "Date    : $date\n".
                "From    : $SUPPORTS\n". 
                "To      : $contact\n";
     $cclist and $mail .=
                "Cc      : $cclist\n";
     $mail .=   "Subject : $subject\n\n$body";           
   
     $DB->do('lock tables thread write, msg write');
     $DB->do('insert thread set contact=?, date=from_unixtime(?), touched=from_unixtime(?), cclist=?, subject=?, resolved=?, comment=?, urgent=0',
                 undef,$contact,$now,$now,$cclist,$subject,$resolve,$comment);
     my $thread_id = $DB->selectrow_array('select LAST_INSERT_ID()');         
     
     $DB->do('insert msg set thread_id=?, date=from_unixtime(?), body=?, issent=1',undef,$thread_id,$now,$mail);
     $DB->do('unlock tables');
     
     (state=>'seek', contact=>$contact, contact_op=>'is', status=>'Changed Today');
  },              
  
  # administrative stuff

  'admin' => sub
  {
    html join '',
         "Sigs (one of these randomly selected for reply)
         <table border=1 rules=rows>",map("<tr><td><pre>".wrap($_->[1])."</pre>
                                <td>".href('Edit',state=>'editsig',id=>$_->[0]).
                               '<td>'.href('Delete',state=>'delsig',id=>$_->[0]),
                            @{$DB->selectall_arrayref('select id, body from sigs')}),'</table>',
         href('Add SIG',state=>'editsig');
  },

  editsig => sub
  {
    my $id=nparam 'id'; my $really = nparam 'really';

    unless ($really)
    {
      my $body='';
      $id and $body=$DB->selectrow_array('select body from sigs where id = ?',undef,$id);
      defined $body or err "Invalid sig $id\n";
      html "<form method=post>
            Edit sig and click ok.<p> 
            <textarea name=body wrap=soft rows=20 cols=75>".escapeHTML($body)."</textarea>
            <input type=submit value='OK'>
            <input type=hidden name=really value=1>
            <input type=hidden name=id value=$id>
            <input type=hidden name=state value=editsig>
            </form>";
    }       

    my $body = text need 'body';
    $DB->do('lock tables sigs write');
    $id and $DB->do('delete from sigs where id=?',undef,$id);
    $DB->do('insert sigs set body=?',undef, $body);
    $DB->do('unlock tables');
    (state=>'admin');
  },             
   
  delsig => sub
  {
    my $id=need 'id'; my $really = nparam 'really';
    $really or html 'Really delete sig?', foot(href('No',state=>'admin'),href('Yes',state=>'delsig', really=>1, id=>$id)); 
    $DB->do('delete from sigs where id=?',undef,$id);
    (state=>'admin');
  },  
  
); # end of state table

my $state = param 'state';
my %next=(state=>'seek');
if (defined $state)
{
  $states{$state} or die "invalid state $state\n";
  
  for (reverse 0..10) # try 10 times
  {
    $DB = DBI->connect('DBI:mysql:support:mbdb','support',undef,{RaiseError => 1, AutoCommit=>1}) and last;
    $_ or die "can't connect to database\n";
    sleep 1; 
  }
  
  %next=$states{$state}->();
}


# on exit or if no state, refresh to seek state 
# back should go HERE dammit, but it doesn't...
$HEAD .= "<meta http-equiv='refresh' content='0;url=".myurl(%next)."'>\n";
html 'Please wait...';

